/*	road.h
 *
 *	Author: Enrico Malavasi
 *      Ver. 2.10
 *      Last revised:  August 1990
 *
 * 	Purpose: Fundamental definition of data structures.
 */
#ifndef  _ROAD_
#define  _ROAD_    
#define    ROAD_VERSION    "3.20"
#define    ROAD_HEAD       "RoAD-3.2> "

/*  ------------------------------------------------------------
 *  Definition of the direction constants
 */
#define NORTH        1
#define UP           2
#define EAST         3
#define SOUTH        (-1)
#define DOWN         (-2)
#define WEST	     (-3)
#define NODIRECTION  0

#define   FREE_NORTH     0x01
#define   FREE_UP        0x02
#define   FREE_EAST      0x04
#define   FREE_WEST      0x08
#define   FREE_DOWN      0x10
#define   FREE_SOUTH     0x20

typedef double       pFarad,
                     fFarad;
typedef float        Ohm,
                     RoadWeight,
                     micron;
typedef int          RoadCoord,
                     RoadCost,
                     RoadStatus;
typedef char         PathName[128];

/*  ------------------------------------------------------------
 *  Structure <out_pin>
 *  It implements pins on the border of sub-cells.
 */
typedef struct RoadTerm {
    int           cellNo;                /* Sub-cell                */
    int           equivNo;               /* Equivalent pin          */
    int           netNo;
    RoadCoord     position;              /* Position on side        */
    RoadCost      exitCost;
    char          layer;		       /* Layer                   */
    char          side; 	  	       /* Side of sub-cell        */
    unsigned char type;                  /* Term type               */
    struct rectangle *rect;              /* Rectangle ............. */
    char         *TermName;              /* Terminal name ......... */
} RoadTerm, *Pins;
#define  TERM_FIXED_TYPE    0
#define  TERM_FLOAT_TYPE    1
#define  TERM_DUMMY_TYPE    2
#define  TERM_SHIELD_TYPE   4

typedef struct RoadInstance {
    char      *name;
    char      *master;
    char      *view;
    RoadCoord  x;                     /* Instance position       */
    RoadCoord  y;                  
    int        transf;                /* Instance transformation */
    int        noov;
} RoadInstance;

/*  ------------------------------------------------------------
 *  Structure <sub_cell>
 *  It implements the instantiations of	sub-cells.
 */
typedef struct RoadCell {
    RoadCoord  x1, y1;
    RoadCoord  x2, y2;
    char       layer;
    int        instanceNo;
} Cell;

/*  ------------------------------------------------------------
 *  Structure <rectangle>
 *  It implements the virtual grid elements.
 */
typedef struct rectangle {
    RoadCoord  x1, y1;
    RoadCoord  x2, y2;
/*  -----------------------------------------------------------
 *	adjacent:	0(=1-1) = North     5(=-1+6) = South  |
 *			1(=2-1) = Up        4(=-2+6) = Down   |
 *			2(=3-1) = East      3(=-3+6) = West   |
 *  -----------------------------------------------------------
 */
    struct rectangle **adjacent;   /* No Up Ea We Do So        */
    RoadCost   cost[3];	           /* Resident predefined cost */
    char       netNo;
    char       layer;
    char       free;
    unsigned char flag;
}  **PATH, *Rect, rectangle;
#define     RECTFLAG_TERM    0x0001

/*  ------------------------------------------------------------
 *  Distributed terminals
 */
typedef struct DTERM {
    PATH       rect;                 /* Array of front rectangles ... */
    RoadTerm  *pin;                  /* Father-pin index ............ */
} DTERM;

typedef struct _rdWire {
    Rect     *rect;
    int      *side;
    int       size;
    int       netNo;
    char      free;
    char      wDesc;    
    RoadCoord *wWidth;
} _rdWire;
#define    MINWWIDTH  0
#define    DEFWWIDTH  1
#define    VARWWIDTH  2

typedef struct RoadBox {
  RoadCoord   x1, y1;
  RoadCoord   x2, y2;
  int         z;
} RoadBox;
typedef struct newTerm {
  int           equivNo;               /* Equivalent pin          */
  int           netNo;
  RoadBox       box;                   /* Layer indication is here    */
  char         *TermName;              /* Terminal name ......... */
} newTerm;
typedef struct roadObject {
  int  type;
  union {
    rectangle  rect;
    newTerm    term;
    Cell       cell;
  } object;
} roadObject;
typedef  int  roadType;
#define    ROAD_RECT    1
#define    ROAD_TERM    2
#define    ROAD_CELL    3

/*  ------------------------------------------------------------
 *  LAYER Structure. It is implemented by the <layers> array
 */
typedef struct  RoadLayer {
    Ohm         square_res;             /* Square-resistance   (in Ohm) */
    Ohm         via_resistance;         /* Equivalent via res. (in Ohm) */
    pFarad      unity_cap;              /* Unity capacitance   (in pF/um) */
    pFarad      via_capacitance;        /* Equivalent via cap. (in pF) */
    micron      bulk_dist;              /* Distance to bulk    (in um) */
    micron      wire_thickness;         /* Wire height         (in um) */
    RoadCoord   wire_width;             /* Min. wire width     (in rd_coord)  */
    RoadCoord   via_width;              /* Min. via area width (in rd_coord) */
    RoadCoord   wire_dist;              /* Min. wire distance  (in rd_coord) */
    RoadWeight  cost;                   /* Layer cost .......... */
    RoadWeight  via_cost;               /* Upward via cost ..... */
    int         preferDir;              /* Preferred direction */
    int         type;                   /* see technol.h  */
    char       *name;
    char      **masking_layers;         /* masking layers  */
} RoadLayer;

/*  ------------------------------------------------------------
 *  RoAD Net  structure. Implemented by the <rd_net> array.
 */
typedef struct RoadNet	{
    RoadWeight  resistive_weight;	      /* Resistive weight                */
    RoadWeight  capacitive_weight;      /* Capacitive weight               */
    int         bulk_shld;              /* Optional: shield electric net   */
    RoadCoord   wwidth;                 /* Wire width */
    char       *name;	              /* Net name                        */
    char        link;                   /* Partitioned net                 */
} RoadNet;

#include "functions.h"
#endif
