/*	roadGlobals.h
 *
 *	Author: Enrico Malavasi
 *      Ver. 2.10
 * 	Last revised: August 1990
 *
 *	Purpose: Global and extern variables definition.
 */
#ifndef  _ROADGLOBALS_
#define  _ROADGLOBALS_  
#include "port.h"
#include "maxmax.h"
#include "road.h"
#include "roadMacros.h"

/*  ------------------------------------------------------------
 *  Status structure. It is used to describe the system status.
 */
typedef struct STATUS {
  int           LayerNum;        /* Number of available layers .......   */
  int           RectNum;         /* Number of rectangles .............   */
  int           CellNum;         /* Number of sub-cells ..............   */
  int           InstNum;         /* Number of instances ..............   */
  int           WireNum;         /* Number of routed wires ...........   */
  int           NetNum;          /* Number of electric nodes .........   */
  int           PinsNum;         /* Number of pins ...................   */
  int           BulkNet;         /* Net connected to bulk ............   */
  unsigned int  General;
} STATUS;
#ifdef FIRSTDEF
STATUS status  = { 0, 0, 0, 0, 0, 0, 0, -1, 0 };
#else
extern STATUS  status;
#endif

#define  TOTAL_RECT_NUM  status.RectNum
#define  SUB_CELL_NUM    status.CellNum
#define  INST_NUM        status.InstNum
#define  WIRE_NUM        status.WireNum
#define  NET_NUM         status.NetNum
#define  PIN_NUM         status.PinsNum
#define  LAYER_NUM       status.LayerNum
/*  ------------------------------------------------------------
 *  status.General macros.
 */
#define  GEN_SYMM              0x0001         /* Symmetry enabled */
#define  NET_READ              (0x0001<<1)    /* Net-file enabled */
#define  BATCH_MODE            (0x0001<<2)    /* Batch mode */
#define  RD_VERBOSE_FLAG       (0x0001<<3)    /* Verbose    */
#define  GRAPH_VERB            (0x0001<<4)    /* Verbose    */
#define  SMART_H               (0x0001<<6)    /* Smart H-estimate in propagation */
#define  DEBUG_OPTION          (0x0001<<7)
#define  COMMON_FLAG           (0x0001<<8) 

#define RD_CHECK_FLAG(f)    ( status.General & f )
#define RD_SET_FLAG(f)      { status.General |= f;    }
#define RD_RESET_FLAG(f)    { status.General &= (~f); }
#define RD_RESET_ALLFLAGS     status.General = 0

#define INTERACTIVE_MODE (! RD_CHECK_FLAG(BATCH_MODE))
#define VERBOSE(fct)     (RD_CHECK_FLAG(RD_VERBOSE_FLAG)) ? fct : 0

#define    CRIT_COUPLING_MASK    0x0001
#define    CRIT_MAXCAP_MASK      (0x0001<<1)
#define    CRIT_SHIELD_MASK      (0x0001<<2)
#define    CRIT_ALL_MASK       CRIT_COUPLING_MASK | CRIT_MAXCAP_MASK | CRIT_SHIELD_MASK

#ifdef  FIRSTDEF
Rect              rects = NULL;      /* Rectangles in the routable area  */
Cell             *cells = NULL;      /* Sub-cells ...................... */
RoadInstance     *insts = NULL;      /* Loaded instances ............ */
RoadTerm          *pins = NULL;      /* Terminals....................... */
_rdWire         *rdWire = NULL;
RoadLayer       *layers = NULL;      /* Layers array ................. */
RoadNet         *rd_net = NULL;      /* Electric nodes array ........ */
int           RECT_ALLOC;     /* Number of allocated rectangles */
float       preferWeight;     /*  Preferential direction weight  */
RoadCoord   SymmetryAxis;     /*  Symm axis position */

/*  ------------------------------------------------------------
 *  Names
 */
int   DumpBlock = 0;

#else

extern Rect              rects;   /* Rectangles in the routable area */
extern Cell             *cells;   /* Sub-cells - Obstacles ......... */
extern RoadInstance     *insts;   /* Loaded instances ............ */
extern RoadTerm          *pins;   /* Pins .......................... */
extern _rdWire          *rdWire;
extern RoadLayer       *layers;   /* Layers array ................ */
extern RoadNet         *rd_net;   /* Electric nodes array ........ */
extern int          RECT_ALLOC;   /* Allocated rectangles .......... */
extern float      preferWeight;   /*  Preferential direction weight  */
extern RoadCoord  SymmetryAxis;

#endif
#endif
