/*      roadMacros.h
 *
 *	Author: Enrico Malavasi, Univ. of Bologna
 *      Ver. 3.20    -    August 1991
 *
 *	Purpose: General utility macros file.
 */
#ifndef  _ROADMACROS_
#define  _ROADMACROS_
#include  "elmacro.h"

#define BEEP      printf("%c",7)
/*  ============================================================
 *  Utility macros applied to ROAD structures ...
 */
#define  rdLayer(X)  ((X)->layer)
#define  rdRightX(X) ((X)->x2)
#define  rdLeftX(X)  ((X)->x1)
#define  rdUpperY(X) ((X)->y2)
#define  rdLowerY(X) ((X)->y1)
#define  rdNetNo(X)    ((X)->netNo)
#define  rdWidth(X)  (rdRightX(X) - rdLeftX(X))
#define  rdHeight(X) (rdUpperY(X) - rdLowerY(X))

/*  ============================================================
 *  Macros for rects
 */
#define rRightX(r)   rdRightX(r)
#define rLeftX(r)    rdLeftX(r)
#define rUpperY(r)   rdUpperY(r)
#define rLowerY(r)   rdLowerY(r)
#define rwidth(r)    rdWidth(r)
#define rheight(r)   rdHeight(r)
#define rLayer(r)    rdLayer(r)
#define rNet(r)      rdNetNo(r)
#define neighbor(r,dir)	((r)->adjacent[(dir) + (((dir)>0)?(-1):6)])

#define  RD_SETRECTFLAG(r,F)       (r)->flag |= (F)
#define  RD_RESETRECTFLAG(r,F)     (r)->flag &= (~(F))
#define  RD_CHECKRECTFLAG(r,F)     ((r)->flag & (F))
/*  ============================================================
 *  Macros for cells
 */
#define cRightX(c)   rdRightX(c)
#define cLeftX(c)    rdLeftX(c)
#define cUpperY(c)   rdUpperY(c)
#define cLowerY(c)   rdLowerY(c)
#define cwidth(c)    rdWidth(c)
#define cheight(c)   rdHeight(c)
#define cLayer(c)    rdLayer(c)
/*  ============================================================
 *  Macros for wires
 */
#define wireNetNo(wn)        rdWire[wn].netNo
#define wireSize(wn)         rdWire[wn].size
#define wireFree(wn)         rdWire[wn].free
#define wireDesc(wn)         rdWire[wn].wDesc
#define wSegmWidth(wn,pos)   rdWire[wn].wWidth[pos]
#define wSegmSide(wn,pos)    rdWire[wn].side[pos]
#define wSegmRect(wn,pos)    rdWire[wn].rect[pos]
#define wSegmLayer(wn,pos)   rLayer(wSegmRect(wn,pos))
#define wSegmHere(wn,r,side)                            \
                (((r) != NULL)                       && \
                 (free_neighbor((r),(side)) == NULL) && \
  	         (neighbor((r),(side)) != NULL)      && \
	         (rNet(r) == wireNetNo(wn)))
/*  */
#define are_equivalent(p1,p2)    (((p1) != NULL) &&\
				  ((p2) != NULL) &&\
				  ((p1)->equivNo == (p2)->equivNo) &&\
				  ((p1) != (p2)))
#define freeSide(r,pattrn)    ((r)->free & (pattrn))
#define ViaUp(r)              ((r)->free & 0x02)
#define ViaDown(r)            ((r)->free & 0x10)
#define setFreeSide(r,pattrn)    (r)->free |= (pattrn);
#define resetFreeSide(r,pattrn)  (r)->free &= (~ (pattrn));
#define resetFreeAllSides(r)     (r)->free = 0;

/*  ============================================================
 *  Map a distance on a given layer to a cost.
 */
#define  DistToCost(x, l)   ((x)*layers[l].cost)

/*  ============================================================
 *  Direction inquiring macros ...
 */
#define north_or_south(dir)     (((dir) == NORTH) || ((dir) == SOUTH))
#define east_or_west(dir)       (((dir) == EAST)  || ((dir) == WEST))
#define up_or_down(dir)         (((dir) == UP)    || ((dir) == DOWN))

/*  ============================================================
 *  Loops ...
 */
#define four_sides(dir)       for(dir=WEST ; dir<=EAST ; dir+=2)
#define six_sides(x)          for(x=WEST ; x<=EAST ; x+= ((x == (-1))? 2 :1))
#define ForAllInsts(i,f)      for(i=f ; i<status.InstNum ; i++)
#define ForAllPins(i)         for(i=0 ; i<status.PinsNum ; i++)
#define ForAllRects(i)        for(i=0 ; i<status.RectNum ; i++)
#define ForAllWires(i)        for(i=0 ; i<status.WireNum ; i++)
#define ForAllLayers(i)       for(i=0 ; i<status.LayerNum ; i++)
#define ForAllCells(i,f)      for(i=f ; i<=status.CellNum ; i++)
#define ForAllNets(i)         for(i=0 ; i<status.NetNum ; i++)
#define NOTFOUND  (-1)

/*  ------------------------------------------------------------
 *  Nets and nodes
 */
#define  NullNetName(n)  ((rd_net[n].name==NULL)||(rd_net[n].name[0]=='\0'))
#define  isSupply(x)     ((strcmp(rd_net[x].name, "VSS") == 0) || \
                          (strcmp(rd_net[x].name, "VDD") == 0))

#define contMessage(x, y) { fprintf(stdout, x, y); fflush(stdout); }
#define initMessage(x, y) { fprintf(stdout, "\n%s", ROAD_HEAD);\
			      contMessage(x, y); }
#define warnMessage(x, y) { fprintf(stderr, "\nWARNING: "); \
			    fprintf(stderr, x, y); \
			    fprintf(stderr, "\n"); fflush(stderr); }
#define errMessage(x, y)  { fprintf(stderr, "\n*** ERROR: "); \
			    fprintf(stderr, x, y); \
			    fprintf(stderr, " ***\n"); exit( 1 ); }
#endif
