/*      fuse.c
 *
 *      Author: Enrico Malavasi
 *      Ver. 3.10
 *      Last revised: July 1991
 *
 */
#include "roadGlobals.h"

#define DELPINS(c,d)  {int l; ForAllPins(l)\
			 if((pins[l].cellNo == c) && (pins[l].side == d))\
			   delete_pin(l--);}

FuseCells()
{
    int   i, j, l;        /* Loop counters.  */
    int flag, L;
    RoadCoord z[16];      /* Min. distances. */
    Cell  *Ci, *Cj;       /* Cell pointer.   */

    ForAllLayers( l )
      z[l] = layers[l].wire_width + layers[l].wire_dist*2;

    do {
	flag = 0;
	ForAllCells( i, 1 ) {
	    Ci = cells+i;
	    L  = Ci->layer;
	    for(j=1 ; j<=SUB_CELL_NUM ; j++) {
		if(j == i)  continue;
		Cj = cells+j;
		if(Ci->layer != Cj->layer)           continue;

/*  ============================================================
 *  1.) Vertically aligned - i under j - i not larger than j
 */
		if((Ci->x1 >= Cj->x1) && (Ci->x2 <= Cj->x2))
		  if((Ci->y2 < Cj->y2) && (Ci->y2 > (Cj->y1-z[L]))) {
		      DELPINS(i, NORTH);
		      if(Ci->y1 < Cj->y1)
			if((Ci->x1 == Cj->x1) && (Ci->x2 == Cj->x2))
			  DELPINS(j, SOUTH);
		      Ci->y2 = Cj->y2;
		      flag = 1; i--;
		      break;
		  }
  
/*  ============================================================
 *  Horizontally aligned - i left of j - i not larger than j
 */
		if((Ci->y1 >= Cj->y1) && (Ci->y2 <= Cj->y2))
		  if((Ci->x2 < Cj->x2) && (Ci->x2 > (Cj->x1-z[L]))) {
		      DELPINS(i, EAST);
		      if(Ci->x1 < Cj->x1)
			if((Ci->y1 == Cj->y1) && (Ci->y2 == Cj->y2))
			  DELPINS(j, WEST);
		      Ci->x2 = Cj->x2;
		      flag = 1; i--;
		      break;
		  }

/*  ============================================================
 *  1.) Vertically aligned - i under j - i not smaller than j
 */
		if((Ci->x1 <= Cj->x1) && (Ci->x2 >= Cj->x2))
		  if((Ci->y1 < Cj->y1) && (Ci->y2 > (Cj->y1-z[L]))) {
		      DELPINS(j, SOUTH);
		      if(Ci->y2 < Cj->y2)
			if((Ci->x1 == Cj->x1) && (Ci->x2 == Cj->x2))
			  DELPINS(i, NORTH);
		      ForAllPins(l)
			if(pins[l].cellNo == j)
			  if(abs(pins[l].side) == EAST)
			    pins[l].position += (Cj->y1 - Ci->y1);
		      Cj->y1 = Ci->y1;
		      flag = 1; i--;
		      break;
		  }

/*  ============================================================
 *  Horizontally aligned - i left of j - i not smaller than j
 */
		if((Ci->y1 <= Cj->y1) && (Ci->y2 >= Cj->y2))
		  if((Ci->x1 < Cj->x1) && (Ci->x2 > (Cj->x1-z[L]))) {
		      DELPINS(j, WEST);
		      if(Ci->x2 < Cj->x2)
			if((Ci->y1 == Cj->y1) && (Ci->y2 == Cj->y2))
			  DELPINS(j, EAST);
		      ForAllPins(l)
			if(pins[l].cellNo == j)
			  if(abs(pins[l].side) == NORTH)
			    pins[l].position += (Cj->x1 - Ci->x1);
		      Cj->x1 = Ci->x1;
		      flag = 1; i--;
		      break;
		  }
	    }
	}
    } while(flag);

    ForAllCells( i, 1 )
      ForAllCells( j, i+1 ) {
	  if((cells[i].layer      == cells[j].layer) &&
	     (cells[i].instanceNo == cells[j].instanceNo) &&
	     (cells[i].x1         == cells[j].x1) &&
	     (cells[i].y1         == cells[j].y1) &&
	     (cells[i].x2         == cells[j].x2) &&
	     (cells[i].y2         == cells[j].y2)) {
	      ForAllPins(l)
		if(pins[l].cellNo == j)
		  pins[l].cellNo = i;
	      delete_cell(j--);
	  }
      }
}
