/*        crossing.c
 *
 *        Author: Enrico Malavasi, Univ of Bologna
 *        Ver. 1.00    -    September, 1988
 *
 *        Purpose: Capacitive models for crossing wires.
 */
#include <math.h>
#include "elmacro.h"
#include "rd_types.h"
#include "roadTechnol.h"

/*  -------------------------------------------------------------
 *  Implements the capacitive model for wires crossing each other
 */
#define   CORRECTION_FACTOR    1    /* 2.5 */
pFarad CRS_XxxModel(d, W1, W2)
micron d;              /* Layers of the given wires.   */
micron W1, W2;         /* Widths of the wires.         */        
{
  pFarad x;
/*
 *  Consider only the minimum overlapped area.
 *  ------------------------------------------ */
  x = TECH_OX_DIELECTRIC * (W1 * W2) / d * CORRECTION_FACTOR;
  return(x);
}

/*  -------------------------------------------------------------
 *  Implements the capacitive model for wires crossing each other
 */
pFarad CRS_FittedModel(d, W1, W2)
micron d;              /* Layers of the given wires.   */
micron W1, W2;         /* Widths of the wires.         */        
{
  pFarad x;
/*
 *  Consider only the minimum overlapped area.
 *  ------------------------------------------ */
  x = (0.16 + 0.093*W1 + 0.071*W2 + 0.048*W1*W2) * 0.001;
  return(x);
}

