/*	amplia.c
 *
 * 	Purpose: Enhance the available memory for the <rectangle> structure.
 *		 A simple <realloc> wouldn't be enough because some of
 *		 the structures have fields containing pointers to the
 *		 <rects> array.
 *	Author: Enrico Malavasi
 *      Ver. 3.20
 * 	Last revised: August 1991
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"

#define     INCREMENT     1000

void
amplia_rect(n)
int n;	/* No.of memory locations to be added ............. */
{
    Rect         r0;             /* Rect pointer used for reallocation */
    char        *r,*s;	       /* General use pointers ............. */
    int          i, j, wn;            /* Loop counters .................... */
    int          d;              /* Rect array allocation shift ...... */
    Rect         *rr;

    /*  ------------------------------------------------------------
     *  Compute the real enhancement to introduce.
     */
    for(i=0 ; i<=n ; i+=INCREMENT)  ;
    RECT_ALLOC += i;	/* New size of the available memory  */
    /*  ------------------------------------------------------------
     *  Save to <r> the pointer to the first item of the struct.
     */
    r = (char *) rects;
    rects =REALLOC(rectangle, rects, RECT_ALLOC);
    for(i=TOTAL_RECT_NUM ; i<RECT_ALLOC ; i++) {
	rr = (Rect *) calloc(6, sizeof(Rect));
	rects[i].adjacent = rr;
	for(j=0 ; j<6 ; j++)
	  rects[i].adjacent[j] = NULL;
	rects[i].free = 0;
	rects[i].flag = 0;
    }
    if(r == NULL)   return;
    /*  ------------------------------------------------------------
     *  Compute the shift of the struct in bytes.
     */
    s = (char *) rects;
    d = s-r;
    
    if(d) {
	/*  ------------------------------------------------------------
	 *  If there was a shift, set the neighborhood relations.
	 */
	ForAllRects( i ) {
	    r0 =rects+i;
	    six_sides(j)
	      if(neighbor(r0,j) != NULL)  {
		  /*  ------------------------------------------------------------
		   *  Swap the neighbors
		   */
		  s = (char *) neighbor(r0, j);
		  s += d;
		  neighbor(r0, j) = (Rect) s;
	      }
	}
	
	/*  ------------------------------------------------------------
	 *  Set the pointer arrays of wires ...
	 */
	ForAllWires(wn) {
	    for(j=0 ; j<wireSize(wn) ; j++)  {
		s = (char *) wSegmRect(wn, j);
		s += d;
		wSegmRect(wn, j) = (Rect) s;
	    }
	}
	
	/*  ------------------------------------------------------------
	 *  ... and pins, if the rect reference has already been set.
	 */
	if(pins[0].rect != NULL)
	  ForAllPins(i) {
	      s = (char *) pins[i].rect;
	      s += d;
	      pins[i].rect = (Rect) s;
	  }
	ShiftPinSheetRects(d);
    }
}
