/*	bldwire.c
 *
 *	Author: Enrico Malavasi
 *      Ver. 2.0
 * 	Last revised:  August, 1990
 *
 *	Purpose: Build a wire in detail.
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"

static int  tot;   /* No. of connected pins ............... */

static void extended_split(r, pos, dir)
Rect   r;          /*  Rect to be split ................... */
int    pos;        /*  Element position in the wire ....... */
int    dir;        /*  Splitting direction ................ */
{
    int i, j, k;
    Rect         s; 
    char         *alfa, *beta; /*  char* locations for the wire shift */
    int          diff,         /*  Difference between alfa and beta   */
                 dir2;         /* Dual direction */
    RoadCoord    ddd;          /* Rect size      */


    k   = status.RectNum;	/* Save the current rects No. .... */
    dir2 = AbsDualDir(dir);
    ddd = rect_size(r, dir2);

    if((pos >= 0) &&
       (wire_shift(WIRE_NUM, pos, -dir2, ddd) > 0)) {
	tot = CompactWire(WIRE_NUM);
	return;
    }
    /*  ------------------------------------------------------------
     * Set the value of the minor orthogonal direction <dir2>
     */
    alfa = (char *) rects;

    if(ddd < 2)
      errRaise("ROAD", 1, "Overcrowding detected. Routed circuit is lost");
    GridSplit(r, (RoadCoord) (ddd/2), dir, NOTFOUND);
    
    /*  -------------------------------------------------------------
     *  If there has been a shift of <rects> array, set the <w> array
     */
    beta = (char *) rects;
    if(diff = beta-alfa)
      for(i=0 ; i<wireSize(WIRE_NUM) ; i++) {
	  alfa = (char *) wSegmRect(WIRE_NUM, i);
	  alfa += diff;
	  wSegmRect(WIRE_NUM, i) = (Rect) alfa;
      }
    
    for(i=k ; i<TOTAL_RECT_NUM ; i++) {
	s = rects+i;                              /* New rectangle ............ */
	if((r = free_neighbor(s, -dir2)) == NULL) /* Original rectangle ....... */
	  continue;
	for(j=0 ; j<wireSize(WIRE_NUM) ; j++)
	  if(wSegmRect(WIRE_NUM, j) == r) {
	      if(wSegmSide(WIRE_NUM, j) == dir) {
		  wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)) = s;
		  wSegmSide(WIRE_NUM, wireSize(WIRE_NUM)) = dir;
		  (wireSize(WIRE_NUM))++;
		  if(j < tot)   tot = CompactWire(WIRE_NUM);
	      } else
		wSegmRect(WIRE_NUM, j) = s;
	  }
    }
    wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)) = NULL;

    for(i=k ; i<TOTAL_RECT_NUM ; i++) {
	s = rects+i;                            /* New rectangle ............ */
	for(j=tot ; j<wireSize(WIRE_NUM) ; j++)
	  if((wSegmRect(WIRE_NUM, j) == s) && (wSegmSide(WIRE_NUM, j) != dir)) {
	      wire_segment_shift(WIRE_NUM, j, -dir2);
	  }
    } 
    tot = CompactWire(WIRE_NUM);
}

/*  ============================================================
 *  This function builds a wire in detail. It gets in input an
 *  array of rectangles representing the path for the wire and
 *  creates the detailed wire as a list of couples (rect, side)
 */
build_wire(NetNo, p, pinlist)
int   NetNo;
PATH  p;	        /* Given path .................. */
Pins *pinlist;	        /* List of pins ................ */
{
    int          i, j, k;           /* Loop counters ............... */
    int          dir, fl;
    Rect         r, s, t;               /* Temporary storage vars ...... */
    RoadTerm    *pi, *pj;

    
    /*  Remove the useless stacked terms segments */
    ForAllPins(i) {
	pi = pins+i;
	r = pi->rect;
	if((s=neighbor(r, UP)) != NULL) {
	    if((pj=PinIsIn(t, NetNo)) != NULL)
	      if(pi->equivNo == pj->equivNo) {    /*  Stacked pins */
		  
		  fl = 0;
		  four_sides(dir)
		    if((t=neighbor(r, dir)) != NULL)
		      if(rectInPath(t, p) != NOTFOUND) {
			  fl = 1;
			  break;
		      }
		  if(fl == 0)
		    delFromPath(r, p);
		  
		  fl = 0;
		  four_sides(dir)
		    if((t=neighbor(s, dir)) != NULL)
		      if(rectInPath(t, p) != NOTFOUND) {
			  fl = 1;
			  break;
		      }
		  if(fl == 0)
		    delFromPath(s, p);
	      }
	}
    }

/*  ------------------------------------------------------------
 *  Allocate enough memory for the new wire.
 */
    rdWire[WIRE_NUM].rect = ALLOC(Rect, TOTAL_RECT_NUM/LAYER_NUM);
    rdWire[WIRE_NUM].side = ALLOC(int,  TOTAL_RECT_NUM/LAYER_NUM);
    wireNetNo(WIRE_NUM) = NetNo; /* Store the net */

/*  -----------------------------------------------------------------
 *  Delete the unused pins, i.e. those equivalent pins which have not
 *  been connected.
 */
    for(k=0 ; pinlist[k]!=NULL ; )  k++;    /* Count the number of pins. */
    for(j=0 ; j<k ; j++)
      for(i=0 ; (r=pinlist[j]->rect)!=p[i] ; i++)
	if(p[i] == NULL)	{
	    pinlist[j--] = pinlist[--k];
	    break;
	}
    pinlist[k] = NULL;

/*   Build the segments of the wire.  */
    wireSize(WIRE_NUM) = 0;
    for(i=0 ; p[i]!=NULL ; i++)	{
	r =free_neighbor(p[i],EAST);
	s =free_neighbor(p[i],WEST);
	if((r != NULL) || (s != NULL))
	  for(j=0 ; p[j]!=NULL ; j++)
	    if(((p[j] == r) || (p[j] == s)) && (i != j)) {
		wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)) = p[i];
		wSegmSide(WIRE_NUM, wireSize(WIRE_NUM)) = NORTH;
		(wireSize(WIRE_NUM))++;
		break;
	    }

	r = free_neighbor(p[i],NORTH);
	s = free_neighbor(p[i],SOUTH);
	if((r != NULL) || (s != NULL))
	  for(j=0 ; p[j]!=NULL ; j++)
	    if(((p[j] == r) || (p[j] == s)) && (i != j)) {
		wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)) = p[i];
		wSegmSide(WIRE_NUM, wireSize(WIRE_NUM)) = EAST;
		(wireSize(WIRE_NUM))++;
		break;
	    }
	
	if((r = neighbor(p[i],UP)) != NULL) {
	    for(j=0 ; p[j]!=NULL ; j++)
	      if(p[j] == r)
		if(i != j)
		  if(((pi=PinIsIn(p[i], NetNo)) == NULL) ||
		     ((pj=PinIsIn(r,  NetNo)) == NULL) ||
		     (pi->equivNo != pj->equivNo)) {
		      wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)) = p[i];
		      wSegmSide(WIRE_NUM, wireSize(WIRE_NUM)) = UP;
		      (wireSize(WIRE_NUM))++;
		      break;
		  }
	}
    }

/*  ------------------------------------------------------------
 *  Create the wire segments correspondent to the pins locations
 */
    for(i=tot=0 ; pinlist[i]!=NULL ; i++) {
	add_to_wire(WIRE_NUM, pinlist[i]->rect, AbsDualDir(pinlist[i]->side));
	SWAP(Rect, wSegmRect(WIRE_NUM, tot), wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)-1));
	SWAP(int,  wSegmSide(WIRE_NUM, tot), wSegmSide(WIRE_NUM, wireSize(WIRE_NUM)-1));
	tot++;
    }

/*  ------------------------------------------------------------
 *  Create the necessary space in front of the pins
 */
  for(i=0 ; pinlist[i]!=NULL ; i++) {
    k = ((pinlist[i]->cellNo == 0) ? (- pinlist[i]->side) : pinlist[i]->side);
    if(free_neighbor(r=pinlist[i]->rect, k) == NULL)
      extended_split(r, -1, AbsDualDir(k));
  }

    WireDeleteDuplicates(WIRE_NUM);
/*
 *  This part could be avoided, but some bugs set the free fields
 *  where they should not ...
 */
  free_wire(WIRE_NUM, 0);
  for(i=0 ; i<WIRE_NUM ; i++) {
      wireFree(i) = 0;
      free_wire(i, 1);
  }

/*  ------------------------------------------------------------
 *  Now split all the rects previously selected.
 */
  for(i=tot,j=0 ; i<wireSize(WIRE_NUM) ; )  {

      r = wSegmRect(WIRE_NUM, i);
      switch(wSegmSide(WIRE_NUM, i))  {
	case NORTH :
	case  EAST :

	  dir = AbsDualDir(wSegmSide(WIRE_NUM, i));
	  if(((s =free_neighbor(r, wSegmSide(WIRE_NUM, i))) == NULL)
	     ||
	     ((free_neighbor(s, dir) == NULL) &&
	      (free_neighbor(r, dir) != NULL))
	     ||
	     ((free_neighbor(s, -dir) == NULL) &&
	      (free_neighbor(r, -dir) != NULL))
	     ||
	     (((s=free_neighbor(r, -dir)) != NULL) &&
	      (free_neighbor(s, wSegmSide(WIRE_NUM, i)) == NULL))
	     ||
	     (((s=free_neighbor(r, dir)) != NULL) &&
	      (free_neighbor(s, wSegmSide(WIRE_NUM, i)) == NULL)))
	    unless(delete_appendixes(WIRE_NUM, r, wSegmSide(WIRE_NUM, i))) {
		extended_split(r, i, dir);
		j = 1;
	    }
	  break;
	  
	case UP :
	  
	  if((free_neighbor(r,EAST) == NULL)
	     ||
	     (free_neighbor(neighbor(r,UP), EAST) == NULL)) {
	      extended_split(r, i, NORTH);
	      j = 1;
	      break;
	  }
	  
	  r = wSegmRect(WIRE_NUM, i);
	  if(((s=free_neighbor(r, NORTH)) == NULL)
	     ||
	     (free_neighbor(neighbor(r,UP),NORTH) == NULL)
	     ||
	     (free_neighbor(s, EAST) == NULL)) {
	      extended_split(r, i, EAST);
	      j = 1;
	  }
	  
	  default    : break;
      }
      
      if((++i) == wireSize(WIRE_NUM)) {
	  if(j == 0)     break;
	  else {
	      i = tot;
	      j = 0;
	  }
      }
  }
    
    wSegmRect(WIRE_NUM, wireSize(WIRE_NUM)) = NULL;
    wireDesc(WIRE_NUM)  = MINWWIDTH;
    if(rd_net[NetNo].wwidth > 0) {
	wireDesc(WIRE_NUM) = DEFWWIDTH;
	rdWire[WIRE_NUM].wWidth = REALLOC(RoadCoord, rdWire[WIRE_NUM].wWidth, LAYER_NUM);
	ForAllLayers( i )
	  wSegmWidth(WIRE_NUM, i) = rd_net[NetNo].wwidth;
    }
    wireFree(WIRE_NUM) = 0;
    rdWire[WIRE_NUM].rect = REALLOC(Rect, rdWire[WIRE_NUM].rect, wireSize(WIRE_NUM));
    rdWire[WIRE_NUM].side = REALLOC(int,  rdWire[WIRE_NUM].side, wireSize(WIRE_NUM));

    WIRE_NUM++;
    free_wire(WIRE_NUM-1, 1);
    deleteNetFromNetList(NetNo);
    
    WireDeleteDuplicates(WIRE_NUM-1);
    PruneWire(WIRE_NUM-1);
}
