/*     capcontr.c
 *
 *     Author: Enrico Malavasi
 *     Ver. 3.00
 *     Last revised: January, 1991
 *
 *     Purpose: Perform a control over the capacitive coupling of nets.
 */
#include "roadGlobals.h"
#include "transf.h"
#define  MAX_CRIT       1.5

reportCoupling( reportFile )
char *reportFile;
{
    FILE *fil;
    int i,j, cnt=0;
    fFarad  Coupl, maxcap;
    RoadWeight weight;

    if((fil=fopen(reportFile, "w")) == NULL) {
	fprintf(stderr, "\nCannot write file %s\n", reportFile);
	rdExit( 1 );
    }
    VERBOSE( printf("Writing the coupling-report file %s\n", reportFile));
    for(i=0 ; i<NET_NUM ; i++)
      for(j=i+1 ; j<NET_NUM ; j++)
	if((weight=InquireCC_coupling(i, j, &maxcap)) > 0) {
	    maxcap *= 1000;
	    Coupl = 1000 * capacitive_coupling(i, j);
	    if(Coupl > 0) {
		if(cnt == 0) {
		    fprintf(fil,"\nAll capacitances are given in fF");
		    fprintf(fil,"\n   Nets       Real coup.   Max coupl.      Weight");
		}
		cnt++;
		fprintf(fil, "\n%s\t%s\t%10.3f",
			rd_net[i].name, rd_net[j].name, Coupl);
		if(maxcap > 0)  fprintf(fil, " %10.3f", maxcap);
		else            fprintf(fil, "      --    ");
		fprintf(fil, "  %c", (Coupl > maxcap) ? '*' : ' ');
		fprintf(fil, "    %7.3f", weight);
	    }
	}
    fprintf(fil, "\n%d coupled critic pair%sreported.\n\n",
	    cnt, (cnt==1)?" ":"s ");
    fclose(fil);
}

rd_Statistics(fout)
FILE *fout;
{
  int l, wn, pos;
  int viaNum, tviaNum;
  RoadCoord len[6], tlen[6], totLen;


  fprintf(fout, "\nWire lengths (in um)");
  fprintf(fout, "\nNET\t[Vias]");
  for(l=0 ; l<LAYER_NUM ; l++)
    fprintf(fout, "\t  %s", layers[l].name);

  for(tviaNum=wn=0 ; wn<LAYER_NUM ; wn++)
    tlen[wn] = 0;

  for(wn=0 ; wn<WIRE_NUM ; wn++) {
    for(viaNum=l=0 ; l<LAYER_NUM ; l++)   len[l] = 0;
    for(pos=0 ; pos<wireSize(wn) ; pos++) {
      switch(wSegmSide(wn, pos)) {
      case UP :
	viaNum++; break;
      case NORTH :
	len[wSegmLayer(wn, pos)] += rwidth(wSegmRect(wn, pos));
	break;
      case EAST  :
	len[wSegmLayer(wn, pos)] += rheight(wSegmRect(wn, pos));
        break;
      }
    }
    fprintf(fout, "\n%s\t[%4d]", rd_net[wireNetNo(wn)].name, viaNum);
    for(l=0 ; l<LAYER_NUM ; l++)
      fprintf(fout, "\t%6d", (int) RoadToMicron(len[l]));
    for(l=0 ; l<LAYER_NUM ; l++)   tlen[l] += len[l];
    tviaNum += viaNum;
  }
  fprintf(fout, "\n...\nTOTAL\t[%4d]", tviaNum);
  for(l=0 ; l<LAYER_NUM ; l++)
    fprintf(fout, "\t%6d", (int) RoadToMicron(tlen[l]));
  fprintf(fout, "\n");    
}
