/*      collapse.c
 *
 *      Author: Enrico Malavasi,  Univ. of Bologna
 *      Ver. 1.00    -    September, 1989
 *
 *      Purpose: Wire collapsing, that is deleting the useless
 *               loops produced by the build_wire utility.
 */
#include "roadGlobals.h"
#include  "space.h"

/*  ============================================================
 *  This function removes the narrow loops of a given wire
 */
collapseWire(wn)
int wn;       /*  Index of the wire to be collapsed  */
{
  RoadCoord diff,          /*  Difference between distances */
            boxWidth;      /*  Protection frame width       */
  int       side, sidej,   /*  Side indicators  */
            i, pos, wPos;     /*  Wire position indicator  */
  Rect      r, rj;
  RoadBox   BB, bUp;            /*  Protection frame */

  if(wn == NOTFOUND) return;

  CONDfree_wire(wn, 0);
  for(i=0 ; i<wireSize(wn) ; i++) {
    wPos = i;
    if((side=wSegmSide(wn, wPos)) == UP)   continue;
    r = wSegmRect(wn, wPos);
/*  ------------------------------------------------------------
 *  Build the protection frame
 */
    boxWidth = spacingBetween(rLayer(r), side, rLayer(r), NORTH);
    if(side != UP) adjustBoxWidth(boxWidth, wn, rLayer(r));
    WireSegmentToBox(r, side, boxWidth, &BB, bUp, NODIRECTION, NODIRECTION);

/*  ------------------------------------------------------------ 
 *  Find the other wire-elements intersecting the protection frame
 */
    loop {
      pos = (-1);
      while((pos=WireIntersection(wn, &BB, pos+1)) != NOTFOUND) {
	if(pos == wPos)  continue;
/*  ------------------------------------------------------------
 *  An narrow loop was found. The micro-translation is defined
 *  and then applied.
 */
	rj    = wSegmRect(wn, pos);
	sidej = wSegmSide(wn, pos);
	if((side == NORTH) && (sidej == NORTH)) {
	  if(r->x2 != rj->x2)   continue;
	  if((diff=rj->y2-r->y2) == 0)  continue;
	  if(diff < 0) {
	    SWAP(int, pos, wPos);
	    CHGSGN(diff);
	  }

	  if(TryToEnlarge(wn, pos, SOUTH, diff) < diff)
	    TryToEnlarge(wn, wPos, NORTH, diff);
	  break;
	}

	if((side == EAST) && (sidej == EAST)) {
	  if(r->y2 != rj->y2)   continue;
	  if((diff=rj->x2-r->x2) == 0)  continue;
	  if(diff < 0) {
	    SWAP(int, pos, wPos);
	    CHGSGN(diff);
	  }

	  if(TryToEnlarge(wn, pos, WEST, diff) < diff)
	    TryToEnlarge(wn, wPos, EAST, diff);
	  break;
	}
      }
      if(pos == NOTFOUND)
	if(side == UP)
	  if(BB.z != bUp.z) {
	    BB = bUp;
	    continue;
	  }
      break;
    }
  }
  CONDfree_wire(wn, 1);
  WireDeleteOverlap(wn);  
}
