#include "port.h"
#include "utility.h"
#include "vov.h"
#include "roadGlobals.h"

/*  ============================================================
 *  Commands
 */
/*  No arguments  */
#define  COMM_EXIT          0
#define  COMM_ROUTETOEND    1      /* Route all unrouted nets  */
#define  COMM_ROUTENEXT     5      /* Route next scheduled net */
#define  COMM_SHIELDALL    10      /* Build all shields  */
#define  COMM_SAVEALL      20      /* Save all */
#define  COMM_COLLAPSE     30      /* Grid collapsing  */
#define  COMM_COUPLING     50      /* Compute coupling between nets */
/* No arguments, graphics */
#define  COMM_INHIBIT      81
/*  One argument  */
#define  COMM_BLOCK       101
#define  COMM_DELETENET   105      /* Delete a given net */
#define  COMM_ROUTENET    110      /* Route a given net  */
#define  COMM_SPACENET    115      /* Space a net */
#define  COMM_SHIFT       180
#define  COMM_HIGHLIGHT   184
#define  COMM_ZOOM        191 
#define  COMM_DEZOOM      192
/*  Two arguments  */
#define  COMM_SHIELD      201

extern int    DumpBlock;

typedef struct _interactComm {
  char *name;
  int   code;
} _interactComm;

static _interactComm interactComm[] = {
    { "Quit",  COMM_EXIT      },
    { "Dump",  COMM_SAVEALL   },
    { "Blck",  COMM_BLOCK     },  /* Switch the block param. */
    { "Inhb",  COMM_INHIBIT   },
    { "Net ",  COMM_ROUTENET  },
    { "Auto",  COMM_ROUTENEXT },
    { "Del ",  COMM_DELETENET },
    { "Spac",  COMM_SPACENET  },
    { "Shld",  COMM_SHIELD    },
    { "Coll",  COMM_COLLAPSE  },
    { "Full",  COMM_DEZOOM    },  /*  Full screen  */
    { "Zoom",  COMM_ZOOM      },  /*  Zoom in  */
    { "Hlgt",  COMM_HIGHLIGHT },
};

typedef struct _scriptComm {
    int code;
    int         *args;
} _scriptComm;

_scriptComm  *scriptComm;
int           scriptCommNum = 0;


static int
  totWireSegments()
{
    int  wn;
    int  tot = 0;
    ForAllWires(wn) {
	tot += wireSize( wn );
    }
    return  tot;
}

static int
  select_net()
{
    char *netName[NODE_MAX];
    int i;
    
    ForAllNets( i )
      netName[i] = rd_net[i].name;
    netName[NET_NUM] = NULL;
    return  which_net("Which net ?", netName);
}

char **
  getTitleNames()
{
    char  **items;
    int     i, k;

    k = sizeof( interactComm ) / sizeof( _interactComm );
    items = ALLOC(char *, k+1);
    for(i=0 ; i<k ; i++) {
	items[i] = util_strsav( interactComm[i].name );
    }
    items[k] = NULL;
    return  items;
}

static int
getNetWithName(name)
char *name;
{
  int i;
  
  for(i=0 ; i<NET_NUM ; i++)
    if(strcmp(rd_net[i].name, name) == 0)
      return(i);
  return(NOTFOUND);
}

static void
  CreateCommand(CommCode, Args, transf)
int    CommCode;
char **Args;
int    transf;
{
  int  i, ArgNum;

  scriptComm = REALLOC(_scriptComm, scriptComm, (scriptCommNum+1));
  scriptComm[scriptCommNum].code = CommCode;
  ArgNum = CommCode / 100;
  if(ArgNum > 0)
    scriptComm[scriptCommNum].args = ALLOC(int, ArgNum);
  for(i=0 ; i<ArgNum ; i++) {
      scriptComm[scriptCommNum].args[i] = name_to_net(Args[i]);
      if(transf)
	scriptComm[scriptCommNum].args[i] = scriptComm[scriptCommNum].args[i];
  }
  scriptCommNum++;
}

SetDefaultScript()
{
    scriptCommNum = 0;
    CreateCommand(COMM_ROUTETOEND, NULL, 0);
    CreateCommand(COMM_SHIELDALL,  NULL, 0);
    CreateCommand(COMM_SAVEALL,    NULL, 0);
}

#define  MAXCOMMNUM  30
#define  MAXCOMMLEN  15

read_script(ScrName)
char *ScrName;
{
    FILE    *fScr;
    int      k, argc;
    int      repet;
    char    **argv;

    if((fScr=fopen(ScrName, "r")) == NULL) {
	fprintf(stderr, "\n%s: File not found\n", ScrName);
	rdExit( 1 );
    }

    scriptCommNum = 0;
    argv = ALLOC(char *, MAXCOMMNUM);
    for(k=0 ; k<MAXCOMMNUM ; k++)
      argv[k] = ALLOC(char, MAXCOMMLEN);
    
    loop {
	inputCommandLine(fScr, &argc, argv);
	if(argc <= 0) break;
	
	if(argv[0][0] == '#')
	  continue;               /*  Comment  */
	
	if(argc > MAXCOMMNUM) {
	    fprintf(stderr, "Too many arguments in script %s\n", ScrName);
	    rdExit( 1 );
	}
	
	if(strcasecmp(argv[0], "COLLAPSE") == 0) {
	    CreateCommand(COMM_COLLAPSE, argv, 0);
	    continue;
	}

	if(strcasecmp(argv[0], "DELETE") == 0) {
	    for(k=1 ; k<argc ; k++)
	      CreateCommand(COMM_DELETENET, argv+k, 1);
	    continue;
	}
	
	if(strcasecmp(argv[0], "ROUTE") == 0) {
	    for(k=1 ; k<argc ; k++) {
		if(strcasecmp(argv[k], "NEXT") == 0) {
		    repet = 1;
		    if(argc > k+1)   repet = atoi(argv[++k]);
		    while( repet-- ) {
			CreateCommand(COMM_ROUTENEXT, argv, 0);
		    }
		} else
		  CreateCommand(((strcasecmp(argv[k], "ALL"), 0) == 0) ?
				COMM_ROUTETOEND : COMM_ROUTENET, argv+k);
	    }
	    continue;
	}
	
	if(strcasecmp(argv[0], "SPACE") == 0) {
	    for(k=1 ; k<argc ; k++)
	      CreateCommand(COMM_SPACENET, argv+k, 1);
	    continue;
	}
	
	if(strcasecmp(argv[0], "SAVE") == 0) {
	    CreateCommand(COMM_SAVEALL, argv, 0);
	    continue;
	}
	
	if(strcasecmp(argv[0], "SET") == 0) {
	    if(strcasecmp(argv[1], "BLOCK") == 0) {
		strcpy(argv[1], "1");
		CreateCommand(COMM_BLOCK, argv+1, 0);
		continue;
	    }
	}
	if(strcasecmp(argv[0], "UNSET") == 0) {
	    if(strcasecmp(argv[1], "BLOCK") == 0) {
		strcpy(argv[1], "0");
		CreateCommand(COMM_BLOCK, argv+1, 0);
		continue;
	    }
	}
  }
    fclose(fScr);
    for(k=0 ; k<MAXCOMMNUM ; k++)
      FREE(argv[k]);
    FREE(argv);
}

static void
  CommandBlock(flg)
int  flg;
{
    if(flg >= 0)  DumpBlock = flg;
    else {
	if(DumpBlock == 0)  DumpBlock = 1;
	else                DumpBlock = 0;
    }
    ShowParamWindow();
}
static void
  CommandDelNet(netNo)
int netNo;
{
    int j, wn;
    
    if(netNo == NOTFOUND)                       return;
    if((wn=net_to_wire(netNo,0)) == NOTFOUND)   return;
    delete_wire(wn);
    AddNetListItem(netNo);
    /*  Delete all the dummy pins of this wire.   */
    ForAllPins(j)
      if((pins[j].netNo == netNo) && (pins[j].cellNo == NOTFOUND))
	delete_pin(j--);
}

static void
  Inhibit()
{
  int        i, NetNo;
  RoadCoord  x,y;
  RoadCoord  x1, y1, x2, y2;

  NetNo = select_net();
  get_point(&x1, &y1);
  get_point(&x2, &y2);

  SORT(RoadCoord, x1, x2);
  SORT(RoadCoord, y1, y2);

  ForAllPins(i)
    if((pins[i].netNo == NetNo) &&
       ((x=pin_x(pins+i)) > x1) &&
       ((y=pin_y(pins+i)) > y1) &&
       (x < x2) && (y < y2))
      pins[i].netNo = -1-NetNo;
}

static void
  ExecSingleCommand( code, args, circuitName )
int  code;
int *args;
char *circuitName;
{
    int  n;
    int  netNo;
    switch(code) {
	
      case COMM_BLOCK :
	n = RD_CHECK_FLAG( BATCH_MODE ) ? args[0] : (-1);
	CommandBlock( n );
	break;

      case COMM_DELETENET :
	netNo = RD_CHECK_FLAG( BATCH_MODE ) ? args[0] : select_net();
	CommandDelNet( netNo );
	show_all(-1);
	break;

      case COMM_EXIT :
	rdExit( 0 );

      case COMM_HIGHLIGHT :
	if INTERACTIVE_MODE {
	    show_wire( net_to_wire(select_net(), 0), 1 );
	}
	break;

      case COMM_INHIBIT :
	if INTERACTIVE_MODE {
	    Inhibit();
	}
	break;	

      case COMM_ROUTENET :
	netNo = RD_CHECK_FLAG( BATCH_MODE ) ? args[0] : select_net();
	RouteSelectedNet( netNo );
	show_all(-1);
	break;

      case COMM_ROUTENEXT :
	RouteSelectedNet( SuggestNextNet() );
	show_all(-1);
	break;

      case COMM_ROUTETOEND :
	while((netNo=SuggestNextNet()) != NOTFOUND) {
	    RouteSelectedNet( netNo );
	    VERBOSE( printf("\n--------------> %d Grid Nodes",    TOTAL_RECT_NUM));
	    VERBOSE( printf("\n--------------> %d Wire Segments\n", totWireSegments()));
	}
	show_all(-1);
	break;

      case  COMM_SAVEALL :
	{   char *outputFile;
	    if(DumpBlock == 0)  outputFile = NewDumpFileName(circuitName);
	    else                outputFile = LastDumpFileName(circuitName);
	    VOVoutput( VOV_UNIX_FILE, outputFile );
	    RoadDump( outputFile );
	}
	break;

      case COMM_SHIELD:
	{   int        a, b;
	    int        shldNet;
	    RoadWeight coupling;  /*  Coupling weight */
	    pFarad     maxcap;    /*  Max.Allowed Capacitance  */
	    
	    a = select_net();  b = select_net();
	    if(b == getCriticCapPair(a, &coupling, &maxcap, &shldNet)) {
		if(shldNet != NOTFOUND)  {
		    shield(a, b, shldNet);
		    fullSpaceWire(net_to_wire(shldNet, 0));
		}
		else if(status.BulkNet != NOTFOUND) {
		    shield(a, b, status.BulkNet);
		    fullSpaceWire(net_to_wire(status.BulkNet, 0));
		}
	    }
	}
	show_all(-1);
	break;

      case  COMM_SPACENET:
	netNo = RD_CHECK_FLAG( BATCH_MODE ) ? args[0] : select_net();
	fullSpaceWire(net_to_wire(netNo, 0));
	show_all(-1);
	break;

      case COMM_DEZOOM :
	RoadZoom( 0 );
	show_all(-1);
	break;
      case COMM_ZOOM :
	RoadZoom( 1 );
	show_all(-1);
	break;
	
      case COMM_COLLAPSE :
	if INTERACTIVE_MODE
	  {   RoadCoord x1,y1;
	      get_point(&x1, &y1);
	      collapseGrid(find_rect(x1, y1, 0), NORTH);
	  }
	show_all(-1);
	break;
	
      default:
	break;
    }
}    

ExecuteScript( circuitName )
char *circuitName;
{
    int i;
    for(i=0 ; i<scriptCommNum ; i++) {
	ExecSingleCommand( scriptComm[i].code, scriptComm[i].args, circuitName );
	if(RD_CHECK_FLAG(DEBUG_OPTION))    rdGeneralCheck();
    }
}

roadExecInteractive( circuitName )
char *circuitName;
{
    int  CommNo;
    int i, n,
        nds[2];

    show_all(-1);
    loop {
	CommNo = which_comm();
	if(CommNo < 0)                              continue;
	if(interactComm[CommNo].code == COMM_EXIT)   break;
	ExecSingleCommand( interactComm[CommNo].code, NULL, circuitName );
	if(RD_CHECK_FLAG(DEBUG_OPTION))    rdGeneralCheck();
    }
}
