/*	concost.c
 *
 * 	Purpose: Functions related to the setting or the evaluation
 *		 of the connection cost between two rectangles.
 *		 Such cost is always expressed in terms of a couple
 *		 (rectangle, direction).
 *	Author: Enrico Malavasi, Univ. of Bologna
 *      Ver. 3.20
 *      Last revised: August 1991
 *
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"

/*  ============================================================
 *  This function returns the connection cost of a given rectangle
 *  in a given direction.
 */
RoadCost
  connection_cost(r, dir)
Rect r;                  /* Given rectangle .............. */
int  dir;                /* Side on which to read the cost */
{
    if(r == NULL)    return(0);
    if(dir > 0)      return(r->cost[dir-1]);
    else             return(connection_cost(neighbor(r, dir), -dir));
}

/*  ============================================================
 *  This function sets the connection cost value for a given
 *  rectangle on a given direction.
 */
set_connection_cost(r, dir, c)
Rect r;	                /* Rect to be set ............. */
int  dir;      		/* Side ....................... */
RoadCost  c;		/* Cost to set ................ */
{
    Rect p;

    if((c > MAX_COST) ||  (c < 0))       c = MAX_COST;

    if(dir > 0)    r->cost[dir-1] = c;
    else if((p=neighbor(r, dir)) != NULL)
      p->cost[-1-dir] = c;
}
