/*      costwr.c
 *
 *      Author: Enrico Malavasi
 *      Ver. 3.00
 *      Last revised: January, 1991
 *
 *      Purpose: Set the wire-proximity cost of the cost function. Such cost
 *               comes from an user option about the wire coupling.
 */
#include "roadGlobals.h"
#include "transf.h"

#define SAME_LAY_COST  10     /* Parallel cost   */
#define OTHERLAY_COST  10     /* Orthogonal cost  */

#define MAX_WIRE_DIST  200  /* Max. distance (in um.) for the wire influence */
#define add_cost(p, c, d)   set_connection_cost(p,d,connection_cost(p,d)+c)


/*  ==============================================================
 *  Given a wire index <wn>, this function computes and stores the
 *  additional cost due to the proximity of the given wire. This
 *  can be done on a peculiar grid node or everywhere; in the second
 *  case the cost function is triangle shaped.
 */
costwr(wn, NetNo)
int wn;		/* Index of the wire about which to set the proximity cost */
int  NetNo;     /* Net for which the cost has to be set  */
{
    Rect      r1, r0, s;
    RoadCoord dist_max;             /* Max. influence distance. */
    int       i, cnt,
              side, dside, mside, mdside;
    float     c_cross, c_near, c_cost;
    float     m, cost, size, dist;
    pFarad    maxcap;

    if(wireNetNo(wn) == NetNo) return;
    if((m=InquireCC_coupling(wireNetNo(wn), NetNo, &maxcap)) <= 0) return;
    
    dist_max = MicronToRoad(MAX_WIRE_DIST);
    if(m > 1) {
	c_cross = c_near = m * MAX_COST;
    } else {
	c_cross   = m * OTHERLAY_COST;
	c_near    = m * SAME_LAY_COST;
	dist_max  *= (c_near / SAME_LAY_COST);
	if(dist_max < MIN_NODE_SIZE) return;
    }
    
    for(i=0 ; (r1=r0=wSegmRect(wn, i))!=NULL ; i++) {
	side  = wSegmSide(wn, i);
	mside = (- side);
	dside = AbsDualDir(side);
	mdside= (- dside);
	if(side == UP) continue;

	c_cost = c_near;
	for(cnt=0 ; cnt<3 ; cnt++) {
	    if(cnt == 1)  {  r1 = neighbor(r0, DOWN); c_cost = c_cross; }
	    if(cnt == 2)  {  r1 = neighbor(r0, UP);   c_cost = c_cross; }
	    if(r1 == NULL)  continue;

	    for(dist=0,s=neighbor(r1,side) ; s!=NULL ; s=free_neighbor(s, side)) {
		dist += rect_size(s, side);
		if(dist > dist_max)     break;
		cost = c_cost * (dist_max-dist) / dist_max;
	
		if(neighbor(s, side) != NULL) {
		    size = rect_size(neighbor(s,side),side);
		    add_cost(s,((RoadCost) (cost*size)),side);
		}
		if(neighbor(s, dside) != NULL) {
		    size = rect_size(neighbor(s,dside),dside);
		    add_cost(s, ((RoadCost) (cost*size)), dside);
		}
		if(neighbor(s,mdside) != NULL) {
		    size = rect_size(s,mdside);
		    add_cost(s, ((RoadCost) (cost*size)), mdside);
		}
		if(neighbor(s, UP) != NULL)
		  add_cost(s, ((RoadCost) cost), UP);
	    }
	    
	    for(dist=0,s=r1 ; s!=NULL ; s=free_neighbor(s, mside)) {
		dist += rect_size(s, side);
		if(dist > dist_max) break;
		cost = c_cost * (dist_max-dist) / dist_max;
		
		if(neighbor(s, side) != NULL) {
		    size = rect_size(s,side);
		    add_cost(s,((RoadCost) (cost*size)),side);
		}
		if(neighbor(s, dside) != NULL) {
		    size = rect_size(neighbor(s,dside),dside);
		    add_cost(s, ((RoadCost) (cost*size)), dside);
		}
		if(neighbor(s,mdside) != NULL) {
		    size = rect_size(s,mdside);
		    add_cost(s, ((RoadCost) (cost*size)), mdside);
		}
		if(neighbor(s, UP) != NULL)
		  add_cost(s, ((RoadCost) cost), UP);
	    }
	}
    }
}
