/*        delete.c
 *
 *        Author: Enrico Malavasi, University of Bologna
 *        Ver. 1.00   -     July, 1989
 *
 *        Purpose: Utilities for the deletion of RoAD items.
 */
#include "roadGlobals.h"

delete_cell(cellNo)     /*   CELL   */
int cellNo;
{
    int i;

    cells[cellNo] = cells[SUB_CELL_NUM];
    ForAllPins(i)
      if(pins[i].cellNo == SUB_CELL_NUM)
	pins[i].cellNo = cellNo;
    SUB_CELL_NUM--;
}
delete_wire(wireNo)
int wireNo;
{
    int  i;
    int  netNo;      /*  net index of the wire to be deleted */

    if((wireNo >= WIRE_NUM) || (wireNo < 0))  return;
    netNo = wireNetNo(wireNo);

    /*  Reset the neighborhood relations and physically delete the wire. */
    if(wireNo < (--WIRE_NUM)) {
	SWAP(_rdWire,     rdWire[wireNo], rdWire[WIRE_NUM]);
    }
    free_wire(WIRE_NUM, 0);
    FREE(rdWire[WIRE_NUM].rect);
    FREE(rdWire[WIRE_NUM].side);
    AddNetListItem(netNo);
    
    /*   Delete floating terms for this wire.  */
    ForAllPins(i)
      if(pins[i].type == TERM_DUMMY_TYPE)
	if(pins[i].netNo == netNo)
	  delete_pin(i--);
}
delete_pin(pinNo)     /*   PIN    */
int pinNo;            /*  Pin index  */
{
    int  i, j;

  /*  First delete the equival references to this pin.  */
    ForAllPins(i)
      if((pins[i].equivNo == pinNo) && (i != pinNo)) {
	  for(j=i ; i<PIN_NUM ; i++)
	    if(pins[i].equivNo == pinNo)
	      pins[i].equivNo = j;
	  break;
      }

    /*  ------------------------------------------------------------
     *  Next change the references to the last pin, and bring it to
     *  this position.
     */
    if(pinNo < (--PIN_NUM)) {
	for(i=0 ; i<=PIN_NUM ; i++)
	  if(pins[i].equivNo == PIN_NUM)
	    pins[i].equivNo = pinNo;
	pins[pinNo] = pins[PIN_NUM];
    }
}
delete_rect(rectNo)    /*   RECT   */
int rectNo;            /*   Rect index   */
{
    Rect r, ra, rn;
    int i, j;

    r  = rects+rectNo;
    ra = rects+(--TOTAL_RECT_NUM);
    if(r == ra) {
	six_sides(i) {
	    j = (-i);
	    if((rn=neighbor(r, i)) != NULL)
	      neighbor(rn, j) = NULL;
	}
	return;
    }

    rdCopyRect( r, ra );
    six_sides(i) {
	j = (-i);
	if((rn=neighbor(r, i)) != NULL)   neighbor(rn, j) = NULL;
	rn = neighbor(ra,i);
	neighbor(r, i) = rn;
	if(rn != NULL)                    neighbor(rn, j) = r;
    }
}
