/*      divide.c
 *
 *      Purpose: Create the rectangular subdivision of the routable area.
 *      "DIVIDE UT IMPERA !"
 *      Author: Enrico Malavasi, Univ. of Bologna
 *      Ver. 3.2
 *      Last revised: August 1991
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"

divide()
/*  ============================================================
 *  This function creates the default rectangular subdivision of
 *  the routable area, namely the grid. It takes into account
 *  only the edges of all cells. Further subdivisions, due to
 *  the positions of pins and the size limitation over the grid
 *  elements, will be carried out by <fragment>.
 */
{
    Rect r;
    RoadCoord x1, y1, x2, y2;
    int  i, j;

    /*   Initialization of the <rects> array   */
    amplia_rect(LAYER_NUM);
    TOTAL_RECT_NUM = LAYER_NUM;
    ForAllLayers( i ) {
	r = rects+i;
	rLeftX(r)  = cLeftX(cells);
	rLowerY(r) = cLowerY(cells);
	rRightX(r) = cRightX(cells);
	rUpperY(r) = cUpperY(cells);
	r->layer = i;
	r->free = 0;
	six_sides(j)      neighbor(r, j) = NULL;
	if(i > 0)         set_neighb(r, r-1, UP);
    }

    /*  ------------------------------------------------------------
     *  In case of symmetry, the symmetry buffer rects are created
     */
    if(GetSymmetryFlag()) {
	r = find_rect(cRightX(cells)-1, cLowerY(cells)+1, 0);
	GridSplit(r, cRightX(cells)-1-rLeftX(r), NORTH, NOTFOUND);
    }
    /*  ------------------------------------------------------------
     *  The cells create a fragmentation of the grid structure
     */
    ForAllCells( i, 1 ) {
	x1 = cLeftX(cells+i);
	y1 = cLowerY(cells+i);
	x2 = cRightX(cells+i);
	y2 = cUpperY(cells+i);
	ForAllRects(j) {
	    r = rects+j;
	    if((rLowerY(r) < y2) &&
	       (rUpperY(r) > y1)) {
		if((rLeftX(r)  < x1) &&
		   (rRightX(r) > x1)) {
		    GridSplit(r, x1-rLeftX(r), NORTH, NOTFOUND);
		    j--;
		    continue;
		}
		
		if((rLeftX(r)  < x2) &&
		   (rRightX(r) > x2)) {
		    GridSplit(r, x2-rLeftX(r), NORTH, NOTFOUND);
		    j--;
		    continue;
		}
	    }
	    
	    if((rLeftX(r) < x2) &&
	       (rRightX(r) > x1)) {
		if((rLowerY(r) < y1) &&
		   (rUpperY(r) > y1)) {
		    GridSplit(r, y1-rLowerY(r), EAST, NOTFOUND);
		    j--;
		    continue;
		}
		
		if((rLowerY(r) < y2) &&
		   (rUpperY(r) > y2)) {
		    GridSplit(r, y2-rLowerY(r), EAST, NOTFOUND);
		    j--;
		    continue;
		}
	    }
	}

	ForAllRects(j) {
	    if(rects[j].layer == cells[i].layer)
	      if((rLeftX(rects+j)  >= x1) &&
		 (rRightX(rects+j) <= x2) &&
		 (rLowerY(rects+j) >= y1) &&
		 (rUpperY(rects+j) <= y2))
		delete_rect(j--);
	}
    }
}
