/*      dump.c
 *
 *	Purpose: Read and write the RDF, where the system status is dumped
 *	Author: Enrico Malavasi
 *	Ver. 3.20
 *	Last revised:  August 1991
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"

/* DumpBlock is used to build the RDF file name for writing:
 * DumpBlock == 0  ==> A new file is created;
 * DumpBlock != 0  ==> The last version is superseded. */
extern int   DumpBlock;

static  int    _Int[20];
static  FILE   *Fin, *Fout;

#define  DEFAULT_RESVAL   0.5
#define  DEFAULT_CAPVAL   0.0
#define  WriteInt(N)     fwrite(_Int, sizeof(int), N, Fout)
#define  WriteString(s)  fwrite(s,sizeof(char),strlen(s)+1,Fout)
#define  ReadInt(N)      fread(_Int, sizeof(int), N, Fin)

static void ReadString(s)
char *s;
{
  int i;

  for(i=0 ; (s[i]=fgetc(Fin))!='\0' ; i++)  ;
}

RoadDump( outputFile )
char *outputFile;
/*  Write the RDF of the current circuit. */
{
    int   i, j;
    int   VersNo;
    float alpha;
    
    sscanf(ROAD_VERSION, "%f", &alpha);
    VersNo = alpha * 100;    VersNo /= 100;

    initMessage("Writing dump file %s", outputFile);
    if((Fout=fopen(outputFile, "w")) == NULL)
      errMessage("Cannot write the dump file %s", outputFile);
/*  ------------------------------------------------------------
 *  Global parameters
 */
  _Int[0] = TOTAL_RECT_NUM;
  _Int[1] = SUB_CELL_NUM + CELL_MAX*VersNo;  /* Encode the version number */
  _Int[2] = INST_NUM;
  _Int[3] = PIN_NUM;
  _Int[4] = WIRE_NUM;
  _Int[5] = NET_NUM;
  _Int[6] = (GetSymmetryFlag()) ? GEN_SYMM : 0;
  WriteInt(7);

/*  ------------------------------------------------------------
 *  Dump the rect grid
 */
  ForAllRects(i) {
    _Int[0] = rects[i].x1;
    _Int[1] = rects[i].y1;
    _Int[2] = rects[i].x2;
    _Int[3] = rects[i].y2;
    for(j=0 ; j<6 ; j++)
      _Int[j+4] = ((rects[i].adjacent[j] != NULL) ?
		   rects[i].adjacent[j]-rects : -1);
    _Int[10] = rects[i].layer;
    WriteInt(11);
  }

/*  ------------------------------------------------------------
 *  Dump the cells
 */
  for(i=0 ; i<=SUB_CELL_NUM ; i++) {
    _Int[0] = cells[i].x1;
    _Int[1] = cells[i].y1;
    _Int[2] = cells[i].x2;
    _Int[3] = cells[i].y2;
    _Int[4] = cells[i].layer;
    _Int[5] = cells[i].instanceNo;
    WriteInt(6);
  }

/*  ------------------------------------------------------------
 *  Dump the instances
 */
  for(i=0 ; i<INST_NUM ; i++) {
    WriteString(insts[i].name);
    WriteString(insts[i].master);
    WriteString(insts[i].view);
    _Int[0] = insts[i].x;
    _Int[1] = insts[i].y;
    _Int[2] = insts[i].transf;
    WriteInt(3);
  }

/*  ------------------------------------------------------------
 *  Dump the pins
 */
  ForAllPins(i) {
    _Int[0] = pins[i].netNo;
    _Int[1] = pins[i].layer;
    _Int[2] = pins[i].side;
    _Int[3] = pins[i].position;
    _Int[4] = pins[i].rect-rects;
    _Int[5] = pins[i].cellNo;
    _Int[6] = pins[i].equivNo;
    for(j=0 ; j<pins[i].type ; j++)
      _Int[5] += (SUB_CELL_NUM+1);
    WriteInt(7);
    WriteString(pins[i].TermName);
  }

/*  ------------------------------------------------------------
 *  Dump the wire paths
 */
    ForAllWires( i ) {
	_Int[0] = wireNetNo( i );
	_Int[1] = wireDesc( i );
	j = 2;
	if(wireDesc(i) == DEFWWIDTH)
	  _Int[j++] = wSegmWidth(i, 0);
	WriteInt(j);
	for(j=0 ; j<wireSize(i) ; j++) {
	    _Int[0] = wSegmRect(i, j)-rects;
	    _Int[1] = wSegmSide(i, j);
	    WriteInt(2);
	}
	_Int[0] = (-1);
	WriteInt(2);
    }

/*  ------------------------------------------------------------
 *  Dump the rd_net
 */
  for(i=0 ; i<NET_NUM ; i++) {
    WriteString(rd_net[i].name);
  }
  fclose(Fout);
}

ReadRoadDump( inputFile )
char *inputFile;          /*  Circuit name  */
/* Read the RDF of the current circuit.  */
{
    int     i, j, k, VersNo;
    int     WireNo;
    char    name[128];

    if((Fin=fopen(inputFile, "r")) == NULL)
      errMessage("Cannot read the dump file %s", inputFile);
    initMessage("Reading dump file %s ... ", inputFile);

    /*  Read and examine the input string   */
    ReadInt(7);
    TOTAL_RECT_NUM = RECT_ALLOC = 0;
    rects = NULL;
    amplia_rect(_Int[0]);
    TOTAL_RECT_NUM = _Int[0];
    SUB_CELL_NUM   = _Int[1];
    INST_NUM       = _Int[2];
    PIN_NUM        = _Int[3];
    WIRE_NUM       = _Int[4];
    NET_NUM       = _Int[5];
    if(WIRE_NUM > 0)  RD_RESET_FLAG( GEN_SYMM );
    if(_Int[6] & GEN_SYMM)   SetSymmetryFlag( 1 );
    for(VersNo=0 ; SUB_CELL_NUM>=CELL_MAX ; VersNo++)
      SUB_CELL_NUM -= CELL_MAX;
    
    cells      = ALLOC(Cell, SUB_CELL_NUM+1);
    insts      = ALLOC(RoadInstance, INST_NUM);
    pins       = ALLOC(RoadTerm, PIN_NUM);
    rdWire     = ALLOC(_rdWire, NET_NUM*3);
    rd_net     = ALLOC(RoadNet, NET_NUM);
    
    initMessage("Reading the grid frame ... ", 0);
    ForAllRects(i) {
	ReadInt(11);
	rects[i].x1 = _Int[0];
	rects[i].y1 = _Int[1];
	rects[i].x2 = _Int[2];
	rects[i].y2 = _Int[3];
	for(j = 0; j < 6; j++) {
	    k = 4+j;
	    rects[i].adjacent[j] = (_Int[k] < 0) ? NULL :  rects+_Int[k];
	}
	rects[i].layer = _Int[10];
	rects[i].flag = 0;
    }
    contMessage("Done! %d grid nodes read", TOTAL_RECT_NUM);

  initMessage("Reading the obstacle list  ... ", 0);
  for(i=0 ; i<=SUB_CELL_NUM ; i++) {
    ReadInt(6);
    cells[i].x1 = _Int[0];
    cells[i].y1 = _Int[1];
    cells[i].x2 = _Int[2];
    cells[i].y2 = _Int[3];
    cells[i].layer      = _Int[4];
    cells[i].instanceNo = _Int[5];
  }
  contMessage("Done! %d obstacles read", SUB_CELL_NUM);

  initMessage("Reading the instances  ... ", 0);
  for(i=0 ; i<INST_NUM ; i++) {
    ReadString(name);
    insts[i].name = util_strsav(   name );
    ReadString(name);
    insts[i].master = util_strsav( name );
    ReadString(name);
    insts[i].view = util_strsav(   name );
    ReadInt(3);
    insts[i].x =     _Int[0];
    insts[i].y =     _Int[1];
    insts[i].transf= _Int[2];
  }
  contMessage("Done! %d instances read", INST_NUM);

  initMessage("Reading the pin list   ... ", 0);
  ForAllPins(i) {
    ReadInt(7);
    pins[i].netNo    = _Int[0];
    pins[i].layer    = _Int[1];
    pins[i].side     = _Int[2];
    pins[i].position = _Int[3];
    pins[i].rect     = (_Int[4] < 0) ? NULL : rects+_Int[4];
    if(pins[i].rect != NULL)  (pins[i].rect)->flag |= RECTFLAG_TERM;
    pins[i].cellNo   = _Int[5];
    pins[i].equivNo  = _Int[6];
    ReadString(name);
    pins[i].TermName = util_strsav( name );
    for(pins[i].type=0 ; pins[i].cellNo>(SUB_CELL_NUM+1) ; pins[i].type++)
      pins[i].cellNo -= (SUB_CELL_NUM+1);
  }
  contMessage("Done! %d pins read", PIN_NUM);

    initMessage("Reading the wire list  ... ", 0);
    ForAllWires( i ) {
	ReadInt(1);
	wireSize( i ) = 0;
	wireNetNo(i)  = _Int[0];
	wireDesc(i)   = MINWWIDTH;
	
	/*  VERSION 2: Wire widths have been introduced  */
	if(VersNo >= 2) {
	    ReadInt(1);
	    wireDesc(i) = _Int[0];
	    if(wireDesc(i) == DEFWWIDTH) {
		rdWire[i].wWidth = ALLOC(RoadCoord, 1);
		ReadInt(1);
		wSegmWidth(i, 0) = _Int[0];
	    }
	}
	/*  Version 2 ============================== */
	for(j=0 ; ; j++) {
	    ReadInt(2);
	    if(_Int[0] < 0)  break;
	    add_to_wire(i, rects+_Int[0], _Int[1]);
	}
	wireFree(i) = 0;
	free_wire(i, 1);
    }
    contMessage("Done! %d wires read", WIRE_NUM);
    
    initMessage("Reading the net list  ...  ", 0);
    ForAllNets( i ) {
	if(VersNo < 3) {
	    ReadInt(1);
	}
	ReadString(name);
	rd_net[i].name = util_strsav( name );
    }
    
    /*  ------------------------------------------------------------
     *  Set the net fields to their default values
     */
    ForAllNets( i ) {
	rd_net[i].resistive_weight = DEFAULT_RESVAL;
	rd_net[i].capacitive_weight= DEFAULT_CAPVAL;
	rd_net[i].bulk_shld        = NOTFOUND;
	rd_net[i].link             = NO;
    }
    contMessage("Done! %d nets read\n", NET_NUM);
    fclose(Fin);
}
