/*    enlarge.c
 *
 *    Author: Enrico Malavasi,  Univ. of Bologna, Italy
 *    Ver. 1.00      -      September, 1989
 *
 *    Purpose: apply the micro-modification to a wire element
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"
#include "space.h"

typedef struct _wirePair {
    int wireNo;
    int wirePos;
} _wirePair;

static _wirePair
  OTM(wn, pos, dir, room)   /* Obstacle To Move */
int       wn, pos, dir;
RoadCoord room;
{
    Rect r0,     /*  Old proposal */
         r,
         s;      /*  Span  */
    int  cnt=0, i,
         ly, side;
    RoadCoord ss;
    RoadBox   BB, bUp;
    _wirePair   Obst;

    Obst.wireNo = NOTFOUND;
    if(room <= 0)   return(Obst);
    r = r0 = wSegmRect(wn, pos);
    side   = wSegmSide(wn, pos);
    ly = r0->layer;
    ss = spacingBetween(r0->layer, side, r0->layer, NORTH);
    WireSegmentToBox(r0, side, ss,  &BB, &bUp, dir, dir);
    s = (dir > 0) ? neighbor(r0, dir) : r0;
    if(s == NULL)    return(Obst);

    loop {
	if((ss=rect_size(s, dir)) > room) {
	    ShiftBox(&BB, dir, room);
	    if(side == UP) ShiftBox(&bUp, dir, room);
	    break;
	}
	r = neighbor(r0, dir);
	if(r == NULL) {
	    Obst.wireNo = WIRE_NUM;
	    return(Obst);
	}
	unless(XXXavailable(wn, r, side)) {
	    for(i=0 ; i<WIRE_NUM ; i++)
	      if(i != wn)
		if((pos=wire_obstacle(i, r, side, abs(dir))) != NOTFOUND) {
		    Obst.wireNo  = i;
		    Obst.wirePos = pos;
		    return(Obst);
		}
	    Obst.wireNo = WIRE_NUM;
	    return(Obst);
	}
	r0 = r;
	ShiftBox(&BB, dir, ss);
	if(side == UP) ShiftBox(&bUp, dir, ss);
	cnt++;
	room -= ss;
	s = neighbor(s, dir);
    }
    
    if((ss=room) == 0) {
	s = neighbor(s, (-dir));
	ss = rect_size(s, dir);
    }
    loop {
	Obst.wireNo = newAvailableBox(wn, &BB, &bUp, AbsDualDir(dir), AbsDualDir(dir), s, side);
	if(Obst.wireNo != NOTFOUND) break;
	if(cnt == 0)  break;
	ShiftBox(&BB, -dir, ss);
	if(side == UP) ShiftBox(&bUp, -dir, ss);
	s = neighbor(s, (-dir));
	ss = rect_size(s, dir);
	cnt--;
    }
    Obst.wirePos = common;
    return  Obst;
}

RoadCoord
  TryToEnlarge(wn, wPos, theDir, nddwdt)
int    wn, wPos;
int    theDir;       /*  Translation versus (theta) */
RoadCoord  nddwdt;     /*  Translation module (rho)   */
{
    Rect   r;
    Pins   p;
    int    i;
    int    wi, pos, side, adir;
    int        nr;
    RoadCoord  wdt, eff,
               effect;
    _wirePair  Obst, OTM();

    if(nddwdt <= 0)                         return((RoadCoord) 0);
    r      = wSegmRect(wn, wPos);
    if(r == NULL)                           return((RoadCoord) 0);
    side   = wSegmSide(wn, wPos);
    adir = abs(theDir);
    if(side != UP) {
	if(side != abs(theDir))         return((RoadCoord) 0);
	if(((p=PinIsIn(r, wireNetNo(wn)))) &&
	   (abs(p->side) != side))
	  return((RoadCoord) 0);
    }

    nr = r-rects;
    effect = wire_shift(wn, wPos, theDir, nddwdt);
    if(effect > 0)        spAddModList(wn);
    if(effect >= nddwdt)  return(effect);
    r = rects+nr;
    nddwdt -= effect;

    if(effect > 0) /* I have lost control over the wire. */
      return(effect);

    while((wPos=wire_segment_position(wn, r, side, NOTFOUND)) == NOTFOUND)
      r = neighbor(r, adir);
    
    /*  ============================================================
     *  Now find the obstacle and, if possible, move it away
     */
    Obst = OTM(wn, wPos, theDir, nddwdt);
    if(Obst.wireNo >= WIRE_NUM)    return(effect);  /* It is a cell */
    if(Obst.wireNo == NOTFOUND)    return(effect);  /*  something strange  */
    wi  = Obst.wireNo;
    pos = Obst.wirePos;
    
    switch(theDir) {
      case NORTH: wdt = wSegmRect(wi, pos)->y2  - wSegmRect(wn, wPos)->y2; break;
      case SOUTH: wdt = wSegmRect(wn, wPos)->y2 - wSegmRect(wi, pos)->y2;  break;
      case EAST : wdt = wSegmRect(wi, pos)->x2  - wSegmRect(wn, wPos)->x2; break;
      case WEST : wdt = wSegmRect(wn, wPos)->x2 - wSegmRect(wi, pos)->x2;  break;
    }
    if(wdt <= 1)  return(effect);
    wdt = nddwdt - wdt +
      spacingBetween(wSegmLayer(wn, wPos), wSegmSide(wn, wPos),
		     wSegmLayer(wi, pos),  wSegmSide(wi, pos)) +
		       additionalSpacing(wn, wi);
    
    CONDfree_wire(wn, 1);
    free_wire(wi, 0);
    eff = TryToEnlarge(wi, pos, theDir, wdt);
    free_wire(wi, 1);
    CONDfree_wire(wn, 0);
    
    if(eff == 0)  return(effect);
    else          return(effect + TryToEnlarge(wn, wPos, theDir, nddwdt));
}
