/*  graph.c
 *
 *	by Enrico Malavasi
 *      Ver. 3.2
 * 	Last revised: September 1991
 *
 *	Purpose: Graphic initialization and utilities
 */
#include "port.h"
#include "utility.h"
#include "graph.h"
#include "roadTechnol.h"

#define   SetThisColor(Elem) \
{  XAllocNamedColor(&MAINDisplay, MAINColormap, ccc, &ThisCol, &ExactCol);\
   Elem = ThisCol.pixel; }

defaults()
{
    char    ccc[10];
    int     i;
    XColor  ThisCol, ExactCol;

    ZX0 = ZY0 = 0;
    ZX1 = ZY1 = 750;
    RoadZoom(0);

    color_set.layer_color = ALLOC(RoadColor, LAYER_NUM);
    strcpy(ccc, "LightGrey"); SetThisColor(color_set.BackGround);
    strcpy(ccc, "yellow"); SetThisColor(color_set.HighLight);
    strcpy(ccc, "black"); SetThisColor(color_set.Obstacles);
    
    for(i=0 ; i<LAYER_NUM ; i++) {
	switch(layers[i].type) {
	  case TYPE_DIFF: strcpy(ccc, "green"); break;
	  case TYPE_POLY: strcpy(ccc, "red"); break;
	  case TYPE_MET1: strcpy(ccc, "cyan"); break;
	  case TYPE_POL2: strcpy(ccc, "orange"); break;
	  case TYPE_MET2: strcpy(ccc, "BlueViolet"); break;
	}
	
	SetThisColor(color_set.layer_color[i]);
    }
}

#define   MINDISCRIM   5
RoadZoom(i)
int i;      /*  Zoom in if it is set, Full screen else  */
{
    RoadCoord  x1, y1, x2, y2;

    if(i) {
	get_point(&x1, &y1);
	get_point(&x2, &y2);   /*  Returns uncorrected RoadCoords  */
	SORT(RoadCoord, x1,x2);
	SORT(RoadCoord, y1,y2);
	x2 -= x1;   /*  Diffs  */
	y2 -= y1;
	if((x2 < MINDISCRIM) || (y2 < MINDISCRIM)) return;
	if(x2 > y2) {
	    y1 -= ((x2-y2)/2);
	    y2 = x2;
	}
	if(x2 < y2) {
	    x1 -= ((y2-x2)/2);
	    x2 = y2;
	}

	if(x1 < cells[0].x1)    x1 += (cells[0].x1 - x1);
	if(y1 < cells[0].y1)    y1 += (cells[0].y1 - y1);

	rdView.x1 = x1;
	rdView.y1 = y1;
	rdView.x2 = x1+x2;
	rdView.y2 = y1+y2;
    } else {
	rdView = cells[0];
    }
    ZDX = (ZX1 - ZX0);
    ZDX /=  (rdView.x2 - rdView.x1);
    ZDY = (ZY1 - ZY0);
    ZDY /= (rdView.y2 - rdView.y1);
    ZDX = min(ZDX, ZDY);
    ZDY = ( ZDX);
}

RoadPan()
{
  RoadCoord  x1, y1;

  get_point(&x1, &y1);
  
}

#define   CORRECT_ALL  { x1 = Xcorrect(x1); x2 = Xcorrect(x2); \
                         y1 = Ycorrect(y1); y2 = Ycorrect(y2); }

static int       current_LineWidth = 1;

set_line_width(lw)
int lw;
{
  current_LineWidth = ((lw <= 0) ? 0 : (lw*ZDX));
  XSetLineAttributes(&MAINDisplay, MAINGc,
                     current_LineWidth, LineSolid, CapProjecting, JoinMiter);
}

/*  ============================================================
 *  Draw a segment with the current color.
 */
segment(x1, y1, x2, y2)
RoadCoord x1, y1, x2, y2;
{
  CORRECT_ALL;
  if((x1 != x2) || (y1 != y2))
    XDrawLine(&MAINDisplay,MAINWindow,MAINGc,x1, y1, x2, y2);
}

dashed_segment(x1, y1, x2, y2)
RoadCoord x1, y1, x2, y2;
{
  XSetLineAttributes(&MAINDisplay, MAINGc,
		     current_LineWidth, LineOnOffDash, CapButt, JoinMiter);
  segment(x1, y1, x2, y2);
  XSetLineAttributes(&MAINDisplay, MAINGc,
                     current_LineWidth, LineSolid, CapButt, JoinMiter);
}

/*  ============================================================
 *  Draw an empty box with the current color.
 */
box(x1,y1,x2,y2)
RoadCoord x1,y1,x2,y2;
{
  if(x1 > x2)  SWAP(RoadCoord, x1, x2);
  if(y1 > y2)  SWAP(RoadCoord, y1, y2);
  CORRECT_ALL;
  x2 -= x1;
  y1 -= y2;

  XDrawRectangle(&MAINDisplay, MAINWindow, MAINGc, x1, y2, x2, y1);
}

/*  ------------------------------------------------------------
 *  Filled box.
 */
filled_box(x1,y1, x2, y2, c)
RoadCoord x1,y1,x2,y2;
int c;
{
  SORT(RoadCoord, x1, x2);
  SORT(RoadCoord, y1, y2);
  CORRECT_ALL;
  x2 -= x1;
  y1 -= y2;
  set_color(c);
  XFillRectangle(&MAINDisplay, MAINWindow, MAINGc, x1, y2, x2, y1);
}

/*  ============================================================
 *  This function writes a string in a given position, on a given
 *  window
 */
testo(x, y, s, w)
int    x, y;
char  *s;
Window w;
{
  if(w == MAINWindow)
    XDrawString(&MAINDisplay, w, MAINGc, x, y, s, strlen(s));
  else
    XDrawImageString(&MAINDisplay, w, PARAGc, x, y, s, strlen(s));
}

/*  ==================================================================
 *  Read a pair of coordinates relative to a point on the main window.
 */
get_point(x, y)
RoadCoord *x, *y;
{
  XEvent ev;

  while(! XCheckTypedWindowEvent(&MAINDisplay, MAINWindow, ButtonPress, &ev))  ;
  get_coordinates(x, y);
  return;
}

get_coordinates(x, y)
RoadCoord *x, *y;
{
  Window       rw, cw;
  int          rx, ry;
  unsigned int mask;

  XQueryPointer(&MAINDisplay, MAINWindow, &rw, &cw, &rx, &ry, x, y, &mask);
  RoadSync(1);

  (*x) = Xuncorrect(*x);
  (*y) = Yuncorrect(*y);
}
