/*	inpobst.c
 *
 *	Author: Enrico Malavasi, Univ. of Bologna
 *      Ver. 3.2
 * 	Last revised: August 1991
 *
 * 	Purpose: Read the data about the circuit. Some of these
 *               functions are public because must be accessed
 *               by the interface routines.
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"
#include "transf.h"

static void
  addObstacle(instNo)
int  instNo;
{
    RoadBox  BB, bbmin;
    int  i, j, dir;
    int  l;
    Rect      r;
    RoadTerm *p;

    bbmin.x1 = bbmin.y1 = 0;
    bbmin.x2 = bbmin.y2 = 0;

    ForAllLayers( l ) {
    
	BB.x1 = BB.y1 = 100000;
	BB.x2 = BB.y2 = -100000;
    
	ForAllCells( i, 1 ) {
	    if((cells[i].instanceNo == instNo) && (cells[i].layer == l)) {
		BB.x1 = min(BB.x1, rdLeftX(cells+i));
		BB.x2 = max(BB.x2, rdRightX(cells+i));
		BB.y1 = min(BB.y1, rdLowerY(cells+i));
		BB.y2 = max(BB.y2, rdUpperY(cells+i));
	    }
	}
	
	if( BB.x1 >= BB.x2 ) continue;
	if( bbmin.x1 >= bbmin.x2) {
	    bbmin = BB;
	} else {
	    bbmin.x1 = max(bbmin.x1, BB.x1);
	    bbmin.y1 = max(bbmin.y1, BB.y1);
	    bbmin.x2 = min(bbmin.x2, BB.x2);
	    bbmin.y2 = min(bbmin.y2, BB.y2);
	}
    }
    
    if(bbmin.x1 >= bbmin.x2)   return;
    ForAllRects(i) {
	r = rects+i;
	if((rdLeftX(r)  >= bbmin.x1)  &&
	   (rdRightX(r) <= bbmin.x2) &&
	   (rdLowerY(r) >= bbmin.y1) &&
	   (rdUpperY(r) <= bbmin.y2)) {
	    ForAllPins( j ) {
		if(pins[j].rect == r) {
		    VERBOSE (printf("Deleting terminal of net %s\n",
				    rd_net[pins[j].netNo].name));
		    delete_pin(j--);
		}
	    }
	    delete_rect(i--);
	}
    }
}

void
roadGeneralCheck()
{
    int i;

    if(WIRE_NUM > 0)      RD_RESET_FLAG( GEN_SYMM );

    for(i=0 ; i<INST_NUM ; i++)
      if(insts[i].noov == YES)
	addObstacle(i);
    
    ForAllPins( i )
      if(pins[i].rect == NULL)
	if((pins[i].rect=find_rect(pin_x(pins+i), pin_y(pins+i), rdLayer(pins+i))) == NULL)
	  delete_pin( i-- );
    
    fusePins();
    
    if(RD_CHECK_FLAG( GEN_SYMM ))    look_for_symmetry();
    CreateNetList();
    CreateCongestionAreas();
    CreatePinSheetList();
    
    ForAllWires(i) {
	if(wireSize(i) < 1) {
	    delete_wire(i);
	}
    }
}
