/*      ioutil.c
 *
 *      Utilities for (non sophisticated) file parsing.
 *      The items read are returned converted to upper case.
 *
 *      By Enrico Malavasi,  Univ. of Bologna, Italy
 *      Last revised:  March, 1990
 */
#include  "port.h"
#include  <string.h>
#include  "utility.h"

/*  Word terminator */
#define   SPACE(c)    (((c)==' ')||((c)=='\t'))

/*  ============================================================
 *  This function reads the next line from a file (already open
 *  for reading) and returns the number of items (words) found
 *  in <argc>, and the items themselves in <argv>.
 *  <argv> must already be properly allocated.
 *  Empty lines are skipped
 */
inputCommandLine(fil, argc, argv)
FILE  *fil;
int   *argc;
char **argv;
{
  char  Line[256];
  int   i, j;

/*  ------------------------------------------------------------
 *  Read the line
 */
  for(i=0 ; (Line[i]=fgetc(fil))!='\n' ; i++)
    if(Line[i] == EOF)     break;

  for((*argc)=0,j=0 ; ; ) {
    while(SPACE(Line[j])) j++;    /*  Remove the spaces  */
    if(Line[j] == EOF)       return;
    if(Line[j] == '\n') {
      if((*argc) > 0)        return;
      inputCommandLine(fil, argc, argv);
      return;
    }
    for(i=0 ; (! SPACE(Line[j])) ; ) {
      if(Line[j] == EOF)         break;
      if(Line[j] == '\n')        break;
      argv[*argc][i++] = Line[j++];
    }
    argv[*argc][i] = '\0';
    (*argc)++;
  }
}

/*  ======================================================================
 *  This function returns the next unread item (word) found in a
 *  file (already open for reading). If no new items are found,
 *  NULL is returned.
 */
#define   MAXCOMMNUM      20     /* Max number of commands on a line */
#define   MAXCOMMLEN      20     /* Max length of a command in chars */
char *NextCommand(f)
FILE *f;
{
    int         i;
    static int  argc = 0;
    static int  pnt = 0;                /*  Number of elements read  */
    static char **argv = NULL;

    if(argv == NULL) {
	argv = ALLOC(char *, MAXCOMMNUM);
	for(i=0 ; i<MAXCOMMNUM ; i++)
	  argv[i] = ALLOC(char, MAXCOMMLEN);
    }
    while( pnt>=argc ) {
	inputCommandLine(f, &argc, argv);
	if(argc <= 0)  {
	    for(i=0 ; i<MAXCOMMNUM ; i++)
	      FREE(argv[i]);
	    FREE(argv);
	    argv = NULL;
	    return NULL;
	}
	pnt = 0;
	if(argv[0][0] == '#') argc = 0;
    }

    return argv[pnt++];
}

/*  ============================================================
 *  String case change functions
 */
StringToLower(s)
char *s;
{
  int i;
  for(i=0 ; s[i]!='\0' ; i++)
    s[i] = tolower(s[i]);
}
StringToUpper(s)
char *s;
{
  int i;
  for(i=0 ; s[i]!='\0' ; i++)
    s[i] = toupper(s[i]);
}
