/*       road.c
 *
 *       Author: Enrico Malavasi
 *       Ver.  : 3.10
 *       Last revised:  April, 1991
 *
 *       Purpose: main program for ROAD: A ROuter for Analog Design.
 */
#define   FIRSTDEF  
#include "port.h"
#include "oct.h"
#include "error.h"
#include "utility.h"
#include "options.h"
#include "vov.h"
#include "road.h"
#include "roadGlobals.h"
#include "roadTechnol.h"

optionStruct optionList[] = {
    {"c", "file",               "Only report cap-coupling and exit."},
    {"D", "",                   "DEBUG option."},
    {"f", "",                   "Batch mode (Def. interactive)."},
    {"h", "",                   "Print this message."},
    {"i", "rdf-file",           "Name of the input file."},
    {"n", "",                   "Ignore the NDF file."},
    {"s", "script-file",        "Read and execute a script."},
    {"S", "",                   "Disable symmetry check."},
    {"T", "tech_file",          "Technology file."},
    {"v", "",                   "Verbose mode."},
    {"V", "",                   "Graphic verbose mode."},
    { OPT_RARG,	"circuit",	"Circuit name."},
    { 0, 0, 0 }
};

#define   NDF_EXTENSION   "ndf"

main(argc,argv)
int argc;
char **argv;
{
    char   *reportFile  = NULL;
    char   *circuitName = NULL;
    char   *inputFile   = NULL;
    char   *scriptFile  = NULL;
    char   *technologyFile;
    char option;
    
    fprintf(stderr, "RoAD: a Router for Analog Design");
    fprintf(stderr, "\nVersion %s\n", ROAD_VERSION);

    RD_RESET_ALLFLAGS;
    RD_SET_FLAG(GEN_SYMM);
    RD_SET_FLAG(NET_READ);
    technologyFile = util_strsav( TECHNOLFILE );
    while ( ( option = optGetOption (argc, argv)) != EOF) {
	switch( option ) {
	  case 'D':
	    RD_SET_FLAG(DEBUG_OPTION);     break;
	  case 'c':
	    reportFile = util_strsav( optarg );               break;
	  case 'f':
	    RD_SET_FLAG(BATCH_MODE);         break;
	  case 'n':
	    RD_RESET_FLAG(NET_READ);       break;
	  case 'S':
	    RD_RESET_FLAG(GEN_SYMM);       break;
	  case 'v':
	    RD_SET_FLAG(RD_VERBOSE_FLAG);            break;
	  case 'V':
	    RD_SET_FLAG(GRAPH_VERB);            break;
	  case 'i':
	    inputFile = util_strsav( optarg );       break;
	  case 'T':
	    technologyFile = util_strsav( optarg );  break;
	  case 's':
	    RD_SET_FLAG(BATCH_MODE);
	    scriptFile = util_strsav( optarg );      break;
	  default:
	    optUsage();                       break;
	}
    }

    if(optind != (argc-1)) optUsage();
    circuitName     = util_strsav( argv[optind] );
    if( inputFile == NULL)   inputFile = LastDumpFileName( circuitName );
    if(RD_CHECK_FLAG( BATCH_MODE )) RD_RESET_FLAG( GRAPH_VERB );

    VOVbegin( argc, argv );    
    VOVinput( VOV_UNIX_FILE, inputFile );
    ReadRoadDump( inputFile );

    VOVinput( VOV_UNIX_FILE, technologyFile );
    read_technol( technologyFile );

    {   char  *ndfFile;
	ndfFile = ALLOC( char, strlen(circuitName) + strlen(NDF_EXTENSION) + 2);
	sprintf(ndfFile, "%s.%s", circuitName, NDF_EXTENSION);
	if( input_netlist( ndfFile ))  VOVinput( VOV_UNIX_FILE, ndfFile );
    }
    roadGeneralCheck();
    
    if( reportFile ) {
	VOVoutput( VOV_UNIX_FILE, reportFile );
	reportCoupling( reportFile );
	rdExit( 0 );
    }
    
    /*   Enter the routing session (batch or interactive    */
    if(RD_CHECK_FLAG(BATCH_MODE)) {
	SetDefaultScript();
	if( scriptFile ) {
	    VOVinput( VOV_UNIX_FILE, scriptFile );
	    read_script( scriptFile );
	}
	ExecuteScript( circuitName );
    } else {
	rdGraphicInit( argc, argv );    /* Graphic initialization */
	roadExecInteractive( circuitName );
    }
    
    rdExit( 0 );
}

/*  ============================================================
 *  This function provides a controlled exit from ROAD. It should be
 *  used instead of <exit>
 */
rdExit( s )
int s;
{
    if( s == 0 ) {
	rd_Statistics(stdout);
	printf("\n"); fflush(stdout);
    }
    VOVend( s );
}
