/*  	neighb.c
 *
 * 	Purpose: Utilities connected to the adjacency relation
 *               of rectangles.
 *	Author: Enrico Malavasi
 *      Ver. 3.20
 *      Last revised: August 1991
 */
#include "port.h"
#include "utility.h"
#include "road.h"
#include "roadGlobals.h"
#include "roadMacros.h"

set_neighb(r, l, side)
Rect r, l;           /* Rect <r> is put on the <side> side of Rect <l> */
int side;
/*  Set the neighbor relationship between rect/s
 *  <r> and <l>, so that <r> is set on the <side> side of <l>.
 */
{
    if(l != NULL) neighbor(l,side) = r;
    side = (- side);
    if(r != NULL) neighbor(r,side) = l;
}

Rect
  free_neighbor(r,dir)
Rect r;          /*  Given rectangle */
int dir;         /*  Given side      */
/*  Return the pointer to the neighbor of a given rectangle on a given
 *  side, if it exists AND the corresponding side is free.
 */
{
    switch(dir) {
      case NORTH : return(freeSide(r, FREE_NORTH) ? NULL : neighbor(r,dir));
      case EAST  : return(freeSide(r, FREE_EAST)  ? NULL : neighbor(r,dir));
      case WEST  : return(freeSide(r, FREE_WEST)  ? NULL : neighbor(r,dir));
      case SOUTH : return(freeSide(r, FREE_SOUTH) ? NULL : neighbor(r,dir));
      default    : return(neighbor(r, dir));
    }
}

set_free_neighbor(r, dir, f)
Rect r;         /* Given rectangle */
int  dir;       /* Given side      */
int  f;         /* flag            */
/*  Set the <free> field of the <r> rectangle in the <dir> side
 *    FREE,    if <f>==0 ,              NOT-FREE if <f>!=0
 */
{
    Rect s;

    if((s=neighbor(r,dir)) == NULL) return;
    if(f)
      switch(dir) {
	case NORTH : setFreeSide(r, FREE_NORTH); setFreeSide(s, FREE_SOUTH);	return;
	case EAST  : setFreeSide(r, FREE_EAST);  setFreeSide(s, FREE_WEST);	return;
	case WEST  : setFreeSide(r, FREE_WEST);  setFreeSide(s, FREE_EAST);	return;
	case SOUTH : setFreeSide(r, FREE_SOUTH); setFreeSide(s, FREE_NORTH);  return;
	case UP    : setFreeSide(r, FREE_UP);    setFreeSide(s, FREE_DOWN);   return;
	case DOWN  : setFreeSide(r, FREE_DOWN);  setFreeSide(s, FREE_UP);
	default    : return;
      }
    else
      switch(dir) {
	case NORTH : resetFreeSide(r, FREE_NORTH); resetFreeSide(s, FREE_SOUTH); return;
	case EAST  : resetFreeSide(r, FREE_EAST);  resetFreeSide(s, FREE_WEST);  return;
	case WEST  : resetFreeSide(r, FREE_WEST);  resetFreeSide(s, FREE_EAST);  return;
	case SOUTH : resetFreeSide(r, FREE_SOUTH); resetFreeSide(s, FREE_NORTH); return;
	case UP    : resetFreeSide(r, FREE_UP);    resetFreeSide(s, FREE_DOWN);  return;
	case DOWN  : resetFreeSide(r, FREE_DOWN);  resetFreeSide(s, FREE_UP);
        default    : return;
      }
}

/*  ============================================================
 *  Sets the free-neighbor field corresponding to the path of a
 *  given wire.
 */
free_wire(wn, f)
int  wn;        /* Given wire            */
int  f;         /* See set_free_neighbor */
{
    int  i, netNo;

    if(wireFree(wn) != f) {
	wireFree(wn) = f;
	if(f) {
	    netNo = wireNetNo(wn);
	    for(i=0 ; i<wireSize(wn) ; i++) {
		set_free_neighbor(wSegmRect(wn, i), wSegmSide(wn, i), f);
		rNet(wSegmRect(wn, i)) = netNo;
	    }
	} else {
	    for(i=0 ; i<wireSize(wn) ; i++)
	      set_free_neighbor(wSegmRect(wn, i), wSegmSide(wn, i), f);
	}
    }
}
