/*	newrditem.c
 *
 *	Author: Enrico Malavasi
 *      Ver. 3.0
 * 	Last revised: January, 1991
 *
 *	Purpose: Instantiate a new Road item. The functions of this module
 *               are in the form  newRoadItem, where <Item> is the name
 *               of the object to be created.
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"

/*  NEW PIN  */
newRoadPin(netNo, layer, side, position, rect, cellNo, equiv, Tn)
int     netNo;
char    layer;
char    side;
RoadCoord   position;
Rect    rect;
int     cellNo;
int     equiv;
char    *Tn;
{
    if(layer >= LAYER_NUM)
      errMessage("newRoadPin: Terminal defined with wrong layer ref. ", 0);

    pins = REALLOC(RoadTerm, pins, (PIN_NUM+1));
    if(equiv == NOTFOUND) equiv = PIN_NUM;
    pins[PIN_NUM].netNo    = netNo;
    pins[PIN_NUM].layer    = layer;
    pins[PIN_NUM].side     = side;
    pins[PIN_NUM].type     = TERM_FIXED_TYPE;
    pins[PIN_NUM].position = position;
    pins[PIN_NUM].rect     = rect;
    pins[PIN_NUM].cellNo   = cellNo;
    pins[PIN_NUM].equivNo  = equiv;
    pins[PIN_NUM].TermName = util_strsav(Tn);
    strcpy(pins[PIN_NUM].TermName, Tn);
    PIN_NUM++;
}

/*  NEW NET  */
#define   DEF_WWIDTH        0      /*  Default wire width  */
#define   DEF_CAPACIT       0.0    /*  Default capacitive weight  */
#define   DEF_RESIST        0.5    /*  Default resistive weight   */
newRoadNet(name)
char     *name;
{
    int  netNo;
  
    if((name == NULL) || (name[0] == '\0'))
      errMessage("Attempt to use a net with no name", 0);
    if((netNo=name_to_net(name)) != NOTFOUND)
      return(netNo);
    rd_net = REALLOC(RoadNet, rd_net, (NET_NUM+1));
    rd_net[NET_NUM].link              = NO;
    rd_net[NET_NUM].wwidth            = DEF_WWIDTH;
    rd_net[NET_NUM].resistive_weight  = DEF_RESIST;
    rd_net[NET_NUM].capacitive_weight = DEF_CAPACIT;
    rd_net[NET_NUM].bulk_shld         = NOTFOUND;
    rd_net[NET_NUM].name              = NULL;
    rd_net[NET_NUM].name = util_strsav( name );
    return(NET_NUM++);
}

SwapCells(n1, n2)
int  n1,
     n2;
{
  int  i;

  SWAP(Cell, cells[n1], cells[n2]);
  ForAllPins(i) {
    if(pins[i].cellNo == n1)  pins[i].cellNo = (-1-n2);
    if(pins[i].cellNo == n2)  pins[i].cellNo = (-1-n1);
  }
  ForAllPins(i)
    if(pins[i].cellNo < 0) pins[i].cellNo = (-1-pins[i].cellNo);
}

SortCells()
{
  int i,j;

  for(i=1 ; i<=SUB_CELL_NUM ; i++)
    for(j=i+1 ; j<=SUB_CELL_NUM ; j++) {
      if(cells[i].layer < cells[j].layer)  continue;
      if(cells[i].layer == cells[j].layer) {
	if(cells[i].x1 <  cells[j].x1)     continue;
	if((cells[i].x1 == cells[j].x1) &&
	   (cells[i].y1 <  cells[j].y1))   continue;
      }
      SwapCells(i, j);
    }
}

void
  rdCopyRect(new, old)
Rect new;
Rect old;
{
    rLeftX(new)  = rLeftX(old);
    rLowerY(new) = rLowerY(old);
    rRightX(new) = rRightX(old);
    rUpperY(new) = rUpperY(old);
    new->netNo   = old->netNo;
    new->layer   = old->layer;
    new->free    = old->free;
    new->flag    = old->flag;
}
