/*	prune.c
 *
 *	Purpose: Pruning wires
 *	Author:       Enrico Malavasi
 *      Ver.:         3.20
 *      Last revised: August 1991
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"


int
  CompactWire(wn)
int  wn;
/*  Take a wire and bring all its elements containing a pin to the beginning.
 *  Return the total number <tot> of pins found.
 */
{
    int      tot = 0;    /*  Total number of pins found  */
    int      j;          /*  Loop counters  */
    Pins     p;

    for(j=tot ; j<wireSize(wn) ; j++) {
	if(wSegmSide(wn, j) != UP) {
	    if((p=PinIsIn(wSegmRect(wn,j), wireNetNo(wn))) != NULL) {
		if(p->side != wSegmSide(wn,j)) {
		    if(j > tot) {
			SWAP(Rect, wSegmRect(wn, j), wSegmRect(wn, tot));
			SWAP(int,  wSegmSide(wn, j), wSegmSide(wn, tot));
		    }
		    tot++;
		}
	    }
	}
    }
    return tot;
}

#define  FREE_WIRE(W,F)  { ForAllWires(i)                    \
			     if(wireNetNo(i)==wireNetNo(W)) \
			       free_wire(i,F); }
void
  PruneWire(wn)
int  wn;
/*  Prune wires' useless appendixes.  */
{
    int  i, ws, tot;

    tot = CompactWire(wn);
    FREE_WIRE(wn, 0);
    do {
	rdSetWireMark( wn );
	ws = wireSize(wn);
	wSegmRect(wn, ws) = NULL;
	for(i=tot ; i<wireSize(wn) ; i++) {
	    if(! rdCheckWireDMark(wn, i)) {
		(wireSize(wn))--;
		wSegmRect(wn, i) = wSegmRect(wn, wireSize(wn));
		wSegmSide(wn, i) = wSegmSide(wn, wireSize(wn));
		wSegmRect(wn, wireSize(wn)) = NULL;
		i--;
	    }
	}
    } while(ws != wireSize(wn));
    FREE_WIRE(wn, 1);
    ForAllWires( i )
      if(i != wn) {
	  wireFree(i) = 0;
	  free_wire(i, 1);
      }
    
    for(i=0 ; i<tot ; i++) {
	set_free_neighbor(wSegmRect(wn, i), wSegmSide(wn, i), 0);
	if(isConnected(wn, i))
	  set_free_neighbor(wSegmRect(wn, i), wSegmSide(wn, i), 1);
	else {
	    (wireSize(wn))--;
	    wSegmRect(wn, i) = wSegmRect(wn, wireSize(wn));
	    wSegmSide(wn, i) = wSegmSide(wn, wireSize(wn));
	    wSegmRect(wn, wireSize(wn)) = NULL;
	}
	if(tot > wireSize(wn))     tot = wireSize(wn);
    }
}



