/*      room.c
 *
 *      Purpose: Getting the available room.
 *      Author: Enrico Malavasi
 *      Ver. 3.20
 *      Last revised: August 1991
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"
#include "space.h"

XXXavailable(wn, r, theDir)
int   wn;            /*  Given wire number  */
Rect  r;             /*  Inquired rect  */
int   theDir;        /*  Inquired side */
{
    Rect s;
    int  mtheDir;
    
    if(r == NULL)  return(NO);
    if(ViaUp(r) || ViaDown(r))     return(NO);
    if((s=free_neighbor(r, EAST))  == NULL)  return(NO);
    if(free_neighbor(s, NORTH)     == NULL)  return(NO);
    if((s=free_neighbor(r, NORTH)) == NULL)  return(NO);
    if(free_neighbor(s, EAST)      == NULL)  return(NO);
    if(theDir == UP) {
	if((r=neighbor(r, UP)) == NULL)        return(NO);
	if((s=free_neighbor(r, EAST))  == NULL)  return(NO);
	if(free_neighbor(s, NORTH)     == NULL)  return(NO);
	if((s=free_neighbor(r, NORTH)) == NULL)  return(NO);
	if(free_neighbor(s, EAST)      == NULL)  return(NO);
	return(YES);
    }
    mtheDir = AbsDualDir(theDir);
    if((r=free_neighbor(r, -mtheDir)) == NULL)  return(NO);
    if(ViaUp(r) || ViaDown(r))     return(NO);
    if(free_neighbor(r, mtheDir)    == NULL)  return(NO);
    if((s=free_neighbor(r, theDir)) == NULL)  return(NO);
    if(free_neighbor(s, mtheDir)    == NULL)  return(NO);
    return(YES);
}

newAvailableBox(wn, BB, bUp, exclude, exclUp, r, side)
int      wn;
RoadBox *BB, *bUp;
int      exclude,
         exclUp;
Rect     r;
int      side;
/*  Return NOTFOUND if the given Box is obstacle-free
 *         WIRE_NUM+cellNo if it intersects cell cellNo
 *         WN if it intersects wire WN
 *  The intersecting wire elements on the given excludedDir side are ignored.
 */
{
    int       cn, wi;
    _wirePos  wp;
    
    if((cn=CellInBox(BB, 0)) != NOTFOUND)
      return(WIRE_NUM+cn);
    if((wi=WireInBox(wn, BB, exclude, r, &wp)) != NOTFOUND)
      return wi;
    if(side == UP) {
	if((cn=CellInBox(bUp, 0)) != NOTFOUND)
	  return(WIRE_NUM+cn);
	if((wi=WireInBox(wn, bUp, exclUp, r, &wp)) != NOTFOUND)
	  return wi;
    }
    return NOTFOUND;
}

/*  ============================================================
 *  The following functions checks whether the given wire crosses
 *  the Box: in this case the position of the crossing wire
 *  element is returned, otherwise NOTFOUND.
 */
WireIntersection(wn, BB, begin)
int      wn;       /*  Index of the wire to be checked */
RoadBox *BB;       /*  Box to be inspected */
int      begin;    /*  First wire element to be considered  */
{
    Rect    r;
    RoadCoord   add, add0;     /*  Added width due to the via size  */
    int     side, i;
    RoadBox Bj;
    
    if(checkWireBox(wn, BB) == NO) return(NOTFOUND);
    add0 = addViaSpacing(BB->z);
    for(i=begin ; i<wireSize(wn) ; i++) {
	r    = wSegmRect(wn, i);
	side = wSegmSide(wn, i);
	if(r->layer != BB->z) {
	    if(side != UP)  continue;
	    if(r->layer != (BB->z-1)) continue;
	}
	Bj.x1 = (side == NORTH) ? r->x1 : r->x2;
	Bj.y1 = (side == EAST)  ? r->y1 : r->y2;
	Bj.x2 = r->x2;
	Bj.y2 = r->y2;
	if(side == UP) {
	    add = add0;
	    Bj.x1 -= add;  Bj.y1 -= add;
	    Bj.x2 += add;  Bj.y2 += add;
	}
	if((Bj.x1 < BB->x2) && (Bj.x2 > BB->x1) &&
	   (Bj.y1 < BB->y2) && (Bj.y2 > BB->y1))
	  return(i);
    }
    return(NOTFOUND);
}

/*  ============================================================
 *  This function returns the spacing needed between a wire
 *  segment and another item. If <side2> is NODIRECTION, the
 *  spacing between the first wire segment and an obstacle is
 *  returned. Otherwise, the spacing between the two given wire
 *  segments is computed.
 */
RoadCoord spacingBetween(l1, side1, l2, side2)
int   l1, side1, l2, side2;
{
  RoadCoord  wdt1, wdt2;

  if(side2 == NODIRECTION) {
    if(side1 != UP)
      return((RoadCoord)
	     (layers[l1].wire_dist+layers[l1].wire_width*0.5));
    if((l1 == l2) || (l2 == (l1+1)))
      return((RoadCoord)
	     (layers[l2].via_width*0.5+layers[l2].wire_dist));
  }

/*  ============================================================
 *  1.)  No vias
 */
  if(side1 != UP) {
    if(side2 != UP) {
      if(l1 == l2)
	return((RoadCoord)
	       (layers[l1].wire_dist+layers[l1].wire_width));
      else
	return(NOSPACE);
    }
/*  ============================================================
 *  2.)  1 plain, 2 is a via
 */
    if((l1 == l2) || (l1 == (l2+1)))
      return((RoadCoord) (layers[l1].wire_dist+(layers[l1].wire_width+layers[l1].via_width)*0.5));
    return(NOSPACE);
  }

/*  ============================================================
 *  3.)  1 is a via. 2 plain
 */
  if(side2 != UP) {
    if((l1 == l2) || (l2 == (l1+1)))
      return((RoadCoord) (layers[l2].wire_dist+(layers[l2].wire_width+layers[l2].via_width)*0.5));
    return(NOSPACE);
  }

/*  ============================================================
 *  4.)  Both are vias
 */
  if(l1 == l2) {
    wdt1 = layers[l1].via_width + layers[l1].wire_dist;
    wdt2 = layers[l2].via_width + layers[l2].wire_dist;
    return((RoadCoord) (max(wdt1, wdt2)));
  }
  if(l1 == (l2+1))
    return((RoadCoord)
	   (layers[l1].via_width+layers[l1].wire_dist));
  if(l2 == (l1+1))
    return((RoadCoord)
           (layers[l2].via_width+layers[l2].wire_dist));

  return(NOSPACE);
}

RoadCoord additionalSpacing(w1, w2)
int  w1, w2;
{
    return(NOSPACE);
}
