/*	sched.c
 *
 *	Author: Enrico Malavasi
 *      Ver. 3.00
 * 	Last revised: January, 1991
 *
 *	Purpose: Initial scheduling of wires, evaluation of the congested
 *		 areas and choice of a net to route before all the others.
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"


typedef struct CA {  RoadBox box;
		     int   net;
		   } CA;
static CA   *CongArea = NULL;
static int   CASize;

CreateCongestionAreas()
{
  int i, j, l, x,y;

  CongArea = REALLOC(CA, CongArea, NET_NUM*LAYER_NUM);

  for(j=CASize=0 ; j<NET_NUM ; j++)
    for(l=0 ; l<LAYER_NUM ; l++) {
      CongArea[CASize].box.x1 = cells[0].x2;
      CongArea[CASize].box.y1 = cells[0].y2;
      CongArea[CASize].box.x2 = cells[0].x1;
      CongArea[CASize].box.y2 = cells[0].y1;
      CongArea[CASize].box.z  = l;
      CongArea[CASize].net   = j;

      ForAllPins(i)
	if((pins[i].netNo == j) &&
	   (pins[i].layer == l))
	  if(pins[i].cellNo != NOTFOUND) {
	    switch(pins[i].side) {
	    case NORTH:
	      x = rRightX(pins[i].rect); y =  rLowerY(pins[i].rect); break;
	    case SOUTH:
	      x = rRightX(pins[i].rect); y =  rUpperY(pins[i].rect); break;
	    case EAST:
	      x = rLeftX(pins[i].rect);  y =  rUpperY(pins[i].rect); break;
	    case WEST:
	      x = rRightX(pins[i].rect); y =  rUpperY(pins[i].rect); break;
	    }
	    CongArea[CASize].box.x1 = min(CongArea[CASize].box.x1, x);
	    CongArea[CASize].box.y1 = min(CongArea[CASize].box.y1, y);
	    CongArea[CASize].box.x2 = max(CongArea[CASize].box.x2, x);
	    CongArea[CASize].box.y2 = max(CongArea[CASize].box.y2, y);
	  }

/*  ---------------------------------------------------------------------
 *  Symmetric nets introduce a congestion across the symmetry axis.
 */
      if((GetSymmetryFlag()) &&
	 (InquireSymmetryList(j) != NOTFOUND))
	CongArea[CASize].box.x2 = cells[0].x2;

      CASize++;
    }

/*  ------------------------------------------------------------
 *  Delete the meaningless congested areas
 */
    for(i=0 ; i<CASize ; i++)
      if((CongArea[i].box.x1 >= CongArea[i].box.x2) ||
	 (CongArea[i].box.y1 >= CongArea[i].box.y2))
	CongArea[i--] = CongArea[--CASize];

/*  ------------------------------------------------------------
 *  Delete the congestion areas corresponding to the high
 *  cost layers for every net
 */
#define   COST_THRESHOLD   10
    ForAllNets(i) {
	LayersCost(i);
	ForAllLayers(l)
	  if(layers[l].cost >= COST_THRESHOLD)
	    for(j=0 ; j<CASize ; j++)
	      if((CongArea[j].net == i) && (CongArea[j].box.z == l))
		CongArea[j--] = CongArea[--CASize];
    }
}

SuggestNextNet()
{
    int nextNet;

    loop {
	if((nextNet=getFirstNetListItem()) == NOTFOUND)  return NOTFOUND;
	if(! isNetRouted( nextNet ))                     return nextNet;
	deleteNetFromNetList(nextNet);
    }
}

/*  ============================================================
 *  This function estimates the congestion cost on the ground
 *  of the CA frames. Returns a congestion weight (0-1)
 */
#define   NORMAL_AMOUNT    10
void
CongestionEstimate(NetNo)
int  NetNo;
{
    int        i, j;
    float      m;
    pFarad     maxcap;
    Rect       r;

    for(j=i=0 ; j<CASize ; j++)
      if((CongArea[j].net != NetNo) && (InquireNetList(CongArea[j].net) != NOTFOUND)) {
	  if((m=InquireCC_coupling(NetNo, CongArea[j].net, &maxcap)) > 0) {
	      m = (1 + m * 0.05);
	      ForAllRects(i) {
		  r = rects+i;
		  if((r->layer >= CongArea[j].box.z) &&
		     (r->x2 < CongArea[j].box.x2) &&
		     (r->y2 < CongArea[j].box.y2) &&
		     (r->x2 > CongArea[j].box.x1) &&
		     (r->y2 > CongArea[j].box.y1)) {
		      r->cost[0] *= m;
		      r->cost[2] *= m;
		  }
	      }
	  }
      }
}

netCongestion(NetNo)
int  NetNo;
{
  int i, n;

/*  ============================================================
 *  OK. This is a stupid way of computing congestion. Or better
 *  it is not congestion, only the number of pins. But it is better
 *  than nothing.
 */
  for(i=0,n=0 ; i<PIN_NUM ; i++)
    if(pins[i].netNo == NetNo)
      n++;
  return n;
}

/*  ============================================================
 *  Return the semiperimeter of the smallest bounding box containing
 *  all the pins of the given net. It is a rough estimate of net length.
 */
RoadCoord
netLength(netNo)
int netNo;
{
  int i;
  for(i=0 ; i<CASize ; i++)
    if((InquireNetList(CongArea[i].net) != NOTFOUND) &&
       (CongArea[i].net != netNo)) {
      return((RoadCoord) (CongArea[i].box.x2 - CongArea[i].box.x1) +
	                 (CongArea[i].box.y2 - CongArea[i].box.y1));
    }
  return((RoadCoord) (cells[0].x2 - cells[0].x1) + (cells[0].y2 - cells[0].y1));
}
