/*      space.c
 *
 *      Author: Enrico Malavasi
 *      Ver. 2.10
 *      Last revised:  October, 1990
 *
 *      Purpose: Full wire spacing
 */
#define  FIRSTSPACE   1
#include "roadGlobals.h"
#include  "space.h"

#define    MAXMODNUM   100
static int spModList[MAXMODNUM];   /* List of modified wires */
static int spModListNum;

void
  spAddModList(wn)
int  wn;
/*  Add the given wire index to the list of the modified wires  */
{
    int i;
    for(i=0 ; i<spModListNum ; i++) {
	if(spModList[i] == wn)  return;
    }
    spModList[spModListNum++] = wn;
}

/*  ============================================================
 *  <wn> is a valid wire index.
 *  The wire if FREE-0
 */
static
  spaceFromCells(wn)
int  wn;
{
  int       modified,  /*  Flag: has the wire been modified ? */
            begin,
            wPos,
            side,
            direction,
            cn;
  Rect      r;
  RoadCoord wdt, slc,
            vector;
  RoadBox   BB;
  Pins      ppp;
  Cell     *cc;

  modified = NO;
  for(wPos=0 ; (r=wSegmRect(wn, wPos))!=NULL ; wPos++) {
    side = wSegmSide(wn, wPos);
    if(((ppp=PinIsIn(r, wireNetNo(wn))) != NULL) &&
       (side != UP) && (ppp->side != side))     continue;
    wdt = spacingBetween(rLayer(r), side, rLayer(r), NODIRECTION);
    if(side != UP) adjustBoxWidth(wdt, wn, rLayer(r));
    WireSegmentToBox(r, side, wdt, &BB, NULL, 0, 0);

    if(GetSymmetryFlag())
      if(side != NORTH) {
	  slc = ((InquireSymmetryList(wireNetNo(wn)) == wireNetNo(wn)) ? SymmetryAxis : (SymmetryAxis+wdt/2));
	  BB.x2 = min(BB.x2, slc);
      }

    for(begin=0 ; ; ) {
      if((cn=CellInBox(&BB, begin)) == NOTFOUND)
	if(side == UP) {
	  wdt = spacingBetween(rLayer(r)+1, UP, rLayer(r)+1, NODIRECTION);
	  if(side != UP) adjustBoxWidth(wdt, wn, rLayer(r));
	  WireSegmentToBox(neighbor(r, UP), side, wdt, &BB, NULL, 0, 0);
	  cn = CellInBox(&BB, begin);
	}
      if(cn == NOTFOUND) break;
      begin = cn + 1;

      cc = cells+cn;
      vector = 0;

      switch(side) {
      case NORTH :
	if(cn == 0) {
	  if(cLowerY(cc) > BB.y1) {
	    vector    = cLowerY(cc) - BB.y1;
	    direction = NORTH;
	  } else if(cUpperY(cc) < BB.y2) {
	    vector    = BB.y2 - cUpperY(cc);
	    direction = SOUTH;
	  }
	  break;
	}
	if((cLowerY(cc) < BB.y2) && (cLowerY(cc) > rUpperY(r))) {
	  vector    = BB.y2 - cLowerY(cc);
	  direction = SOUTH;
	} else if((cUpperY(cc)>BB.y1) && (cUpperY(cc)<rUpperY(r))) {
	  vector    = cUpperY(cc) - BB.y1;
	  direction = NORTH;
	}
	break;
      
      case EAST  :
	if(cn == 0) {
	  if(cLeftX(cc) > BB.x1) {
	    vector    = cLeftX(cc) - BB.x1;
	    direction = EAST;
	  } else if(cRightX(cc) < BB.x2) {
	    vector    = BB.x2 - cRightX(cc);
	    direction = WEST;
	  }
	  break;
	}
	if((cLeftX(cc) < BB.x2) && (cLeftX(cc) > rRightX(r))) {
	  vector    = BB.x2 - cLeftX(cc);
	  direction = WEST;
	} else if((cRightX(cc) > BB.x1) && (cRightX(cc) < rRightX(r))) {
	  vector    = cRightX(cc) - BB.x1;
	  direction = EAST;
	}
	break;
      
      case UP :
	if(cn == 0) {
	  if(cLowerY(cc) > BB.y1) {
	    vector    = cLowerY(cc) - BB.y1;
	    direction = NORTH;
	    break;
	  }
	  if(cUpperY(cc) < BB.y2) {
	    vector    = BB.y2 - cUpperY(cc);
	    direction = SOUTH;
	    break;
	  }
	  if(cLeftX(cc) > BB.x1) {
	    vector    = cLeftX(cc) - BB.x1;
	    direction = EAST;
	    break;
	  }
	  if(cRightX(cc) < BB.x2) {
	    vector    = BB.x2 - cRightX(cc);
	    direction = WEST;
	    break;
	  }
	  break;
	}
	if((cLowerY(cc) < BB.y2) && (cLowerY(cc) > rUpperY(r))) {
	  vector    = BB.y2 - cLowerY(cc);
	  direction = SOUTH;
	  break;
	}
	if((cUpperY(cc) > BB.y1) && (cUpperY(cc) < rUpperY(r))) {
	  vector    = cUpperY(cc) - BB.y1;
	  direction = NORTH;
	  break;
	}
	if((cLeftX(cc) < BB.x2) && (cLeftX(cc) > rRightX(r))) {
	  vector    = BB.x2 - cLeftX(cc);
	  direction = WEST;
	  break;
	}
	if((cRightX(cc) > BB.x1) && (cRightX(cc) < rRightX(r))) {
	  vector    = cRightX(cc) - BB.x1;
	  direction = EAST;
	  break;
	}
	break;
      }

      if(vector != 0)
	if(TryToEnlarge(wn, wPos, direction, vector) > 0) {
	  modified = YES;
	  if(wPos >= wireSize(wn)) return(YES);
	  break;
	}
    }
  }
  return(modified);
}

/*  ============================================================
 *  This function spaces a wire with the recursive spacing
 *  algorithm applied to each of its elements.
 */
static space00(wn)
int wn;       /*  Index of the wire to be spaced  */
{
  RoadCoord nddwdt,        /*  Total micro-translation      */
            wdt,
            diff;          /*  Difference between distances */
  int     modified,
          direction,
          side, sidei,   /*  Side indicators  */
          wi, pos,       /*  Intersecting wire index  */
          wPos;          /*  Wire position indicator  */
  Rect    r, ri;
  Pins    ppp;
  RoadBox BB, bUp;       /*  Protection frame */
  _wirePos  wp;

  modified = NO;
  if(wn == NOTFOUND) return(modified);
  CONDfree_wire(wn, 0);

  for(wPos=0 ; (r=wSegmRect(wn, wPos))!=NULL ; wPos++) {
    side = wSegmSide(wn, wPos);
/*  ------------------------------------------------------------
 *  Build the protection frame
 */
    wdt = spacingBetween(rLayer(r), side, rLayer(r), NORTH);
    WireSegmentToBox(r, side, wdt, &BB, &bUp, NODIRECTION, NODIRECTION);
    wi = WireInBox(wn, &BB, (side==UP) ? NODIRECTION : AbsDualDir(side), r, &wp);
    if(wi == NOTFOUND)
      if(side == UP)
	wi = WireInBox(wn, &bUp, NODIRECTION, neighbor(r, UP), &wp);

    if(wi == NOTFOUND)     continue;
    if((pos = wp.pos) == NOTFOUND) {
	fprintf(stderr, "Warning. <space00>: Continuing after internal error\n");
	continue;   /*   Strange. This should not happen   */
    }
/*  ============================================================
 *  An intersection was found. The micro-translation is defined
 *  and then applied.
 */
    ri    = wSegmRect(wi, pos);
    sidei = wSegmSide(wi, pos);
    
    wdt = spacingBetween(rLayer(r), side, rLayer(ri), sidei) -
      spacingBetween(rLayer(r), side, rLayer(ri), NORTH) +
      additionalSpacing(wn, wi);
    if((side != EAST) && (sidei != EAST)) {
      diff = (BB.y2-rUpperY(ri))  - (rUpperY(ri)-BB.y1);
      if(diff == 0) {
	continue;
      }
      if(diff > 0) {
	nddwdt = wdt+rUpperY(ri)-BB.y1;
	if((wdt=TryToEnlarge(wn, wPos, NORTH, nddwdt)) < nddwdt) {
	  CONDfree_wire(wn, 1);
	  free_wire(wi, 0);
	  TryToEnlarge(wi, pos, SOUTH, nddwdt-wdt);
	  free_wire(wi, 1);
	  CONDfree_wire(wn, 0);
	}
	modified = YES;
	if(wPos >= wireSize(wn)) break;
	continue;
      }
      if(diff < 0) {
	nddwdt = wdt+BB.y2-rUpperY(ri);
	if((wdt=TryToEnlarge(wn, wPos,SOUTH,nddwdt)) < nddwdt) {
	  CONDfree_wire(wn, 1);
	  free_wire(wi, 0);
	  TryToEnlarge(wi, pos, NORTH, nddwdt-wdt);
	  free_wire(wi, 1);
	  CONDfree_wire(wn, 0);
	}
        modified = YES;
	if(wPos >= wireSize(wn)) break;
        continue;
      }
    }
    if((side != NORTH) && (sidei != NORTH)) {
      diff = (BB.x2-rRightX(ri)) - (rRightX(ri)-BB.x1);
      if(diff == 0) {
	continue;
      }
      if(diff > 0) {
	nddwdt = wdt+rRightX(ri)-BB.x1;
	if((wdt=TryToEnlarge(wn, wPos, EAST, nddwdt)) < nddwdt) {
	  CONDfree_wire(wn, 1);
	  free_wire(wi, 0);
	  TryToEnlarge(wi, pos, WEST, nddwdt-wdt);
	  free_wire(wi, 1);
	  CONDfree_wire(wn, 0);
	}
        modified = YES;
	if(wPos >= wireSize(wn)) break;
        continue;
      }
      if(diff < 0) {
	nddwdt = wdt+BB.x2-rRightX(ri);
	if((wdt=TryToEnlarge(wn, wPos, WEST, nddwdt)) < nddwdt) {
	  CONDfree_wire(wn, 1);
	  free_wire(wi, 0);
	  TryToEnlarge(wi, pos, EAST, nddwdt-wdt);
	  free_wire(wi, 1);
	  CONDfree_wire(wn, 0);
	}
        modified = YES;
	if(wPos >= wireSize(wn)) break;
        continue;
      }
    }
  }

  if(spaceFromCells(wn) == YES)
    modified = YES;

  CONDfree_wire(wn, 1);
  return(modified);
}

/*  ============================================================
 *  This function spaces the throughholes of the given wire
 */
static spaceVias(wn)
int wn;       /*  Index of the wire to be checked  */
{
  RoadCoord  rho,           /*  Basic micro-translation module  */
             diffx, diffy;  /*  Difference between distances */
  int        pos1, pos2,
             dir;
  int        begin, next;
  int        posLow, posUp; /*  Lower and upper element positions */
  Rect       rLow, rUp;     /*  Lower and upper element  */
  Rect       s;

  if(wn == NOTFOUND) return;

  CONDfree_wire(wn, 0);
  for(begin=0 ; (pos1=wireSegmOnSide(wn, UP, begin))!=NOTFOUND ; begin=pos1+1) {
    for(next=pos1+1;(pos2=wireSegmOnSide(wn,UP,next))!=NOTFOUND;next=pos2+1) {
      if(rLayer(wSegmRect(wn, pos1)) == rLayer(wSegmRect(wn, pos2)))  continue;

      if(rLayer(wSegmRect(wn, pos1)) > rLayer(wSegmRect(wn, pos2))) {
	rLow = wSegmRect(wn, pos2);
	rUp  = wSegmRect(wn, pos1);
	posLow = pos2;
	posUp = pos1;
      } else {
	rLow = wSegmRect(wn, pos1);
	rUp = wSegmRect(wn, pos2);
	posLow = pos1;
	posUp = pos2;
      }
      rho = spacingBetween(rLayer(rLow), UP, rLayer(rUp), UP);

      loop {
	diffx = rRightX(rUp) - rRightX(rLow);
	diffy = rUpperY(rUp) - rUpperY(rLow);
	if((abs(diffx) >= rho) || (abs(diffy) >= rho))
	  break;

/*  ------------------------------------------------------------
 *  They must be spaced
 */
	if((diffx != 0) || (diffy != 0)) {
	  if(diffx != 0) {
	    dir = (diffx > 0) ? WEST : EAST;
	    if(TryToEnlarge(wn, posLow, dir, rho-diffx)>= (rho-diffx))
	      break;
	    if(TryToEnlarge(wn, posUp, -dir, rho-diffx) >= (rho-diffx))
	      break;
	  }
	  if(diffy != 0) {
	    dir = (diffy > 0) ? SOUTH : NORTH;
            if(TryToEnlarge(wn, posLow, dir, rho-diffy)>=(rho-diffy))
	      break;
	    if(TryToEnlarge(wn, posUp, -dir, rho-diffy) >= (rho-diffy))
              break;
	  }
	  break;
	} else {
#define ELEMENT(RR, SS)  wire_segment_position(wn, RR, SS, NOTFOUND)
/*  ------------------------------------------------------------
 *  They are one on top of the other
 */
	  if(ELEMENT(rLow, NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, WEST, rho) >= rho)
	      break;
	  if(ELEMENT(rLow, EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, SOUTH,rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(rLow, EAST),NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, EAST, rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(rLow, NORTH),EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, NORTH, rho) >= rho)
	      break;
	  s = neighbor(rUp, UP);
	  if(ELEMENT(s,NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, WEST, rho) >= rho)
	      break;
	  if(ELEMENT(s,EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, SOUTH, rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(s,EAST),NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, EAST, rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(s,NORTH),EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, NORTH, rho) >= rho)
	      break;

	  if(ELEMENT(rLow, NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, EAST, rho) >= rho)
	      break;
	  if(ELEMENT(rLow, EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, NORTH,rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(rLow, EAST),NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, WEST, rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(rLow, NORTH),EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posUp, SOUTH, rho) >= rho)
	      break;
	  if(ELEMENT(s,NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, EAST, rho) >= rho)
	      break;
	  if(ELEMENT(s,EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, NORTH, rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(s,EAST),NORTH) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, WEST, rho) >= rho)
	      break;
	  if(ELEMENT(neighbor(s,NORTH),EAST) != NOTFOUND)
	    if(TryToEnlarge(wn, posLow, SOUTH, rho) >= rho)
	      break;
	  break;
	}
      }
    }
  }
  CONDfree_wire(wn, 1);
}

/*  ============================================================
 *  The following function spaces a wire. The spacing procedure
 *  is iterated several times, unless a spacing step doesn't
 *  provide any modification.
 *  At last, a complete pruning is carried out.
 *  This routine may change the wire index !!!
 */
#define   MAX_LOOP    2   /*  Max number of iterations allowed  */
int
fullSpaceWire(wn)
int wn;
{
    int        i, j, flag;
    RoadCoord  lyr[LAYER_MAX];

    spModListNum = 1;
    spModList[0] = wn;

    if(wn == NOTFOUND) {
	warnMessage("Nonexistent wire!", 0);
	return;
    }
    updateWireBoxes(NOTFOUND);
    contMessage("Spacing %s ... ", rd_net[wireNetNo(wn)].name);
    
    for(j=0 ; j<LAYER_NUM ; j++)
      lyr[j] = layers[j].wire_dist;
    for(i=0 ; i<MAX_LOOP ; i++) {
	WireDeleteDuplicates(wn);
	if((flag=space00(wn)) == NO) break;
	for(j=0 ; j<LAYER_NUM ; j++) {
	    layers[j].wire_dist = (lyr[j] * (MAX_LOOP-i-1)) / MAX_LOOP;
	    if(layers[j].wire_dist <= 0) layers[j].wire_dist = 1;
	}
    }
    for(j=0 ; j<LAYER_NUM ; j++)
      layers[j].wire_dist = lyr[j];
    
    for(i=0 ; i<spModListNum ; i++) {
	WireDeleteDuplicates(spModList[i]);
	WireDeleteOverlap(spModList[i]);
	spaceVias(spModList[i]);
	collapseWire(spModList[i]);
	PruneWire(spModList[i]);
	updateWireBoxes(spModList[i]);
    }
    if((flag == YES) && (MAX_LOOP > 1))
      contMessage("Partially ", 0);
    contMessage("Done", 0);
}
