/*	split.c
 *
 *	Author: Enrico Malavasi
 *      Ver. 3.1
 * 	Last revised: April, 1991
 *
 * 	Purpose: Create a vertical splitting line across the rectangles
 *		 of the grid, splitting every node of a given column
 *		 into two neighbor elements.
 *      Possible modifications:
 *              Wire sizes, rects base.
 */
#include "roadGlobals.h"

static int  TheDir, PosAlDir, MTheDir, NegAlDir,
            (*mySplitFunct)();

static rdSplitCol(r, ddd)
Rect r;
RoadCoord ddd;
{
    Rect         l, x;        /* Temporary <rect> storage var. .... */
    int i,j;         /* Loop variables ................... */

    if((ddd <= 0) || (ddd >= rect_size(r, PosAlDir))) return(FAILURE);

    /*  Follow the column southward  */
    r = follow_Neighbor(r, MTheDir);

    /*  Divide all the rectangles of the current layer   */
    do  {
	i = r-rects;
	if(TOTAL_RECT_NUM >= RECT_ALLOC) {
	    amplia_rect(1);
	}
	r = rects+i;
	    
	l = rects + TOTAL_RECT_NUM;
	TOTAL_RECT_NUM++;

	r->free = 0;
	rdCopyRect( l, r );
	l->flag = 0;
	rRightX(r) = rLeftX(l) = rLeftX(r) + ddd;

	/*  ------------------------------------------------------------
	 *  Set the neighborhood relations
	 */
	neighbor(l, TheDir) = NULL;
	
	set_neighb(neighbor(r, PosAlDir), l, PosAlDir);
	set_neighb(r, l, NegAlDir);
	
	if ((x=neighbor(r, MTheDir)) != NULL)
	  set_neighb(neighbor(x, PosAlDir), l, MTheDir);
	else
	  neighbor(l, MTheDir) = NULL;
	if(((x = neighbor(r,DOWN)) != NULL) && (rRightX(x) < rRightX(l)))
	  set_neighb(neighbor(x, PosAlDir), l, DOWN);
	else
	  neighbor(l, DOWN) = x;
	if(((x = neighbor(r,UP)) != NULL) && (rRightX(x) < rRightX(l)))
	  set_neighb(neighbor(x, PosAlDir), l, UP);
	else
	  neighbor(l, UP) = x;
	
    } while((r = neighbor(r, TheDir)) != NULL);

    /*  Follow the new half-column southward to look for a way upward  */
    if(rLayer(l) < LAYER_NUM-1)  {
	i = l-rects;
	do
	  if(((r=neighbor(l, UP)) != NULL) &&
	     (r == neighbor(neighbor(l, NegAlDir), UP)))
	    if(rect_size(r, PosAlDir) > ddd) {
		j = l-rects;
		(*mySplitFunct)(r,ddd);
		l = rects+j;
	    }
	while((l=neighbor(l, MTheDir)) != NULL);
	l = rects+i;
    }

    /*  ... and downward    */
    if(rLayer(l) > 0) {
	do
	  if(((r=neighbor(l,DOWN)) != NULL) &&
	     (r == neighbor(neighbor(l, NegAlDir), DOWN)))
	    if(rect_size(r, PosAlDir) > ddd) {
		j = l-rects;
		(*mySplitFunct)(r,ddd);
		l = rects+j;
	    }
	while((l=neighbor(l, MTheDir)) != NULL);
    }

    return SUCCESS;
}

static rdSplitRow(r, ddd)
Rect r;
RoadCoord ddd;
{
    Rect         l, x;         /* Temporary <rect> storage var. .... */
    int i,j;          /* Loop variables ................... */

    if((ddd <= 0) || (ddd >= rect_size(r, PosAlDir))) return(FAILURE);

    /*  Follow the row westward     */
    r = follow_Neighbor(r, MTheDir);

    /*  Divide all the rectangles of the row on the current layer   */
    do  {
	i = r-rects;
	if(TOTAL_RECT_NUM >= RECT_ALLOC) {
	    amplia_rect(1);
	}
	r = rects+i;

	l = rects + TOTAL_RECT_NUM;
	TOTAL_RECT_NUM++;
	
	r->free = 0;
	rdCopyRect( l, r );
	l->flag = 0;
	rUpperY(r) = rLowerY(l) = rLowerY(r)+ ddd;    /* ... nearly ... ...... */

	/*  Set the neighborhood relations  */
	neighbor(l, TheDir) = NULL;
	
	set_neighb(neighbor(r, PosAlDir), l, PosAlDir); /* Northward  */
	set_neighb(r, l, NegAlDir);		  /* Southward  */
	
	if((x=neighbor(r, MTheDir)) != NULL)    /* Westward   */
	  set_neighb(neighbor(x, PosAlDir), l, MTheDir);
	else
	  neighbor(l, MTheDir) = NULL;
	if(((x = neighbor(r,DOWN)) != NULL) && (rUpperY(x) < rUpperY(l)))
	  set_neighb(neighbor(x, PosAlDir), l, DOWN);
	else
	  neighbor(l, DOWN) = x;
	if(((x = neighbor(r,UP)) != NULL) && (rUpperY(x) < rUpperY(l)))
	  set_neighb(neighbor(x, PosAlDir), l, UP);
	else
	  neighbor(l, UP) = x;
	
    } while((r = neighbor(r, TheDir)) != NULL);
    
    /*  -------------------------------------------------------------
     *  Follow the new half-row westward to look for a way upward ...
     */
    if(rLayer(l) < LAYER_NUM-1)  {
	i = l-rects;
	do
	  if(((r=neighbor(l, UP)) != NULL) &&
	     (r == neighbor(neighbor(l, NegAlDir), UP)))
	    if(rect_size(r, PosAlDir) > ddd) {
		j = l-rects;
		(*mySplitFunct)(r,ddd);
		l = rects+j;
	    }
	while((l=neighbor(l, MTheDir)) != NULL);
	l = rects+i;
    }
    
    /*  ------------------------------------------------------------
     *  ... and downward
     */
    if(rLayer(l) > 0) {
	do
	  if(((r=neighbor(l,DOWN)) != NULL) &&
	     (r == neighbor(neighbor(l, NegAlDir), DOWN)))
	    if(rect_size(r, PosAlDir) > ddd) {
		j = l-rects;
		(*mySplitFunct)(r,ddd);
		l = rects+j;
	    }
	while((l=neighbor(l, MTheDir)) != NULL);
    }
    
    return(SUCCESS);
}

/*  ============================================================
 *  Manager of the splitting functions.
 */
GridSplit(r, ddd, DiR, letFree)
Rect r;
RoadCoord ddd;
int DiR, letFree;
{
    int  i, j;
    int  retVal;
    int  oldRectNum;

    if(ddd <= 0)  return(FAILURE);
    retVal = FAILURE;
    ForAllWires(i)
      free_wire(i, 0);

    oldRectNum = TOTAL_RECT_NUM;

    if(north_or_south(DiR)) {
	TheDir   =   NORTH;
	PosAlDir =   EAST;
	MTheDir  =   SOUTH;
	NegAlDir =   WEST;
	mySplitFunct = rdSplitCol;
	retVal = (*mySplitFunct)(r, ddd);
    } else if(east_or_west(DiR)) {
	TheDir   =   EAST;
	PosAlDir =   NORTH;
	MTheDir  =   WEST;
	NegAlDir =   SOUTH;
	mySplitFunct = rdSplitRow;
	retVal = (*mySplitFunct)(r, ddd);
    }

    if(oldRectNum < TOTAL_RECT_NUM) {
	int   wn;
	Pins  p;
	Rect  rp;

	if(pins[0].rect != NULL) {
	    for(i=0,p=pins ; i<PIN_NUM ; i++,p++) {
		if((rp=neighbor(p->rect, PosAlDir)) == NULL)   continue;
		if((rp-rects) >= oldRectNum) {
		    if(((TheDir == NORTH) && (pin_x(p) > rRightX(p->rect))) ||
		       ((TheDir == EAST) && (pin_y(p) > rUpperY(p->rect)))) {
			(p->rect)->flag &= (~ RECTFLAG_TERM);
			p->rect = rp;
			rp->flag |= RECTFLAG_TERM;
		    }
		}
	    }
	}

	ForAllWires( wn ) {
	    for(j=0 ; j<wireSize( wn ) ; j++) {
		rp = wSegmRect(wn, j);
		if((rp=neighbor(rp, PosAlDir)) == NULL)    continue;
		if((rp-rects) >= oldRectNum) {
		    if((wSegmSide(wn, j) == PosAlDir) || (wSegmSide(wn, j) == UP)) {
			wSegmRect(wn, j) = rp;
		    } else if(wSegmSide(wn, j) == TheDir) {
			add_to_wire(wn, rp, TheDir);
		    }
		}				
	    }
	}
    }
    
    ForAllWires(i)
      if(i != letFree)
	free_wire(i, 1);
    return retVal;
}
