/*	STRUTIL.C
 *
 *	Author: Enrico Malavasi, Univ. of Bologna
 *      Ver. 1.00
 * 	May, 1988
 *
 *	Purpose: Utilities for string management.
 */
#include <stdio.h>
#define SUCCESS        1
#define FAILURE        0
#define BLANK_CHAR(x)     ((x == ' ')\
			|| (x == '\t')\
			|| (x == '\n')\
			|| (x == EOF))

/*  ------------------------------------------------------------
 *  This function carries out a case-insensitive comparison
 *  between two strings.
 */
equal_nocase(s1, s2)
char *s1,*s2;		/* Strings to be compared .................. */
{
  int i= (-1);

  if(strlen(s1) != strlen(s2))   return(FAILURE);

  while(s1[++i] != '\0')
    if(toupper(s1[i]) != toupper(s2[i]))
      return(FAILURE);

  return(SUCCESS);
}

/*  ------------------------------------------------------------
 *  This function reads the first string met in a given open file.
 *  Blanks and tabs met before such string are ignored.
 *  The string read is returned. If EOF is reached before any valid
 *  string be met, the NULL pointer is returned.
 */
char *input_string(fil)
FILE *fil;		/* File where to read the string .......... */
{
  static char s[80];	/* The max string length is 80 characters   */
  int i = 0;	/* Index of the read characters ........... */

/*
 *  Read the blank characters.
 *  -------------------------- */
  do  {
    s[0]=fgetc(fil);
    if(s[0] == EOF) return(NULL);
  } while BLANK_CHAR(s[0]);

/*
 *  Read the string found.
 *  ---------------------- */
  do
    s[++i] = fgetc(fil);
  while(! BLANK_CHAR(s[i]));
  if(s[i] == EOF) return(NULL);
  else		s[i] = '\0';	/* End of the string ... */

  return(s);
}

/*  ------------------------------------------------------------
 *  This function returns the first character read from the given
 *  file. If EOL is reached before a valid character be read, the
 *  '\0' termination character is returned.
 */
GetEventualChar(fil)
FILE *fil;
{
  char c;

  do
    if(((c=fgetc(fil)) == '\n') ||
       (c == EOF))
      return('\0');
  while(BLANK_CHAR(c));
  return(c);
}
