/*      symmlook.c
 *
 *      Author: Enrico Malavasi, Univ. of Bologna
 *      Ver. 3.1
 *      Last revised: April, 1991
 *
 *      Purpose: Look for a symmetry constraint.
 */
#include "roadGlobals.h"

/*  ============================================================
 *  This function checks the necessity of the symmetry procedure
 *  and, if it is required, splits the circuit into two parts.
 *  The left part will be routed until completion of the symmetric
 *  set of wires.
 */
look_for_symmetry()
{
    int   i, j, l;

    unless(GetSymmetryFlag()) return;
    /*  Have all the symmetric nets been routed ? */
    /*  Check the symmetry of the placement.    */

    SymmetryAxis = (cells[0].x2 + cells[0].x1) /2.0;
    for(i=0 ; (i<=SUB_CELL_NUM)&&(GetSymmetryFlag()) ; i++) {
	
	if((SymmetryAxis - cells[i].x1) == (cells[i].x2 - SymmetryAxis))
	  continue;
	
	SetSymmetryFlag(NO);
	for(j=0 ; j<=SUB_CELL_NUM ; j++)
	  if((i != j) &&
	     (rdLowerY(cells+i) == rdLowerY(cells+j)) &&
	     (rdUpperY(cells+i) == rdUpperY(cells+j)) &&
	     (rdLayer(cells+i) == rdLayer(cells+j)) &&
	     ((SymmetryAxis - cells[i].x1) == (cells[j].x2 - SymmetryAxis)) &&
	     ((SymmetryAxis - cells[i].x2) == (cells[j].x1 - SymmetryAxis))) {
	      SetSymmetryFlag(YES);
	      break;
	  }
    }
    unless(GetSymmetryFlag()) {
	warnMessage("Placement non symmetric [instance %s].\n*** Symmetry constraints ignored", insts[cells[i].instanceNo].name);
	return;
    }
    
    /*  --------------------------------------------------------------
     *  This should be the last opportunity for changing the symm.flag
     */
    SetSymmetryFlag(NO);
    for(i=0 ; i<NET_NUM ; i++)
      if(InquireSymmetryList(i) != NOTFOUND) {
	  SetSymmetryFlag(YES);
	  break;
      }
    unless(GetSymmetryFlag()) return;
    initMessage("Symmetric circuit. The left half will be routed first\n", 0);
    
    /*  ============================================================
     *  The circuit is partly symmetric. Save the permanent data and 
     *  delete half of it.
     *  ------------------------------------------------------------ */
    SaveMirrData();
    for(i=0 ; i<=SUB_CELL_NUM ; i++) {
	if(cells[i].x2 <= SymmetryAxis)      continue;
	/*  ------------------------------------------------------------
	 *  Whole cells on the right side.
	 */
	if(cells[i].x1 >= SymmetryAxis) {
	    
	    for(l=0 ; l<PIN_NUM ; l++)
	      if(pins[l].cellNo == i)
		pins[l].netNo = NOTFOUND;
	    
	    delete_cell(i--);
	    continue;
	}
	
	/*  ------------------------------------------------------------
	 *  Cells half on the right side, half on the left
	 */
	cells[i].x2 = SymmetryAxis + 1;
	for(l=0 ; l<PIN_NUM ; l++)
	  if(pins[l].cellNo == i)
	    switch(pins[l].side) {
	      case NORTH:
	case SOUTH:
		if(pins[l].position > (SymmetryAxis - cells[i].x1))
		  pins[l].netNo = NOTFOUND;
		break;
	      case EAST:
		pins[l].netNo = NOTFOUND;
	      default: break;
	    }
    }
    
    /*  ------------------------------------------------------------
     *  Delete all the undesired pins.
     */
    for(i=0 ; i<PIN_NUM ; i++)
      if(pins[i].netNo == NOTFOUND)
	delete_pin(i--);
    
    divide();
    fragment();
    CreateNetList();
    CreatePinSheetList();
}
