/*	technol.c
 *
 *	Author: Enrico Malavasi
 *	Ver. 3.20
 *	Last revised: May, 1991
 *
 *	Purpose: Input the technology data from the technology file
 */
#include "port.h"
#include "utility.h"
#include "roadGlobals.h"
#include "roadTechnol.h"
#include "transf.h"

#define  ENDOFFILE \
       { fprintf(stderr,"\nTechnology file: Syntax error\n"); rdExit( 1 );}
#define MISSING_MODIFIER(mod,lay)  \
       { fprintf(stderr, "\nMissing %s modifier in layer %d\n",mod,lay); rdExit( 1 ); }
#define TechnError  \
       { fprintf(stderr, "Illegal parameter in the technology file\n");  rdExit( 1 ); }

#define   MAXCOMMNUM      20     /* Max number of commands on a line */
#define   MAXCOMMLEN      160    /* Max length of a command in chars */

/*  ============================================================
 *  This function reads the technologic data. They are caught
 *  from the  TECHNOL_FILE input file.
 */
read_technol( technologyFile )
char *technologyFile;
{
  FILE    *fTec;
  int      i, k;
  int      Lnum;
  char    *comm, *Home, *getenv();
  char    *arg, *NextCommand();
  PathName TechFile;
  float    a;
  extern pFarad LTG_XxxModel();

  strcpy(TechFile, technologyFile);
  if((fTec=fopen(TechFile, "r")) == NULL)
    if((Home=getenv("HOME")) != NULL) {
      sprintf(TechFile, "%s/%s", Home, technologyFile);
      if((fTec=fopen(TechFile, "r")) == NULL) {
	  fprintf(stderr, "\n%s: File not found\n", technologyFile);
	  rdExit( 1 );
      }
    }
  initMessage( "Reading technology file %s ... ", TechFile);

  LAYER_NUM = 0;
  layers = ALLOC(RoadLayer, LAYER_MAX);
  while((comm=NextCommand(fTec)) != NULL) {

/*  ------------------------------------------------------------
 *  TECHNOLOGY NAME
 *  In the current version it is not used
 */
    if(strcasecmp(comm, "TECHNOLOGY") == 0) {
      if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
      initMessage("Technology is %s", arg);
      fflush(stderr);
      continue;
    }

/*  ------------------------------------------------------------
 *  AVAILABLE LAYER
 */
    if(strcasecmp(comm, "LAYER") == 0) {
      Lnum = LAYER_NUM++;
      if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
      layers[Lnum].name = util_strsav( arg );
/* Defaults  */
      layers[Lnum].via_resistance  = 1.0;
      layers[Lnum].via_capacitance = 0.0;
      layers[Lnum].masking_layers  = NULL;
      layers[Lnum].bulk_dist       = 0;
      layers[Lnum].square_res      = 0;
      layers[Lnum].via_width       = 0;
      layers[Lnum].wire_dist       = 0;
      layers[Lnum].wire_thickness  = 0;
      layers[Lnum].wire_width      = 0;
      layers[Lnum].preferDir  = NODIRECTION;

      loop {
	if((comm=NextCommand(fTec)) == NULL) ENDOFFILE;

	if(strcasecmp(comm, "ENDLAYER") == 0) break;
	
	if(strcasecmp(comm, "TYPE") == 0) {
          if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  if(strcasecmp(arg, "DIFF") == 0) layers[Lnum].type = TYPE_DIFF;
	  if(strcasecmp(arg, "POLY") == 0) layers[Lnum].type = TYPE_POLY;
	  if(strcasecmp(arg, "MET1") == 0) layers[Lnum].type = TYPE_MET1;
	  if(strcasecmp(arg, "POL2") == 0) layers[Lnum].type = TYPE_POL2;
	  if(strcasecmp(arg, "MET2") == 0) layers[Lnum].type = TYPE_MET2;
	  continue;
	}

	if(strcasecmp(comm, "SQR_RES") == 0) {
          if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  if(a == 0) TechnError;
	  layers[Lnum].square_res = a;      /*  In Ohm  */
	  continue;
	}

	if(strcasecmp(comm, "VIA_RES") == 0) {
          if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  if(a == 0) TechnError;
	  layers[Lnum].via_resistance = a;  /*  In Ohm  */
	  continue;
	}

	if(strcasecmp(comm, "VIA_CAP") == 0) {
          if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  if(a == 0) TechnError;
	  layers[Lnum].via_capacitance = a;   /* in pFarad  */
	  continue;
	}

	if(strcasecmp(comm, "BULK_DIST") == 0) {
	  if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  layers[Lnum].bulk_dist = a;      /*  in micron  */
	  continue;
	}

	if(strcasecmp(comm, "MIN_WIDTH") == 0) {
	  if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  layers[Lnum].wire_width = MicronToRoad(a);    /* in micron  */
	  continue;
	}

	if(strcasecmp(comm, "VIA_WIDTH") == 0) {
	  if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  layers[Lnum].via_width = MicronToRoad(a);     /* in micron  */
	  continue;
	}

	if(strcasecmp(comm, "THICKNESS") == 0) {
	  if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  layers[Lnum].wire_thickness = a;            /*  in micron  */
	  continue;
	}

	if(strcasecmp(comm, "SPACING") == 0) {
	  if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	  sscanf(arg, "%f", &a);
	  layers[Lnum].wire_dist = MicronToRoad(a);  /*  in micron  */
	  continue;
	}

	if(strcasecmp(comm, "MASK") == 0) {
	  for(k=0 ; ; ) {
	    if((arg=NextCommand(fTec)) == NULL) ENDOFFILE;
	    if(strcasecmp(arg, "ENDMASK") == 0) break;
	    layers[Lnum].masking_layers =
	      REALLOC(char *, layers[Lnum].masking_layers, k+2);
	    layers[Lnum].masking_layers[k] = NULL;
	    layers[Lnum].masking_layers[k] = util_strsav( arg );
	    layers[Lnum].masking_layers[++k] = NULL;
	  }
	  continue;
	}

	ENDOFFILE;
      }
      continue;
    }

    if(strcasecmp(comm, "DELTA") == 0) {
      if((arg=NextCommand(fTec)) == NULL)   ENDOFFILE;
      continue;
    }

    if(strcasecmp(comm, "NAMES") == 0) {
      loop {
	if((arg=NextCommand(fTec)) == NULL)   ENDOFFILE;
	if(strcasecmp(arg, "ENDNAMES") == 0)  break;
      }
      continue;
    }

    ENDOFFILE;
  }

  fclose(fTec);
/*  ------------------------------------------------------------
 *  Checks
 */
  for(i=0 ; i<LAYER_NUM ; i++) {
    if(layers[i].square_res == 0)    MISSING_MODIFIER("SQR_RES",   i);
    if(layers[i].wire_width == 0)    MISSING_MODIFIER("MIN_WIDTH", i);
    if(layers[i].wire_thickness== 0) MISSING_MODIFIER("THICKNESS", i);
    if(layers[i].wire_dist == 0)     MISSING_MODIFIER("SPACING",   i);
    if(layers[i].bulk_dist <= 0)     MISSING_MODIFIER("BULK_DIST", i);
    if(layers[i].via_width == 0)
      layers[i].via_width = layers[i].wire_width * 2.0;
  }

/*  ------------------------------------------------------------
 *  Derived items
 */
  for(i=0 ; i<LAYER_NUM ; i++)
    layers[i].unity_cap = LTG_XxxModel((micron) layers[i].bulk_dist,
				       (micron) layers[i].wire_thickness,
				       (micron) 1, (micron) 1);
}
