/*       windows.c
 *
 *       Author: Enrico Malavasi
 *       Ver.  : 3.20
 *       Last revised: August 1991
 *
 *       Purpose: Opening and management of windows.
 */
#define  FIRSTXDEF  1
#include <X11/cursorfont.h>
#include "graph.h"
#include "dds.h"
#include "xpa.h"
#include "transf.h"

extern int  DumpBlock;

static xpa_menu   MYMENU;

#define   LEFTUPPER_X   0
#define   LEFTUPPER_Y   0

void
  rdGraphicInit(argc, argv)
int    argc;
char **argv;
{
    char  **items, **getTitleNames();
    int        i, sel;
    int        MAINScreen;
    xpa_entry  entries[20];
    XGCValues  values;
    Display   *d;

    if((d=XOpenDisplay(NULL)) == NULL) {
	errRaise("ROAD", 1, "Error opening display.\n");
    }
    MAINDisplay  = (*d);
    MAINScreen   = XDefaultScreen(&MAINDisplay);
    MAINColormap = XDefaultColormap(&MAINDisplay, MAINScreen);
    defaults();    /*  Set colors and sizes  */

/*  1.) Create the general window  */
    ROADWindow = XCreateSimpleWindow(&MAINDisplay,
				     XDefaultRootWindow(&MAINDisplay),
				     LEFTUPPER_X, 0, ZX1+200, ZY1, 1,
				     color_set.Obstacles, color_set.BackGround);
/*  2.) Graphic (MAIN) window   */
    MAINWindow = XCreateSimpleWindow(&MAINDisplay,
				     ROADWindow,
				     LEFTUPPER_X, LEFTUPPER_Y,
				     ZX1, ZY1, 1,
				     color_set.Obstacles, color_set.BackGround);
    XSetWindowColormap(&MAINDisplay, MAINWindow, MAINColormap);
    values.foreground = color_set.Obstacles;
    values.background = color_set.BackGround;
    MAINGc = XCreateGC(&MAINDisplay, MAINWindow, GCForeground | GCBackground, &values);
    XSelectInput(&MAINDisplay,  MAINWindow, ExposureMask|ButtonPressMask);
    
    /*  3.) Parameter window     */
    PARAWindow = XCreateSimpleWindow(&MAINDisplay,
				     ROADWindow,
				     ZX1+1, LEFTUPPER_Y, 200, ZY1, 1,
				     color_set.Obstacles, color_set.BackGround);
    XSetWindowColormap(&MAINDisplay, PARAWindow, MAINColormap);
    values.foreground = color_set.Obstacles;
    values.background = color_set.BackGround;
    PARAGc = XCreateGC(&MAINDisplay, PARAWindow,
		       GCForeground | GCBackground,
		       &values);
    /*  XSelectInput(&MAINDisplay, PARAWindow, ButtonPressMask); */
    /*  ShowParamWindow(); */
    
    /*  Map all windows and set the cursor    */  
    XMapRaised(&MAINDisplay, ROADWindow);
    XMapRaised(&MAINDisplay, MAINWindow);
    XMapRaised(&MAINDisplay, PARAWindow);

    XDefineCursor(&MAINDisplay, MAINWindow, XCreateFontCursor(&MAINDisplay, XC_plus));
    
    items = getTitleNames();
    for(i=0 ; ; i++) {
	entries[i].item_name = items[i];
	entries[i].key_char  = '\0';
	entries[i].sub_title = NULL;
	entries[i].sub_entrys= NULL;
	if( entries[i].item_name == NULL )  break;
    }

    MYMENU = xpa_create(&MAINDisplay, "ROAD", entries, 0, NULL);
    dds_initialize(&argc, argv);
}

/*  ============================================================
 *  This function prints a text row on the parameter window.
 */
#define  ROWHEIGHT           16
#define  MAXTEXTLENGTH       30
static void
  PrintParamText(str, row)
char   *str;
int     row;
{
  char TEXT[MAXTEXTLENGTH];

  strcpy(TEXT, str);
  testo(5, 10+row*ROWHEIGHT, TEXT, PARAWindow);
}

/*  ============================================================
 *  This is the function called by the exposure event of the
 *  parameter window.
 */
ShowParamWindow()
{
    int      Net, row=0;
    PathName s;

    if(RD_CHECK_FLAG( BATCH_MODE ))    return;
    PrintParamText("Permanent structures", row++);
    sprintf(s, "Layers:      %d", LAYER_NUM);  PrintParamText(s, row++);
    sprintf(s, "Obstacles :  %d", SUB_CELL_NUM);PrintParamText(s, row++);
    sprintf(s, "Nets  :      %d", NET_NUM);    PrintParamText(s, row++);
    row++;
    sprintf(s, "Temporary structures");        PrintParamText(s, row++);
    sprintf(s, "Grid nodes:  %d", TOTAL_RECT_NUM);PrintParamText(s, row++);
    sprintf(s, "Routed nets: %d", WIRE_NUM);   PrintParamText(s, row++);
    sprintf(s, "Dump Block:  %d", DumpBlock);  PrintParamText(s, row++);
    row++;
    sprintf(s, "SUGG. NET:  %s        ",
	    ((Net = SuggestNextNet()) == NOTFOUND) ?
	    "            " : rd_net[Net].name);
    PrintParamText(s, row++);
}

#define   DELETEROW   "          "
ShowWorkingWindow(action, NetNo)
char  *action;
int    NetNo;
{ 
  if(action != NULL)
    fprintf(stdout, "%s %s ... ", action,  rd_net[NetNo].name); fflush(stdout);

  unless(status.General & BATCH_MODE) {
    if(action == NULL)  action = DELETEROW;
    testo(5, ZY1-10-ROWHEIGHT*2, action, PARAWindow);
    testo(5, ZY1-10-ROWHEIGHT,   action, PARAWindow);
    RoadSync(1);
  }
}

/*  ============================================================
 *  XPA menu management
 *  ============================================================ */
int handler(evt)
XEvent evt;
{
  return 0;
}

void finish(item)
ddsHandle item;
{
    dds_unpost(item);
}				

int other(evt, data)
XEvent *evt;
ddsData data;
{
    return 0;
}

int which_net(title, items)
char  *title;
char **items;			/* Null terminated array */
{
    static ddsComposite m_attr = { "", DDS_VERTICAL, NODE_MAX };
    static ddsEvtHandler h_evt = { other, 0 };
    ddsFlagItem *flag_items;
    ddsHandle top, it, btn;
    ddsItemList flag;
    ddsControl control;
    ddsPosition pos;
    char **idx;
    int i;

    m_attr.title = title;
    top = dds_component(0, DDS_TOP, (ddsData) &m_attr);
    for (idx = items;  *idx;  idx++) {
	/* Null body -- counts items */
    }
    flag_items = ALLOC(ddsFlagItem, (idx - items)+1);
    for (i = 0;  i < (idx-items);  i++) {
	flag_items[i].label = items[i];
	flag_items[i].selected_p = DDS_FALSE;
    }
    flag.num_items = (idx - items);
    flag.items = flag_items;
    flag.exclusive_p = DDS_TRUE;
    flag.selected = -1;		/* None selected */
    flag.scroll_p = DDS_TRUE;
    flag.max_items = 20;
    flag.callback = 0; flag.user_data = 0;
    it = dds_component(top, DDS_ITEM_LIST, (ddsData) &flag);
    control.name = "Ok";
    control.accelerator = "<Key>Return";
    control.callback = finish;
    control.user_data = 0;
    btn = dds_component(top, DDS_CONTROL, (ddsData) &control);
    pos.pos = DDS_MOUSE;  pos.vjust = DDS_MIDDLE;  pos.hjust = DDS_CENTER;
    dds_post(top, &pos, &h_evt);
    dds_get(it, (ddsData) &flag);
    dds_destroy(top);
    FREE(flag_items);
    return flag.selected;
}

int which_comm()
{
    int *sel[1];
    if(xpa_moded(MYMENU, 760, 500, 0, 0, XPA_PRESS, handler, sel) > 0)
      return(*sel[0]);
    else  return(-1);
}
