/*         connReduce.c
 *
 *         Author: Enrico Malavasi
 *         Last revised:  July 1991
 */
#include "port.h"
#include "copyright.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "fang.h"
#include "harpoon.h"
#include "errtrap.h"
#include "utility.h"

#define   SWAP(t,a,b)  {t x; x = a; a = b; b = x; }
#define   min(a,b)     (((a) < (b)) ? (a) : (b))
#define   max(a,b)     (((a) < (b)) ? (b) : (a))

connReduce(net)
octObject  *net;
{
    /*  Reduce the number of connectors in the path implementing the
     *  given net to the minimum possible.
     */
    octGenerator pathGen, segmGen, termGen;
    octObject    path, conn;
    octObject    segm1, segm2, newSeg;
    octObject    layer;
    octObject    term;
    struct octPoint  pts1[2], pts2[2], *pts;
    octCoord     x[2], y[2];
    int    n;

    OH_ASSERT(octInitGenContents(net, OCT_PATH_MASK, &pathGen));
    while(octGenerate(&pathGen, &path) == OCT_OK) {
	OH_ASSERT(octInitGenContents(&path, OCT_TERM_MASK, &termGen));
	while(octGenerate(&termGen, &term) == OCT_OK) {
	    
	    if(symConnectorp(&term) == 0)
	      continue;   /* Not a connector */
	    
	    OH_ASSERT(octInitGenContainers(&term, OCT_PATH_MASK, &segmGen));
	    if(octGenerate(&segmGen, &segm1) == OCT_OK)
	      if(octGenerate(&segmGen, &segm2) == OCT_OK) {
		  
		  if(octGenerate(&segmGen, &path) == OCT_OK)
		    continue;   /* More than two segments connected to term */
		  
		  OH_ASSERT(octGenFirstContainer(&segm1, OCT_LAYER_MASK, &layer));
		  if(octIsAttached(&layer, &segm2) != OCT_OK)
		    continue;   /* It is a via  */
		  
		  ohGetPoints(&segm1, &n, &pts);
		  pts1[0] = pts[0]; pts1[1] = pts[1];
		  ohGetPoints(&segm2, &n, &pts);
		  pts2[0] = pts[0]; pts2[1] = pts[1];
		  
		  if(segm1.contents.path.width != segm2.contents.path.width)
		    continue;   /*  Different width  */
		  
		  if(pts1[0].x == pts1[1].x) {
		      if(pts2[0].x != pts2[1].x)
			continue;   /* The second one is horizontal  */
		      if(pts2[0].x != pts1[1].x)
			continue;   /*  Not aligned  */
		      
		      /*  Both are vertical  */
		      y[0] = min(pts1[0].y, pts1[1].y);
		      y[0] = min(y[0], pts2[0].y);
		      y[0] = min(y[0], pts2[1].y);
		      y[1] = max(pts1[0].y, pts1[1].y);
		      y[1] = max(y[1], pts2[0].y);
		      y[1] = max(y[1], pts2[1].y);
		      x[0] = x[1] = pts1[0].x;
		  } else if(pts1[0].y == pts1[1].y) {
		      if(pts2[0].y != pts2[1].y)
			continue;   /* The second one is vertical  */
		      if(pts2[0].y != pts1[1].y)
			continue;   /*  Not aligned  */
		      
		      /*  Both are horizontal  */
		      x[0] = min(pts1[0].x, pts1[1].x);
		      x[0] = min(x[0], pts2[0].x);
		      x[0] = min(x[0], pts2[1].x);
		      x[1] = max(pts1[0].x, pts1[1].x);
		      x[1] = max(x[1], pts2[0].x);
		      x[1] = max(x[1], pts2[1].x);
		      y[0] = y[1] = pts1[0].y;
		  } else
		    continue;
		  
		  ohCreateSegment(net, &newSeg, x[0], y[0], x[1], y[1],
				  segm1.contents.path.width);
		  octAttach(&layer, &newSeg);
		  OH_ASSERT(octGenFirstContainer(&term, OCT_INSTANCE_MASK, &conn));
		  octDelete(&conn);
		  ohAttachContents(&segm1, &newSeg, OCT_ALL_MASK);
		  ohAttachContents(&segm2, &newSeg, OCT_ALL_MASK);
		  octDelete(&segm1);
		  octDelete(&segm2);
	      }
	}
    }
}
