/*       roadtooct.c
 *
 *       Author: Enrico Malavasi
 *       Ver.  : 3.00   -   Last revised: July 1991
 *
 *       Purpose: Interface program RoAD -> oct
 */
#define  FIRSTDEF  1
#include "port.h"
#include "oct.h"
#include "oh.h"
#include "utility.h"
#include "errtrap.h"
#include "options.h"
#include "vov.h"
#include "roadGlobals.h"
#include "roadTechnol.h"
#include  "transf.h"

#define  _CLEANUP  COMMON_FLAG

EXTERN void  rtoSaveAll();

#define  DEFAULT_OUTPUT_CELL_VIEW     "routed"
#define   NDF_EXTENSION   "ndf"

optionStruct optionList[] = {
    {"i",  "file-name",  "Input dump file name"},
    {"o",  "cell-name",  "Output cell [default: `name':routed]"},
    {"c",  "",           "Do not clean up final wires"},
    {"D",  "",           "Debug option"},
    {"v",  "",           "Verbose mode"},
    {"T", "file-name",   "Technology file"},
    { OPT_RARG,	"name",	 "Circuit name" },
    { 0, 0, 0 }
};

main(argc,argv)
int argc;
char **argv;
{
    char *technologyFile;
    char *circuitName = NULL;
    char *inputFile   = NULL;
    char *outputCell  = NULL;
    char option;
    float scal;

    fprintf(stderr, "road2oct: Interface RoAD to Oct");
    fprintf(stderr, "\nVersion %s\n", ROAD_VERSION);
    errProgramName(argv[0]);                /*  For the error-trapping  */
    
    RD_RESET_ALLFLAGS;
    RD_SET_FLAG(NET_READ);
    RD_SET_FLAG(_CLEANUP);
    technologyFile = util_strsav( TECHNOLFILE );
    while ( ( option = optGetOption (argc, argv)) != EOF) {
	switch( option ) {
	  case 'D':
	    RD_SET_FLAG(DEBUG_OPTION);              break;
	  case 'c':
	    RD_RESET_FLAG(_CLEANUP);                break;
	  case 'o':
	    outputCell = util_strsav( optarg );     break;
	  case 'i':
	    inputFile = util_strsav( optarg );      break;
	  case 'T':
	    technologyFile = util_strsav( optarg ); break;
	  case 'v':
	    RD_SET_FLAG(RD_VERBOSE_FLAG);           break;
	}
    }
    if(optind != (argc-1)) optUsage();
    circuitName = util_strsav( argv[optind] );
    if(outputCell == NULL)  outputCell = util_strsav( circuitName );

    if( inputFile == NULL)   inputFile = LastDumpFileName( circuitName );
    VOVbegin( argc, argv );
    VOVinput( VOV_UNIX_FILE, inputFile );
    ReadRoadDump( inputFile );

    VOVinput( VOV_UNIX_FILE, technologyFile );
    read_technol( technologyFile );   /* Technology Data File input */

    {   char  *ndfFile;
	ndfFile = ALLOC( char, strlen( circuitName ) + strlen(NDF_EXTENSION) + 2);
	sprintf(ndfFile, "%s.%s", circuitName, NDF_EXTENSION);
	input_netlist( ndfFile );     /* Net Descriptor Data input  */
    }
    roadGeneralCheck();
    
    rtoSaveAll( outputCell );               /*  Write the oct cell.   */
    printf("\nDone\n");
    rdExit( 0 );
}

rdExit( s )
int s;
{
    VOVend( s );
}

#include "dummy.h"
