
/*
 * have a symbolicp flag, so don't always have to get the net...
 * physical drc...
 */

#include "copyright.h"
#include "port.h"
#include "rpc.h"
#include "region.h"
#include "tap.h"
#include "oh.h"

char *optProgName = "drc";
octCoord Lambda;

/*ARGSUSED*/  
int
drcExit(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    RPCExit(0);
    return RPC_OK;
}


RPCFunction CommandArray[] = {
	{drcExit, "drc", "exit", "X", 10},
};


octStatus
changedObject(facet, cr, obj)
octObject *facet, *cr, *obj;
{
    octStatus status;

    status = octGetByExternalId(facet, cr->contents.changeRecord.objectExternalId, obj);
    if (status == OCT_OK) {
	return OCT_OK;
    }
    status = octGenFirstContent(cr, OCT_ALL_MASK, obj);
    return status;
}

void
registerGeoCheck();

#define MYABS(a) ((a) > 0 ? (a) : -(a))
#define MYMIN(a, b) (((a) > (b)) ? (b) : (a))


void
checkLayers(object, bb, objLayer, facet)
octObject *object;   /* new object */
octBox *bb;          /* bb of the new object */
octObject *objLayer; /* layer of the new object */
octObject *facet;    /* current facet or master of instance */
{
    octGenerator lgen;
    regGenerator rgen;
    octObject layer, ngbr, olayer, net, onet, term;
    octBox tbb, nbb;
    int np;
    octPoint *pts;
    char buffer[1024];
    octCoord space;

    /* XXX this sucks... */
    if (ohGetByLayerName(facet, &olayer, objLayer->contents.layer.name) < OCT_OK) {
	return;
    }

    /* get the containing net */
    if (octGenFirstContainer(object, OCT_NET_MASK, &onet) < OCT_OK) {
	onet.objectId = oct_null_id;
    }

    tbb = *bb;

    /* check each layer */
    octInitGenContentsSpecial(facet, OCT_LAYER_MASK, &lgen);
    while (octGenerate(&lgen, &layer) == OCT_OK) {
	/* sad, eh? */
	if (strcmp(layer.contents.layer.name, "PLACE") == 0) {
	    continue;
	}
	/* get the design rule */    
	space = tapGetMinSpacing(&olayer, &layer);
	if (space <= 0) {
	    continue;
	}
	tbb.lowerLeft.x -= space;  tbb.lowerLeft.x++;
	tbb.lowerLeft.y -= space;  tbb.lowerLeft.y++;
	tbb.upperRight.x += space; tbb.upperRight.x--;
	tbb.upperRight.y += space; tbb.upperRight.y--;
	regInit(&layer, &tbb, OCT_GEO_MASK, &rgen);
	while (regNext(&rgen, &ngbr, &np, &pts, &nbb) == REG_OK) {
	    if (object->objectId != ngbr.objectId) {
		/* see if it is a terminal frame ... XXX kludge */
		if (octGenFirstContainer(&ngbr, OCT_TERM_MASK, &term) == OCT_OK) {
		    continue;
		}
		/* see if it is on the same net and layer */
		if ((onet.objectId != oct_null_id) &&
		    (layer.objectId == olayer.objectId)) {
		    if (octGenFirstContainer(&ngbr, OCT_NET_MASK, &net) == OCT_OK) {
			if (net.objectId == onet.objectId) {
			    continue;
			}
		    }
		}
		(void) sprintf(buffer, "DRC: %s-%s spacing violation, must be at least %d lambda\n",
			       olayer.contents.layer.name,
			       layer.contents.layer.name,
			       space / Lambda);
		VEMMSG(buffer);
	    }
	}
	regEnd(&rgen);
    }
    return;
}


void
geoCreateCheck(cl)
octObject *cl;
{
    octObject facet, cr, obj, olayer, inst, master;
    octGenerator gen;
    octBox bb;
    octCoord width;
    regObjGen rgen;
    char buffer[1024];

    octGetFacet(cl, &facet);
    octInitGenContentsSpecial(cl, OCT_CHANGE_RECORD_MASK, &gen);
    while (octGenerate(&gen, &cr) == OCT_OK) {
	if (changedObject(&facet, &cr, &obj) == OCT_OK) {
	    /* check for stuff in the area */
	    if (octBB(&obj, &bb) == OCT_OK) {
		/* get layer of obj */
		if (octGenFirstContainer(&obj, OCT_LAYER_MASK, &olayer) != OCT_OK) {
		    continue;
		}
		/* check min width rule */
		width = tapGetMinWidth(&olayer);
		if (width > 0 && width > MYMIN(MYABS(bb.upperRight.y - bb.lowerLeft.y),
				  MYABS(bb.upperRight.x - bb.lowerLeft.x))) {
		    (void) sprintf(buffer, "DRC: min width violation, must be at least %d lambda\n", width / Lambda);
		    VEMMSG(buffer);
		}

		checkLayers(&obj, &bb, &olayer, &facet);

		/* check for surrounding instances - need max rule */
		bb.lowerLeft.x -= 100;
		bb.lowerLeft.y -= 100;
		bb.upperRight.x += 100;
		bb.upperRight.y += 100;
		if (regObjStart(&facet, &bb, OCT_INSTANCE_MASK, &rgen, 0) == REG_OK) {
		    while (regObjNext(rgen, &inst) == OCT_OK) {
			octBox tbb;
			tbb = bb;
			/* transform bb to masters coordinate space */
			tr_oct_inverse_transform(&inst.contents.instance.transform,
						 &tbb.lowerLeft.x,
						 &tbb.lowerLeft.y);
			tr_oct_inverse_transform(&inst.contents.instance.transform,
						 &tbb.upperRight.x,
						 &tbb.upperRight.y);
			/* get the master */
			master.type = OCT_FACET;
			master.contents.facet.facet = "interface";
			master.contents.facet.mode = "r";
			if (octOpenMaster(&inst, &master) >= OCT_OK) {
			    checkLayers(&obj, &tbb, &olayer, &master);
			}
		    }
		    regObjFinish(rgen);
		}
	    }
	}
    }
    (void) octDelete(cl);
    registerGeoCheck(&facet);
    return;
}


void
registerGeoCheck(facet)
octObject *facet;
{
    octObject changeList;

    changeList.type = OCT_CHANGE_LIST;
    changeList.contents.changeList.objectMask = OCT_GEO_MASK|OCT_INSTANCE_MASK;
    changeList.contents.changeList.functionMask = OCT_ATTACH_CONTENT_MASK|OCT_CREATE_MASK|OCT_MODIFY_MASK|OCT_PUT_POINTS_MASK;
    (void) octCreate(facet, &changeList);
    RPCRegisterDemon(&changeList, geoCreateCheck);
    return;
}


/*ARGSUSED*/
int
UserMain(display, spot, cmdList, userOptionWord, array)
char *display;
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
RPCFunction *array[];
{
    octObject facet;
    
    facet.objectId = spot->facet;
    (void) octGetById(&facet);

    if (tapGetProp(&facet, TAP_LAMBDA)) {
	Lambda = TAP_LAMBDA->value.integer;
    } else {
	Lambda = 20;
    }

    registerGeoCheck(&facet);
    *array = CommandArray;
    VEMMSG("This program does incremental design-rule checks\n");
    VEMMSG("on physical designs - it does not deal with\n");
    VEMMSG("electrical connectivity, yet...\n");
    VEMMSG("DRC initialized\n");
    vemPrompt();
    return(sizeof(CommandArray) / sizeof(RPCFunction));
}
