/*LINTLIBRARY*/
#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "rpcApp.h"

octStatus	RPCOctGenerate();
octStatus	RPCOctGenerateDone();
octStatus	RPCOctGenerateError();
octStatus	RPCOctGenerateSpecial();


/*
 * handle OCT functions that create generators
 */
static octStatus
RPCoctInitGen(type, container, mask, generator, sendStream, receiveStream)
long type;
octObject *container;
octObjectMask mask;
octGenerator *generator;
STREAM sendStream;
STREAM receiveStream;
{
    long returnCode;
    octGenerator *realGenerator;

    realGenerator = ALLOC(octGenerator, 1);

    RPCASSERT(RPCSendLong(type, sendStream));
    RPCASSERT(RPCSendLong((long) container->objectId, sendStream));
    RPCASSERT(RPCSendLong((long) mask, sendStream));

    RPCFLUSH(sendStream);

    RPCASSERT(RPCReceiveLong(&returnCode, receiveStream));
    RPCASSERT(RPCReceiveOctGenerator(realGenerator, receiveStream));

    generator->generator_func = RPCOctGenerate;
    generator->state = (char *) realGenerator;

    return (octStatus) returnCode;
}

#define MAXCACHESIZE 1000

struct rpcGenerator {
    octGenerator *generator;
    octObject object[MAXCACHESIZE];
    octStatus status[MAXCACHESIZE];
    long count;
};

    
static octStatus
RPCoctInitGenSpecial(type, container, mask, generator, sendStream, receiveStream)
long type;
octObject *container;
octObjectMask mask;
octGenerator *generator;
STREAM sendStream;
STREAM receiveStream;
{
    long returnCode;
    octGenerator *realGenerator;
    struct rpcGenerator *rpcGenerator;

    realGenerator = ALLOC(octGenerator, 1);
            
    RPCASSERT(RPCSendLong(type, sendStream));
    RPCASSERT(RPCSendLong((long) container->objectId, sendStream));
    RPCASSERT(RPCSendLong((long) mask, sendStream));

    RPCFLUSH(sendStream);

    RPCASSERT(RPCReceiveLong(&returnCode, receiveStream));
    RPCASSERT(RPCReceiveOctGenerator(realGenerator, receiveStream));

    if (returnCode >= OCT_OK) {
	rpcGenerator = ALLOC(struct rpcGenerator, 1);
	rpcGenerator->count = MAXCACHESIZE;
	rpcGenerator->generator = realGenerator;
	generator->generator_func = RPCOctGenerateSpecial;
	generator->state = (char *) rpcGenerator;
    }
    return (octStatus) returnCode;
}



/*
 * basic OCT Calls
 *
 * all work as follows:
 *
 *   send OCT RPC function code
 *   send arguments
 *   wait for return code
 *   wait for return arguments (modified versions of the original arguments)
 */

octStatus
octAttach(object1, object2)
octObject *object1, *object2;
{
    long returnCode = OCT_OK;
    
    RPCASSERT(RPCSendLong((long) OCT_ATTACH_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object2->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octAttachOnce(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_ATTACH_ONCE_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object2->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    return (octStatus) returnCode;
}


octStatus
octIsAttached(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_IS_ATTACHED_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object2->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    return (octStatus) returnCode;
}


octStatus
octDetach(object1, object2)
octObject *object1, *object2;
{
    long returnCode = OCT_OK;
    
    RPCASSERT(RPCSendLong((long) OCT_DETACH_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object2->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octCreate(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_CREATE_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object2, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octDelete(object)
octObject *object;
{
    long returnCode = OCT_OK;
    
    RPCASSERT(RPCSendLong((long) OCT_DELETE_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octCopyFacet(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_COPY_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object1, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object2, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object1, OCTReceiveStream));
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octWriteFacet(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_WRITE_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object1, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object2, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object1, OCTReceiveStream));
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


void
octError(string)
char *string;
{
    RPCASSERT(RPCSendLong(OCT_ERROR_FN_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendString(string, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    return;
}


octStatus
octInitGenContents(container, mask, generator)
octObject *container;
octObjectMask mask;
octGenerator *generator;
{
    return(RPCoctInitGenSpecial(OCT_INIT_GEN_CONTENTS_FUNCTION, container, mask, generator,
				       OCTSendStream, OCTReceiveStream));
}


octStatus
octInitGenContainers(object, mask, generator)
octObject *object;
octObjectMask mask;
octGenerator *generator;
{
    return(RPCoctInitGenSpecial(OCT_INIT_GEN_CONTAINERS_FUNCTION, object, mask, generator,
				       OCTSendStream, OCTReceiveStream));
}


octStatus
octInitGenContentsSpecial(container, mask, generator)
octObject *container;
octObjectMask mask;
octGenerator *generator;
{
    return(RPCoctInitGenSpecial(OCT_INIT_GEN_CONTENTS_FUNCTION, container,
						       mask, generator,
				       OCTSendStream, OCTReceiveStream));
}


octStatus
octInitGenContainersSpecial(object, mask, generator)
octObject *object;
octObjectMask mask;
octGenerator *generator;
{
    return(RPCoctInitGenSpecial(OCT_INIT_GEN_CONTAINERS_FUNCTION, object,
							 mask, generator,
				       OCTSendStream, OCTReceiveStream));
}


/*ARGSUSED*/
octStatus
RPCOctGenerateDone(generator, object)
octGenerator *generator;
octObject *object;
{
    return OCT_GEN_DONE;
}


/*ARGSUSED*/
octStatus
RPCOctGenerateError(generator, object)
octGenerator *generator;
octObject *object;
{
    return OCT_ERROR;
}


octStatus
RPCOctGenerate(generator, object)
octGenerator *generator;
octObject *object;
{
    long returnCode;
    octGenerator *realGenerator;

    realGenerator = (octGenerator *) generator->state;

    RPCASSERT(RPCSendLong(OCT_GENERATE_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctGenerator(realGenerator, OCTSendStream));

    RPCFLUSH(OCTSendStream);

    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));

    RPCASSERT(RPCReceiveOctGenerator(realGenerator, OCTReceiveStream));
    
    if (returnCode == OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
	generator->state = (char *) realGenerator;
    }
    
    if (returnCode == OCT_GEN_DONE) {
	generator->generator_func = RPCOctGenerateDone;
	FREE(generator->state);
        generator->state = NIL(char);
    }

    return (octStatus) returnCode;
}


octStatus
RPCOctGenerateSpecial(generator, object)
octGenerator *generator;
octObject *object;
{
    struct rpcGenerator *rpcGenerator;
    octStatus status;
    long dummy;
    int count;
    octGenerator *realGenerator;

    rpcGenerator = (struct rpcGenerator *) generator->state;

    if (rpcGenerator->count < MAXCACHESIZE) {

	/* Still have oct objects on the remote side */
	status = rpcGenerator->status[rpcGenerator->count];
	if (status == OCT_GEN_DONE) {
	    generator->generator_func = RPCOctGenerateDone;
	    FREE(rpcGenerator);
	    generator->state = NIL(char);
	    return OCT_GEN_DONE;
	}
	if (status != OCT_OK) {
	    generator->generator_func = RPCOctGenerateError;
	    FREE(rpcGenerator);
	    generator->state = NIL(char);
	    return status;
	}
	*object = rpcGenerator->object[rpcGenerator->count++];
	return status;
	
    } else {

	/* Ask for more oct objects */
	realGenerator = rpcGenerator->generator;
	
	RPCASSERT(RPCSendLong(OCT_GENERATE_SPECIAL_FUNCTION, OCTSendStream));
	RPCASSERT(RPCSendOctGenerator(realGenerator, OCTSendStream));
	RPCASSERT(RPCSendLong((long) MAXCACHESIZE, OCTSendStream));
	RPCFLUSH(OCTSendStream);

	count = 0;
	
	while (count < MAXCACHESIZE) {
	    RPCASSERT(RPCReceiveLong(&dummy, OCTReceiveStream));
	    rpcGenerator->status[count] = (octStatus) dummy;
	    
	    if (rpcGenerator->status[count] != OCT_OK) {
		count++;
		break;
	    }
	    RPCASSERT(RPCReceiveOctObject(&(rpcGenerator->object[count++]), OCTReceiveStream));
	}
	RPCASSERT(RPCReceiveOctGenerator(realGenerator, OCTReceiveStream));
	rpcGenerator->count = 0;
	rpcGenerator->generator = realGenerator;
	generator->generator_func = RPCOctGenerateSpecial;
	generator->state = (char *) rpcGenerator;

	status = rpcGenerator->status[rpcGenerator->count];
	if (status == OCT_GEN_DONE) {
	    generator->generator_func = RPCOctGenerateDone;
	    FREE(rpcGenerator);
	    generator->state = NIL(char);
	    return OCT_GEN_DONE;
	}
	if (status != OCT_OK) {
	    generator->generator_func = RPCOctGenerateError;
	    FREE(rpcGenerator);
	    generator->state = NIL(char);
	    return status;
	}
	*object = rpcGenerator->object[rpcGenerator->count++];
	return status;
    }
}


octStatus
octGenFirstContent(object, mask, content)
octObject *object;
octObjectMask mask;
octObject *content;
{
    long returnCode;

    RPCASSERT(RPCSendLong((long) OCT_GEN_FIRST_CONTENT_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));

    RPCASSERT(RPCSendLong((long) mask, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode == OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(content, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octGenFirstContainer(object, mask, container)
octObject *object;
octObjectMask mask;
octObject *container;
{
    long returnCode;

    RPCASSERT(RPCSendLong((long) OCT_GEN_FIRST_CONTAINER_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));

    RPCASSERT(RPCSendLong((long) mask, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode == OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(container, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


/* XXX should be fixed and call the vem side */

octStatus
octFreeGenerator(generator)
octGenerator *generator;
{
    struct rpcGenerator *rpcGenerator;

    rpcGenerator = (struct rpcGenerator *) generator->state;
    if (rpcGenerator != NIL(struct rpcGenerator)) {
        FREE(rpcGenerator);
        generator->state = NIL(char);
    }
    generator->generator_func = RPCOctGenerateError;

    return OCT_OK;
}


octStatus
octGetContainerByName(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_GET_CONTAINER_BY_NAME_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));

    RPCASSERT(RPCSendLong((long) object2->type, OCTSendStream));

    switch (object2->type) {
	case OCT_INSTANCE:
	    RPCASSERT(RPCSendString(object2->contents.instance.name, OCTSendStream));
	    break;

	case OCT_PROP:
	    RPCASSERT(RPCSendString(object2->contents.prop.name, OCTSendStream));
	    break;

	case OCT_TERM:
	    RPCASSERT(RPCSendString(object2->contents.term.name, OCTSendStream));
	    break;

	case OCT_NET:
	    RPCASSERT(RPCSendString(object2->contents.net.name, OCTSendStream));
	    break;

	case OCT_LAYER:
	    RPCASSERT(RPCSendString(object2->contents.layer.name, OCTSendStream));
	    break;

	case OCT_BAG:
	    RPCASSERT(RPCSendString(object2->contents.bag.name, OCTSendStream));
	    break;
    }

    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octGetByName(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_GET_BY_NAME_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));

    RPCASSERT(RPCSendLong((long) object2->type, OCTSendStream));

    switch (object2->type) {
	case OCT_INSTANCE:
	    RPCASSERT(RPCSendString(object2->contents.instance.name, OCTSendStream));
	    break;

	case OCT_PROP:
	    RPCASSERT(RPCSendString(object2->contents.prop.name, OCTSendStream));
	    break;

	case OCT_TERM:
	    RPCASSERT(RPCSendString(object2->contents.term.name, OCTSendStream));
	    break;

	case OCT_NET:
	    RPCASSERT(RPCSendString(object2->contents.net.name, OCTSendStream));
	    break;

	case OCT_LAYER:
	    RPCASSERT(RPCSendString(object2->contents.layer.name, OCTSendStream));
	    break;

	case OCT_BAG:
	    RPCASSERT(RPCSendString(object2->contents.bag.name, OCTSendStream));
	    break;

	case OCT_LABEL:
	    RPCASSERT(RPCSendString(object2->contents.label.label, OCTSendStream));
	    break;
    }

    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octGetById(object)
octObject *object;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_GET_BY_ID_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    }
    
    return (octStatus) returnCode;
}


octStatus
octGetOrCreate(object1, object2)
octObject *object1, *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_GET_OR_CREATE_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object2, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octCreateOrModify(object1, object2)
octObject *object1;
octObject *object2;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_CREATE_OR_MODIFY_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object1->objectId, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object2, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object2, OCTReceiveStream));
    }
    return (octStatus) returnCode;

}


octStatus
octModify(object)
octObject *object;
{
    long returnCode = OCT_OK;
    
    if (object->type == OCT_INSTANCE) {
	RPCASSERT(RPCSendLong((long) OCT_MODIFY_SPECIAL_FUNCTION, OCTSendStream));
	RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
	RPCASSERT(RPCSendString(object->contents.instance.name, OCTSendStream));
	RPCASSERT(RPCSendOctTransform(&(object->contents.instance.transform), OCTSendStream));
	
    } else {
	RPCASSERT(RPCSendLong((long) OCT_MODIFY_FUNCTION, OCTSendStream));
	RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    }
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octOpenFacet(object)
octObject *object;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_OPEN_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


octStatus
octFlushFacet(object)
octObject *object;
{
    long returnCode = OCT_OK;
    
    RPCASSERT(RPCSendLong((long) OCT_FLUSH_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octCloseFacet(object)
octObject *object;
{
    long returnCode = OCT_OK;
    
    RPCASSERT(RPCSendLong((long) OCT_CLOSE_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octOpenMaster(inst, facet)
octObject *inst;
octObject *facet;
{
    long returnCode;

    RPCASSERT(RPCSendLong((long) OCT_OPEN_MASTER_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) inst->objectId, OCTSendStream));
    RPCASSERT(RPCSendString(facet->contents.facet.facet, OCTSendStream));
    RPCASSERT(RPCSendString(facet->contents.facet.mode, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(facet, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}

octStatus
octOpenRelative(rfacet, facet, location)
octObject *rfacet, *facet;
int location;
{
    long returnCode;

    RPCASSERT(RPCSendLong((long) OCT_OPEN_RELATIVE_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(facet, OCTSendStream));
    RPCASSERT(RPCSendLong((long) rfacet->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) location, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(facet, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}
    

octStatus
octFreeFacet(object)
octObject *object;
{
    long returnCode = OCT_OK;
    
    RPCASSERT(RPCSendLong((long) OCT_FREE_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT    
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}


octStatus
octGetPoints(object, num_points, points)
octObject *object;
int *num_points;
struct octPoint *points;
{
    int i;
    long returnCode;
    long sendPoints, dummy;

    RPCASSERT(RPCSendLong(OCT_GET_POINTS_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) *num_points, OCTSendStream));

    if (points != NIL(struct octPoint)) {
	sendPoints = RPC_SEND_POINTS;
    } else {
	sendPoints = RPC_DO_NOT_SEND_POINTS;
    }
    RPCASSERT(RPCSendLong(sendPoints, OCTSendStream));

    RPCFLUSH(OCTSendStream);

    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    RPCASSERT(RPCReceiveLong(&dummy, OCTReceiveStream));
    *num_points = (int) dummy;

    if ((sendPoints == RPC_SEND_POINTS) && (returnCode != OCT_TOO_SMALL)) {

	if (!RPCByteSwapped) {
	    if (fread((char *) points, sizeof(struct octPoint),
		      (int) *num_points, OCTReceiveStream) != *num_points) {
		return OCT_ERROR;
	    }
	} else {
	    for (i = 0; i < *num_points; i++) {
		RPCASSERT(RPCReceiveOctPoint(&points[i], OCTReceiveStream));
	    }
	}	    
    }
    return (octStatus) returnCode;

}


octStatus
octPutPoints(object, num_points, points)
octObject *object;
int num_points;
struct octPoint *points;
{
    int i;
    long returnCode = OCT_OK;

    RPCASSERT(RPCSendLong(OCT_PUT_POINTS_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCASSERT(RPCSendLong((long) num_points, OCTSendStream));

    if (!RPCByteSwapped) {
	if (fwrite((char *) points, sizeof(struct octPoint), (int) num_points, OCTSendStream) != num_points) {
	    return OCT_ERROR;
	}
    } else {
	for (i = 0; i < num_points; i++) {
	    RPCASSERT(RPCSendOctPoint(&(points[i]), OCTSendStream));
	}
    }
    
    RPCFLUSH(OCTSendStream);
#ifndef NOWAIT
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
#endif /* NOWAIT */    
    return (octStatus) returnCode;
}

octStatus
octBB(object, bbox)
octObject *object;
struct octBox *bbox;
{
    long returnCode;

    RPCASSERT(RPCSendLong(OCT_BB_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));

    RPCFLUSH(OCTSendStream);

    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveOctBox(bbox, OCTReceiveStream));
    }

    return (octStatus) returnCode;
}


char *
octErrorString()
{
    char *string;
    static char rpcErrorMessageArea[1024];
    
    RPCASSERT(RPCSendLong(OCT_ERROR_STRING_FUNCTION, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveString(&string, OCTReceiveStream));
    (void) strcpy(rpcErrorMessageArea, string);
    FREE(string);
    return rpcErrorMessageArea;
}
			       

void
octGetFacet(object, facet)
octObject *object;
octObject *facet;
{
    RPCASSERT(RPCSendLong(OCT_GET_FACET_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveOctObject(facet, OCTReceiveStream));
    return;
}


void
octBegin()
{
    extern char *getenv();
    char *srvr;
    srvr = util_strsav(getenv("OCT-SERVER"));
    fprintf(stderr, "using %s for the oct server\n", srvr);
    server_init(srvr);
    return;
}

void
octEnd()
{
    close_server();
    return;
}


void
octTransformGeo(object, transform)
octObject *object;
struct octTransform *transform;
{
    RPCASSERT(RPCSendLong(OCT_TRANSFORM_GEO_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCASSERT(RPCSendOctTransform(transform, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    return;
}


octStatus
octTransformAndModifyGeo(object, transform)
octObject *object;
struct octTransform *transform;
{
    octStatus returnCode = OCT_OK;

    RPCASSERT(RPCSendLong(OCT_TRANSFORM_MODIFY_GEO_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCASSERT(RPCSendOctTransform(transform, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= (long) OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}

void
octTransformPoints(num_pts, pts, xform)
int num_pts;
struct octPoint *pts;
struct octTransform *xform;
{
    long n_pts = (long) num_pts;
    int i;

    RPCASSERT(RPCSendLong(OCT_TRANSFORM_POINTS_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong(n_pts, OCTSendStream));
    for (i = 0; i < n_pts; i++) {
	RPCASSERT(RPCSendOctPoint(&pts[i], OCTSendStream));
    }
    RPCASSERT(RPCSendOctTransform(xform, OCTSendStream));
    for (i = 0; i < n_pts; i++) {
	RPCASSERT(RPCReceiveOctPoint(&pts[i], OCTReceiveStream));
    }
    return;
}


void
octScaleGeo(object, scale)
octObject *object;
double scale;
{
    RPCASSERT(RPCSendLong(OCT_SCALE_GEO_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCASSERT(RPCSendFloat(scale, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    return;
}


octStatus
octScaleAndModifyGeo(object, scale)
octObject *object;
double scale;
{
    long returnCode = OCT_OK;

    RPCASSERT(RPCSendLong(OCT_SCALE_MODIFY_GEO_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCASSERT(RPCSendFloat(scale, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= (long) OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


void
octMarkTemporary(object)
octObject *object;
{
    RPCASSERT(RPCSendLong((long) OCT_MARK_TEMPORARY_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    return;
}


void
octUnmarkTemporary(object)
octObject *object;
{
    RPCASSERT(RPCSendLong((long) OCT_UNMARK_TEMPORARY_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    return;
}


int
octIsTemporary(object)
octObject *object;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_IS_TEMPORARY_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    return (int) returnCode;
}


void
octExternalId(object, xid)
octObject *object;
int32 *xid;
{
    long dummy;

    RPCASSERT(RPCSendLong((long) OCT_XID_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&dummy, OCTReceiveStream));
    *xid = (int32) dummy;
    return;
}


octStatus
octGetByExternalId(facet, xid, object)
octObject *facet;
int32 xid;
octObject *object;
{
    long returnCode;
    
    RPCASSERT(RPCSendLong((long) OCT_GET_BY_XID_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) facet->objectId, OCTSendStream));
    RPCASSERT(RPCSendLong((long) xid, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= (octStatus) OCT_OK) {
	RPCASSERT(RPCReceiveOctObject(object, OCTReceiveStream));
    }
    return (octStatus) returnCode;
}


void
octFullName(object, name)
octObject *object;
char **name;
{
    RPCASSERT(RPCSendLong((long) OCT_FULL_NAME_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendLong((long) object->objectId, OCTSendStream));
    RPCFLUSH(OCTSendStream);
    RPCASSERT(RPCReceiveString(name, OCTReceiveStream));
    return;
}


octStatus
octGetFacetInfo(object, info)
octObject *object;
struct octFacetInfo *info;
{
    long returnCode;

    RPCASSERT(RPCSendLong((long) OCT_GET_FACET_INFO_FUNCTION, OCTSendStream));
    RPCASSERT(RPCSendOctObject(object, OCTSendStream));
    RPCFLUSH(OCTSendStream);

    RPCASSERT(RPCReceiveLong(&returnCode, OCTReceiveStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCReceiveFacetInfo(info, OCTReceiveStream));
    }
    return returnCode;
}


/*ARGSUSED*/
octStatus
octPrintObject(stream, object)
FILE *stream;
octObject *object;
{
    fprintf(stderr, "octPrintObject is not implemented... yet\n");
    return OCT_OK;
}
