
#define OCT_SERVER_VERSION	(long) 1

#define RPC_PKG_NAME "oct-server"

#define NOWAIT

/* can not use fread in NOWAIT mode */
#ifdef SERVER  
#ifdef NOWAIT
#define fread	RPCfread
#endif /* NOWAIT */
#endif /* SERVER */

extern int RPCByteSwapped;

typedef FILE *STREAM;

#ifndef RPCLOCATION
#define RPCLOCATION "unknown"
#endif

#include "errtrap.h"

#define RPCASSERT(function_and_args) \
    if (!function_and_args) { \
	char errbuffer[1024]; \
        (void) sprintf(errbuffer, "RPC Error: %s: %s (line %d): function_and_args", RPCLOCATION, __FILE__, __LINE__); \
	perror(errbuffer); \
	errRaise(RPC_PKG_NAME, -1, errbuffer); \
    }

#define RPCFLUSH(stream) \
    if (fflush(stream) == EOF) { \
	char errbuffer[1024]; \
        (void) sprintf(errbuffer, "RPC Error: %s: fflush", RPCLOCATION); \
	perror(errbuffer); \
	errRaise(RPC_PKG_NAME, -1, errbuffer); \
    }


/* for brain damaged octGetPoints */
#define RPC_SEND_POINTS		1
#define RPC_DO_NOT_SEND_POINTS	0

/* for octError */
#define RPC_OCT_ERROR_END	(long) -10000

/*
 * OCT RPC function codes
 */
#define OCT_ATTACH_FUNCTION			(long) 1
#define OCT_DETACH_FUNCTION			(long) 2
#define OCT_COPY_FACET_FUNCTION			(long) 3
#define OCT_WRITE_FACET_FUNCTION		(long) 4
#define OCT_CREATE_FUNCTION			(long) 5
#define OCT_DELETE_FUNCTION			(long) 6
#define OCT_ERROR_FN_FUNCTION			(long) 7
#define OCT_INIT_GEN_CONTENTS_FUNCTION		(long) 8
#define OCT_INIT_GEN_CONTAINERS_FUNCTION	(long) 9
#define OCT_GENERATE_FUNCTION			(long) 10
#define OCT_GET_BY_NAME_FUNCTION		(long) 11
#define OCT_GET_BY_ID_FUNCTION			(long) 12
#define OCT_GET_OR_CREATE_FUNCTION		(long) 13
#define OCT_CREATE_OR_MODIFY_FUNCTION		(long) 14
#define OCT_MODIFY_FUNCTION			(long) 15
#define OCT_OPEN_FACET_FUNCTION			(long) 16
#define OCT_FLUSH_FACET_FUNCTION		(long) 17
#define OCT_CLOSE_FACET_FUNCTION		(long) 18
#define OCT_FREE_FACET_FUNCTION			(long) 19
#define OCT_GET_POINTS_FUNCTION			(long) 20
#define OCT_PUT_POINTS_FUNCTION			(long) 21
#define OCT_BB_FUNCTION				(long) 22

#define OCT_GENERATE_SPECIAL_FUNCTION		(long) 25
  
#define OCT_MODIFY_SPECIAL_FUNCTION		(long) 29

#define OCT_GET_FACET_FUNCTION			(long) 30
#define OCT_ERROR_STRING_FUNCTION		(long) 31
  
#define OCT_ATTACH_ONCE_FUNCTION		(long) 32
#define OCT_IS_ATTACHED_FUNCTION		(long) 33
#define OCT_OPEN_MASTER_FUNCTION		(long) 34

#define OCT_GET_CONTAINER_BY_NAME_FUNCTION	(long) 35
#define OCT_GEN_FIRST_CONTENT_FUNCTION		(long) 36
#define OCT_GEN_FIRST_CONTAINER_FUNCTION	(long) 37
#define OCT_FREE_GENERATOR_FUNCTION		(long) 38

#define OCT_TRANSFORM_GEO_FUNCTION		(long) 39
#define OCT_TRANSFORM_MODIFY_GEO_FUNCTION	(long) 40
#define OCT_TRANSFORM_POINTS_FUNCTION		(long) 41	
#define OCT_SCALE_GEO_FUNCTION			(long) 42
#define OCT_SCALE_MODIFY_GEO_FUNCTION		(long) 43

#define OCT_MARK_TEMPORARY_FUNCTION		(long) 44
#define OCT_UNMARK_TEMPORARY_FUNCTION		(long) 45
#define OCT_IS_TEMPORARY_FUNCTION		(long) 46

#define OCT_XID_FUNCTION			(long) 47
#define OCT_GET_BY_XID_FUNCTION			(long) 48

#define OCT_FULL_NAME_FUNCTION			(long) 49
#define OCT_GET_FACET_INFO_FUNCTION		(long) 50

#define OCT_OPEN_RELATIVE_FUNCTION		(long) 51
  
/*
 * integer and real oct arrays
 */

struct octIntegerArray {
    int length;
    int *array;
};

struct octRealArray {
    int length;
    double *array;
};


/* receive data across a socket */
extern int	RPCReceiveLong();
extern int	RPCReceiveByte();
extern int	RPCReceiveFloat();
extern int	RPCReceiveOctGenerator();
extern int	RPCReceiveOctObject();
extern int	RPCReceiveOctBox();
extern int	RPCReceiveOctPoint();
extern int	RPCReceiveOctProp();
extern int	RPCReceiveOctTransform();
extern int	RPCReceiveString();
extern int      RPCReceiveIntegerArray();
extern int      RPCReceiveRealArray();

/* send data across a socket */
extern int	RPCSendLong();
extern int	RPCSendByte();
extern int	RPCSendFloat();
extern int	RPCSendOctGenerator();
extern int	RPCSendOctObject();
extern int	RPCSendOctBox();
extern int	RPCSendOctPoint();
extern int	RPCSendOctProp();
extern int	RPCSendOctTransform();
extern int	RPCSendString();
extern int      RPCSendIntegerArray();
extern int      RPCSendRealArray();

