#include "copyright.h"
#include "port.h"
#include "utility.h"

/*
 * code that is used both by the server and the application
 *
 * Rick L Spickelmier, 3/17/86, last modified 3/2/87
 *
 */

#ifdef SERVER  
#include "rpcServer.h"
char RPCErrorBuffer[MESSAGE_SIZE];
#else  
#include "rpcApp.h"
#endif /* SERVER */

int
RPCReceiveOctObject(object, stream)
octObject *object;
STREAM stream;
{
    long dummy, dummy1, dummy2, dummy3;
    int status;
    long startingAngle, endingAngle, innerRadius, outerRadius;
    char temp;
    
    RPCASSERT(RPCReceiveLong(&dummy, stream));
    object->type = (int) dummy;
    RPCASSERT(RPCReceiveLong(&dummy, stream));
    object->objectId = (octId) dummy;

    switch (object->type) {
	
	case OCT_UNDEFINED_OBJECT:
	    return 1;

	case OCT_FACET:
	    return(RPCReceiveString(&object->contents.facet.cell, stream)
		&& RPCReceiveString(&object->contents.facet.view, stream)
		&& RPCReceiveString(&object->contents.facet.facet, stream)
		&& RPCReceiveString(&object->contents.facet.version, stream)
		&& RPCReceiveString(&object->contents.facet.mode, stream));

	case OCT_INSTANCE:
	    return(RPCReceiveOctTransform(&object->contents.instance.transform, stream)
		&& RPCReceiveString(&object->contents.instance.name, stream)
		&& RPCReceiveString(&object->contents.instance.master, stream)
		&& RPCReceiveString(&object->contents.instance.view, stream)
		&& RPCReceiveString(&object->contents.instance.facet, stream)
		&& RPCReceiveString(&object->contents.instance.version, stream));

	case OCT_PROP:
	    return RPCReceiveOctProp(&(object->contents.prop), stream);

	case OCT_TERM:
	    status = RPCReceiveLong(&dummy1, stream)
	          && RPCReceiveLong(&dummy2, stream)
	          && RPCReceiveLong(&dummy3, stream)
	          && RPCReceiveString(&object->contents.term.name, stream);
            object->contents.term.instanceId = (octId) dummy1;
            object->contents.term.formalExternalId = (int32) dummy2;
            object->contents.term.width = (int32) dummy3;
            return status;
    

	case OCT_NET:
	    status = RPCReceiveString(&object->contents.net.name, stream)
		  && RPCReceiveLong(&dummy, stream);
            object->contents.net.width = (int32) dummy;
	    return status;

	case OCT_BOX:
	    return RPCReceiveOctBox(&object->contents.box, stream);

	case OCT_CIRCLE:
	    status = RPCReceiveLong(&startingAngle, stream)
	          && RPCReceiveLong(&endingAngle, stream)
		  && RPCReceiveLong(&innerRadius, stream)
		  && RPCReceiveLong(&outerRadius, stream)
		  && RPCReceiveOctPoint(&object->contents.circle.center, stream);
            if (status) {
                object->contents.circle.startingAngle = (octCoord) startingAngle;
                object->contents.circle.endingAngle = (octCoord) endingAngle;
                object->contents.circle.innerRadius = (octCoord) innerRadius;
                object->contents.circle.outerRadius = (octCoord) outerRadius;
	    }
            return status;

	case OCT_PATH:
	    status = RPCReceiveLong(&dummy, stream);
            object->contents.path.width = (octCoord) dummy;
	    return status;
	    
	case OCT_POLYGON:
	    return 1;

	case OCT_LABEL:
	    status = RPCReceiveString(&object->contents.label.label, stream)
	      && RPCReceiveOctBox(&object->contents.label.region, stream)
		&& RPCReceiveLong(&dummy, stream);
	    if (!status) {
		return 0;
	    }
	    object->contents.label.textHeight = (octCoord) dummy;
	    if (!RPCReceiveByte(&temp, stream)) {
		return 0;
	    }
	    object->contents.label.vertJust = (unsigned) temp;
	    if (!RPCReceiveByte(&temp, stream)) {
		return 0;
	    }
	    object->contents.label.horizJust = (unsigned) temp;
	    if (!RPCReceiveByte(&temp, stream)) {
		return 0;
	    }
	    object->contents.label.lineJust = (unsigned) temp;
	    return 1;

	case OCT_BAG:
	    return RPCReceiveString(&object->contents.bag.name, stream);

	case OCT_LAYER:
	    return RPCReceiveString(&object->contents.layer.name, stream);

	case OCT_POINT:
	    return RPCReceiveOctPoint(&object->contents.point, stream);

	case OCT_EDGE:
	    return(RPCReceiveOctPoint(&object->contents.edge.start, stream)
		&& RPCReceiveOctPoint(&object->contents.edge.end, stream));

	case OCT_CHANGE_LIST:
	    status = RPCReceiveLong(&dummy1, stream)
	          && RPCReceiveLong(&dummy2, stream);
	    object->contents.changeList.objectMask = (octObjectMask) dummy1;
	    object->contents.changeList.functionMask = (octFunctionMask) dummy2;
	    return status;

	case OCT_CHANGE_RECORD:
	    return(RPCReceiveLong(&object->contents.changeRecord.changeType, stream)
	        && RPCReceiveLong(&object->contents.changeRecord.objectExternalId, stream)
	        && RPCReceiveLong(&object->contents.changeRecord.contentsExternalId, stream));

	default:
	    (void) fprintf(stderr, "RPC Error: illegal oct object type: %d\n", object->type);
	    return 0;
    }
}

int
RPCSendOctObject(object, stream)
octObject *object;
STREAM stream;
{
    int status;
    
    RPCASSERT(RPCSendLong((long) object->type, stream));
    RPCASSERT(RPCSendLong((long) object->objectId, stream));

    switch (object->type) {

	case OCT_UNDEFINED_OBJECT:
	    return 1;

	case OCT_FACET:
	    return(RPCSendString(object->contents.facet.cell, stream)
		&& RPCSendString(object->contents.facet.view, stream)
		&& RPCSendString(object->contents.facet.facet, stream)
		&& RPCSendString(object->contents.facet.version, stream)
		&& RPCSendString(object->contents.facet.mode, stream));

	case OCT_INSTANCE:
	    return(RPCSendOctTransform(&object->contents.instance.transform, stream)
		&& RPCSendString(object->contents.instance.name, stream)
		&& RPCSendString(object->contents.instance.master, stream)
		&& RPCSendString(object->contents.instance.view, stream)
		&& RPCSendString(object->contents.instance.facet, stream)
		&& RPCSendString(object->contents.instance.version, stream));

	case OCT_PROP:
	    return RPCSendOctProp(&(object->contents.prop), stream);

	case OCT_TERM:
	    return(RPCSendLong((long) object->contents.term.instanceId, stream)
	        && RPCSendLong((long) object->contents.term.formalExternalId, stream)
	        && RPCSendLong((long) object->contents.term.width, stream)
		&& RPCSendString(object->contents.term.name, stream));

	case OCT_NET:
	    return RPCSendString(object->contents.net.name, stream)
		&& RPCSendLong((long) object->contents.net.width, stream);

	case OCT_BOX:
	    return RPCSendOctBox(&(object->contents.box), stream);

	case OCT_CIRCLE:
	    return(RPCSendLong((long) object->contents.circle.startingAngle, stream)
		&& RPCSendLong((long) object->contents.circle.endingAngle, stream)
		&& RPCSendLong((long) object->contents.circle.innerRadius, stream)
		&& RPCSendLong((long) object->contents.circle.outerRadius, stream)
		&& RPCSendOctPoint(&(object->contents.circle.center), stream));

	case OCT_PATH:
	    return RPCSendLong((long) object->contents.path.width, stream);

	case OCT_POLYGON:
	    return 1;

	case OCT_LABEL:
	    status = RPCSendString(object->contents.label.label, stream)
	      && RPCSendOctBox(&(object->contents.label.region), stream)
		&& RPCSendLong((long) object->contents.label.textHeight, stream);
	    if (!status) {
		return 0;
	    }
	    if (!RPCSendByte((char) object->contents.label.vertJust, stream)) {
		return 0;
	    }
	    if (!RPCSendByte((char) object->contents.label.horizJust, stream)) {
		return 0;
	    }
	    if (!RPCSendByte((char) object->contents.label.lineJust, stream)) {
		return 0;
	    }
	    return 1;

	case OCT_BAG:
	    return RPCSendString(object->contents.bag.name, stream);

	case OCT_LAYER:
	    return RPCSendString(object->contents.layer.name, stream);

	case OCT_POINT:
	    return RPCSendOctPoint(&(object->contents.point), stream);

	case OCT_EDGE:
	    return(RPCSendOctPoint(&(object->contents.edge.start), stream)
		&& RPCSendOctPoint(&(object->contents.edge.end), stream));

	case OCT_CHANGE_LIST:
	    return(RPCSendLong((long) object->contents.changeList.objectMask, stream)
	        && RPCSendLong((long) object->contents.changeList.functionMask, stream));

	case OCT_CHANGE_RECORD:
	    return(RPCSendLong((long) object->contents.changeRecord.changeType, stream)
	        && RPCSendLong((long) object->contents.changeRecord.objectExternalId, stream)
	        && RPCSendLong((long) object->contents.changeRecord.contentsExternalId, stream));

	default:
	    (void) fprintf(stderr, "RPC Error: illegal oct object type: %d\n", object->type);
	    return 0;
    }
}

int
RPCReceiveOctProp(prop, stream)
struct octProp *prop;
STREAM stream;
{
    long dummy;
    int status;

    
    RPCASSERT(RPCReceiveString(&(prop->name), stream));
    RPCASSERT(RPCReceiveLong(&dummy, stream));
    prop->type = (octPropType) dummy;

    switch(prop->type) {

	case OCT_NULL:
	    return 1;

	case OCT_INTEGER:
	    status = RPCReceiveLong(&dummy, stream);
	    prop->value.integer = (int) dummy;
	    return status;

	case OCT_REAL:
	    return RPCReceiveFloat(&(prop->value.real), stream);

	case OCT_STRING:
	    return RPCReceiveString(&(prop->value.string), stream);

	case OCT_ID:
	    status = RPCReceiveLong(&dummy, stream);
	    prop->value.id = (octId) dummy;
	    return status;

	case OCT_STRANGER:
	    return RPCReceiveStranger(&(prop->value.stranger), stream);

	case OCT_INTEGER_ARRAY:
	    return RPCReceiveIntegerArray((struct octIntegerArray *) &(prop->value.integer_array), stream);

	case OCT_REAL_ARRAY:
	    return RPCReceiveRealArray((struct octRealArray *) &(prop->value.real_array), stream);

	default:
	    (void) fprintf(stderr, "RPC Error: illegal property type: %d\n", prop->type);
	    return 0;
    }
}

int
RPCSendOctProp(prop, stream)
struct octProp *prop;
STREAM stream;
{
    RPCASSERT(RPCSendString(prop->name, stream));
    RPCASSERT(RPCSendLong((long) prop->type, stream));

    switch(prop->type) {

	case OCT_NULL:
	    return 1;

	case OCT_INTEGER:
	    return RPCSendLong((long) prop->value.integer, stream);

	case OCT_REAL:
	    return RPCSendFloat(prop->value.real, stream);

	case OCT_STRING:
	    return RPCSendString(prop->value.string, stream);

	case OCT_ID:
	    return RPCSendLong((long) prop->value.id, stream);

	case OCT_STRANGER:
	    return RPCSendStranger(&(prop->value.stranger), stream);

	case OCT_INTEGER_ARRAY:
	    return RPCSendIntegerArray((struct octIntegerArray *) &(prop->value.integer_array), stream);

	case OCT_REAL_ARRAY:
	    return RPCSendRealArray((struct octRealArray *) &(prop->value.real_array), stream);

	default:
	    (void) fprintf(stderr, "RPC Error: illegal property type: %d\n", prop->type);
	    return 0;
	    
    }
}

int
RPCSendIntegerArray(iarray, stream)
struct octIntegerArray *iarray;
STREAM stream;
{
    int i;

    RPCASSERT(RPCSendLong((long) iarray->length, stream));

    if (!RPCByteSwapped) {
	if (fwrite((char *) iarray->array, sizeof(int),
		   (int) iarray->length, stream) != iarray->length) {
	    (void) fprintf(stderr, "RPC Error: fwrite of irray->array failed\n");
	    return 0;
	}
    } else {
	for (i = 0; i < iarray->length; i++) {
	    RPCASSERT(RPCSendLong((long) iarray->array[i], stream));
	}
    }
    
    return 1;
}


int
RPCSendRealArray(rarray, stream)
struct octRealArray *rarray;
STREAM stream;
{
    int i;

    RPCASSERT(RPCSendLong((long) rarray->length, stream));

    for (i = 0; i < rarray->length; i++) {
	RPCASSERT(RPCSendFloat(rarray->array[i], stream));
    }
    return 1;
}


int
RPCReceiveIntegerArray(iarray, stream)
struct octIntegerArray *iarray;
STREAM stream;
{
    int i;
    long dummy;
    

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    iarray->length = (int) dummy;

    if (!RPCByteSwapped) {
	iarray->array = ALLOC(int, iarray->length);
	if (fread((char *) iarray->array, sizeof(int),
		  (int) iarray->length, stream) != iarray->length) {
	    (void) fprintf(stderr, "RPC Error: fread of irray->array failed\n");
	    return 0;
	}
    } else {
	for (i = 0; i < iarray->length; i++) {
	    RPCASSERT(RPCReceiveLong(&dummy, stream));
	    iarray->array[i] = (int) dummy;
	}
    }
    
    return 1;
}

int
RPCReceiveRealArray(rarray, stream)
struct octRealArray *rarray;
STREAM stream;
{
    int i;
    long dummy;
    

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    rarray->length = (int) dummy;

    rarray->array = ALLOC(double, rarray->length);

    for (i = 0; i < rarray->length; i++) {
	RPCASSERT(RPCReceiveFloat(&rarray->array[i], stream));
    }
    return 1;
}


int
RPCReceiveOctTransform(transform, stream)
struct octTransform *transform;
STREAM stream;
{
    long dummy;

    
    RPCASSERT(RPCReceiveOctPoint(&transform->translation, stream));

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    transform->transformType = (octTransformType) dummy;

    if (transform->transformType == OCT_FULL_TRANSFORM) {
	return(RPCReceiveFloat(&transform->generalTransform[0][0], stream)
	    && RPCReceiveFloat(&transform->generalTransform[0][1], stream)
	    && RPCReceiveFloat(&transform->generalTransform[1][0], stream)
	    && RPCReceiveFloat(&transform->generalTransform[1][1], stream));
    } else {
	return 1;
    }
}

int
RPCSendOctTransform(transform, stream)
struct octTransform *transform;
STREAM stream;
{
    if (transform->transformType == OCT_FULL_TRANSFORM) {
	return(RPCSendOctPoint(&(transform->translation), stream)
	    && RPCSendLong((long) transform->transformType, stream)
	    && RPCSendFloat(transform->generalTransform[0][0], stream)
	    && RPCSendFloat(transform->generalTransform[0][1], stream)
	    && RPCSendFloat(transform->generalTransform[1][0], stream)
	    && RPCSendFloat(transform->generalTransform[1][1], stream));
    } else {
	return(RPCSendOctPoint(&(transform->translation), stream)
	    && RPCSendLong((long) transform->transformType, stream));
    }
}

int
RPCReceiveOctBox(box, stream)
struct octBox *box;
STREAM stream;
{
    if (!RPCByteSwapped) {
	if (fread((char *) box, sizeof(struct octBox), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fread of octBox failed\n");
	    return 0;
	}
	return 1;
    } else {
	return(RPCReceiveOctPoint(&box->lowerLeft, stream)
	       && RPCReceiveOctPoint(&box->upperRight, stream));
    }
}


int
RPCSendOctBox(box, stream)
struct octBox *box;
STREAM stream;
{
    if (!RPCByteSwapped) {
	if (fwrite((char *) box, sizeof(struct octBox), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fwrite of octBox failed\n");
	    return 0;
	}
	return 1;
    } else {
	return(RPCSendOctPoint(&(box->lowerLeft), stream)
	       && RPCSendOctPoint(&(box->upperRight), stream));
    }
}


int
RPCReceiveOctPoint(point, stream)
struct octPoint *point;
STREAM stream;
{
    long x, y;
    int status;

    
    if (!RPCByteSwapped) {
	if (fread((char *) point, sizeof(struct octPoint), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fread of octPoint failed\n");
	    return 0;
	}
	return 1;
    } else {
	status = RPCReceiveLong(&x, stream)
	      && RPCReceiveLong(&y, stream);
	if (status) {
	    point->x = (octCoord) x;
	    point->y = (octCoord) y;
	}
	return status;
    }
}


int
RPCSendOctPoint(point, stream)
struct octPoint *point;
STREAM stream;
{
    if (!RPCByteSwapped) {
	if (fwrite((char *) point, sizeof(struct octPoint), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fwrite of octPoint failed\n");
	    return 0;
	}
	return 1;
    } else {
	return(RPCSendLong((long) point->x, stream)
	       && RPCSendLong((long) point->y, stream));
    }
}


int
RPCReceiveOctGenerator(generator, stream)
octGenerator *generator;
STREAM stream;
{
    int status;
    long generator_func, state, free_func;

    
    status = RPCReceiveLong(&generator_func, stream)
          && RPCReceiveLong(&state, stream)
	    && RPCReceiveLong(&free_func, stream);
    
    if (status) {
	generator->generator_func = (octStatus (*)()) generator_func;
	generator->state = (char *) state;
	generator->free_func = (octStatus (*)()) free_func;
    }
    return status;
}


int
RPCSendOctGenerator(generator, stream)
octGenerator *generator;
STREAM stream;
{
    long gen_func, state, free_func;

    gen_func = (long) generator->generator_func;
    state = (long) generator->state;
    free_func = (long) generator->free_func;
    return(RPCSendLong(gen_func, stream) && RPCSendLong(state, stream)
	   && RPCSendLong(free_func, stream));
}

double RPCatof();

int RPCFloatingPointSame = 1;	/* XXX */

int
RPCReceiveFloat(real, stream)
double *real;
STREAM stream;
{
    if (!RPCFloatingPointSame) {
	char *charArray = ALLOC(char, 20);
	double db;

	RPCASSERT(RPCReceiveString(&charArray, stream));
	db = RPCatof(charArray);
	*real = db;
	FREE(charArray);
    } else {
	double db, *ptr;
	long lngs[2];

	RPCASSERT(RPCReceiveLong(&lngs[0], stream));
	RPCASSERT(RPCReceiveLong(&lngs[1], stream));
	ptr = (double *) lngs;
	db = *ptr;
	*real = db;
    }
	
    return 1;
}

int
RPCSendFloat(real, stream)
double real;
STREAM stream;
{
    char charArray[20];
    double db;
    long *lngs;
    
    if (!RPCFloatingPointSame) {
	(void) sprintf(charArray, "%g", real);
	return RPCSendString(charArray, stream);
    } else {
	db = real;
	lngs = (long *) &db;
	return RPCSendLong(lngs[0], stream) && RPCSendLong(lngs[1], stream);
    }
}

int
RPCReceiveStranger(stranger, stream)
struct octUserValue *stranger;
STREAM stream;
{
    long length;
    char *array;
    int i;

    RPCASSERT(RPCReceiveLong(&length, stream));

    stranger->length = length;
    array = ALLOC(char, length);

    if (!RPCByteSwapped) {
	if (fread((char *) array, sizeof(char), (int) length, stream) != length) {
	    (void) fprintf(stderr, "RPC Error: fread of stranger->contents failed\n");
	    return 0;
	}
    } else {
	for (i = 0; i < length; i++) {
	    RPCASSERT(RPCReceiveByte(&array[i], stream));
	}
    }
    
    stranger->contents = array;

    return 1;
}

int
RPCSendStranger(stranger, stream)
struct octUserValue *stranger;
STREAM stream;
{
    int i;

    RPCASSERT(RPCSendLong((long) stranger->length, stream));

    if (!RPCByteSwapped) {
	if (fwrite((char *) stranger->contents, sizeof(char), (int) stranger->length, stream) != stranger->length) {
	    (void) fprintf(stderr, "RPC Error: fwrite of stranger->contents failed\n");
	    return 0;
	}
    } else {
	for (i = 0; i < stranger->length; i++) {
	    RPCASSERT(RPCSendByte(stranger->contents[i], stream));
	}
    }
    
    return 1;
}


int
RPCReceiveString(userArray, stream)
char **userArray;
STREAM stream;
{
    long len;
    char *charArray;

    RPCASSERT(RPCReceiveLong(&len, stream));

    if (len == -1) {
	*userArray = NIL(char);
	return 1;
    }

    charArray = ALLOC(char, (len + 1));
    
    if (len > 0) {
	if (fread(charArray, sizeof(char), (int) len, stream) != len) {
	    (void) fprintf(stderr, "RPC Error: fread of charArray failed\n");
	    return 0;
	}
    }
    
    charArray[len] = '\0';
    *userArray = charArray;

    return 1;
}

int
RPCSendString(charArray, stream)
char *charArray;
STREAM stream;
{
    long len;
    
    if (charArray == NIL(char)) {
	RPCASSERT(RPCSendLong((long) -1, stream));
	return 1;
    }

    len = strlen(charArray);
    RPCASSERT(RPCSendLong(len, stream));

    if (len > 0) {
	if (fwrite(charArray, sizeof(char), (int) len, stream) != len) {
	    (void) fprintf(stderr, "RPC Error: fwrite of charArray failed\n");
	    return 0;
	}
    }
    
    return 1;
}


int
RPCReceiveLong(value, stream)
long *value;
STREAM stream;
{
    char *buf, temp;
    
    if (!RPCByteSwapped) {
	if (fread((char *) value, sizeof(long), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fread of long failed\n");
	    return 0;
	} else {
	    return 1;
	}
    } else {
	if (fread((char *) value, sizeof(long), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fread of long failed\n");
	    return 0;
	} else {
	    buf = (char *) value;
	    temp = buf[0];
	    buf[0] = buf[3];
	    buf[3] = temp;
	    temp = buf[1];
	    buf[1] = buf[2];
	    buf[2] = temp;
	    return 1;
	}    
    }
}


int
RPCReceiveFacetInfo(info, stream)
struct octFacetInfo *info;
STREAM stream;
{
    long dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->timeStamp = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->bbDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->formalDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->createDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->deleteDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->modifyDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->attachDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->detachDate = (int32) dummy;

    RPCASSERT(RPCReceiveLong(&dummy, stream));
    info->numObjects = (int) dummy;

    RPCASSERT(RPCReceiveOctBox(&info->bbox, stream));
    RPCASSERT(RPCReceiveString(&info->fullName, stream));

    return 1;
}


int
RPCSendFacetInfo(info, stream)
struct octFacetInfo *info;
STREAM stream;
{
    RPCASSERT(RPCSendLong((long) info->timeStamp, stream));
    RPCASSERT(RPCSendLong((long) info->bbDate, stream));
    RPCASSERT(RPCSendLong((long) info->formalDate, stream));
    RPCASSERT(RPCSendLong((long) info->createDate, stream));
    RPCASSERT(RPCSendLong((long) info->deleteDate, stream));
    RPCASSERT(RPCSendLong((long) info->modifyDate, stream));
    RPCASSERT(RPCSendLong((long) info->attachDate, stream));
    RPCASSERT(RPCSendLong((long) info->detachDate, stream));
    RPCASSERT(RPCSendLong((long) info->numObjects, stream));
    RPCASSERT(RPCSendOctBox(&info->bbox, stream));
    RPCASSERT(RPCSendString(info->fullName, stream));
    return 1;
}


#ifdef RPCLIBRARY
int
RPCReceiveFunctionToken(value, stream)
long *value;
STREAM stream;
/* this routine gives a better message than RPCReceiveLong */
{
    char *buf, temp;
    
    if (!RPCByteSwapped) {
	if (fread((char *) value, sizeof(long), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC application reports that VEM has been terminated\n");
	    return 0;
	} else {
	    return 1;
	}
    } else {
	if (fread((char *) value, sizeof(long), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC application reports that VEM has been terminated\n");
	    return 0;
	} else {
	    buf = (char *) value;
	    temp = buf[0];
	    buf[0] = buf[3];
	    buf[3] = temp;
	    temp = buf[1];
	    buf[1] = buf[2];
	    buf[2] = temp;
	    return 1;
	}    
    }
}
#endif


int
RPCSendLong(value, stream)
long value;
STREAM stream;
{
    char *buf, temp;
    long dummy;
    
    if (!RPCByteSwapped) {
	if (fwrite((char *) &value, sizeof(long), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fwrite of long failed\n");
	    return 0;
	} else {
	    return 1;
	}
    } else {
	dummy = value;
	buf = (char *) &dummy;
	temp = buf[0];
	buf[0] = buf[3];
	buf[3] = temp;
	temp = buf[1];
	buf[1] = buf[2];
	buf[2] = temp;
	if (fwrite((char *) &dummy, sizeof(long), (int) 1, stream) != 1) {
	    (void) fprintf(stderr, "RPC Error: fwrite of long failed\n");
	    return 0;
	} else {
	    return 1;
	}
    }
}


int
RPCReceiveByte(value, stream)
char *value;
STREAM stream;
{
    return fread(value, sizeof(char), (int) 1, stream) == 1;
}

int
RPCSendByte(value, stream)
char value;
STREAM stream;
{
    return fwrite((char *) &value, sizeof(char), (int) 1, stream) == 1;
}


char *
RPCstrsave(string)
char *string;
{
    char *p;

    p = ALLOC(char, (strlen(string) + 1));
    (void) strcpy(p, string);
    return p;
}


/*
 *	C library - ascii to floating
 */

double	twoe56	= 72057594037927936.; /*2^56*/
double	exp5[]	= {5.,25.,625.,390625.,152587890625.,23232710361480712890625.};

double
RPCatof(p)
register char *p;
{
	extern double ldexp();
	register c, exp = 0, eexp = 0;
	double fl = 0, flexp = 1.0;
	int bexp, neg = 1, negexp = 1;

	while((c = *p++) == ' ');
	if (c == '-') neg = -1;	else if (c == '+'); else --p;

	while ((c = *p++), isdigit(c))
		if (fl < twoe56) fl = 10*fl + (c-'0'); else exp++;
	if (c == '.')
	while ((c = *p++), isdigit(c))
		if (fl < twoe56)
		{
			fl = 10*fl + (c-'0');
			exp--;
		}
	if ((c == 'E') || (c == 'e'))
	{
		if ((c= *p++) == '+'); else if (c=='-') negexp = -1; else --p;
		while ((c = *p++), isdigit(c)) eexp = 10*eexp + (c-'0');
		if (negexp < 0) eexp = -eexp; exp += eexp;
	}
	bexp = exp;
	if (exp < 0) exp = -exp;

	for (c = 0; c < 6; c++)
	{
		if (exp & 01) flexp *= exp5[c];
		exp >>= 1; if (exp == 0) break;
	}

	if (bexp < 0) fl /= flexp; else fl *= flexp;
	fl = ldexp(fl, bexp);
	if (neg < 0) return -fl; else return fl;
}


#ifdef NOWAIT
/*
 * a version of fread that isn't buffers, but won't error on a partial
 * read
 */

RPCfread(ptr, size, count, stream)
register char *ptr;
int size;
int count;
FILE *stream;
{
    register int fd = fileno(stream);
    register int num_read, len = size * count;

    do {
	num_read = read(fd, ptr, len);
	if (num_read <= 0) {
	    if (num_read == 0) {
		(void) fprintf(stderr, "RPCfread: read returned 0!\n");
	    }
	    return (size * count - len) / size;
	}
	len -= num_read;
    } while (len);

    return count;
}
#endif

