#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "rpcServer.h"


void
RPCTrappedError(status)
octStatus status;
{
    if (status < OCT_OK) {
	char buffer[1024];
		       
	errorlog("An OCT error occured and the application was not informed\n");
	(void) sprintf(buffer, "%s\n", octErrorString());
	errorlog(buffer);
	return;
    }
    return;
}


/*
 * process a remote OCT request
 *
 * get the OCT function number and then call the appropriate routine
 *
 * all remote OCT requests proceed as follows:
 *
 *   receive OCT function number
 *   receive OCT arguments
 *   (perform OCT function)
 *   send return code
 *   send modified OCT arguments
 */
int
OCTRequest(functionNumber)
long functionNumber;
{
    int status;
    char buffer[1024];

#ifdef DEBUG
    (void) sprintf(buffer, "OCT function %d\n", functionNumber);
    errorlog(buffer);
#endif

    switch (functionNumber) {

	case OCT_DELETE_FUNCTION:
	    status = RPCoctDelete(sendStream, receiveStream);
	    break;
	
	case OCT_GET_BY_ID_FUNCTION:
	    status = RPCoctGetById(sendStream, receiveStream);
	    break;
	    
	case OCT_MODIFY_FUNCTION:
	    status = RPCoctModify(sendStream, receiveStream);
	    break;
	    
	case OCT_MODIFY_SPECIAL_FUNCTION:
	    status = RPCoctModifySpecial(sendStream, receiveStream);
	    break;
	    
        case OCT_OPEN_FACET_FUNCTION:
            status = RPCoctOpenFacet(sendStream, receiveStream);
	    break;
	    
	case OCT_FLUSH_FACET_FUNCTION:
            status = RPCoctFlushFacet(sendStream, receiveStream);
	    break;
	    
	case OCT_CLOSE_FACET_FUNCTION:
            status = RPCoctCloseFacet(sendStream, receiveStream);
	    break;
	    
	case OCT_FREE_FACET_FUNCTION:
            status = RPCoctFreeFacet(sendStream, receiveStream);
	    break;

	case OCT_OPEN_MASTER_FUNCTION:
            status = RPCoctOpenMaster(sendStream, receiveStream);
            break;

	case OCT_ATTACH_FUNCTION:
            status = RPCoctAttach(sendStream, receiveStream);
	    break;
	    
	case OCT_ATTACH_ONCE_FUNCTION:
            status = RPCoctAttachOnce(sendStream, receiveStream);
	    break;
	    
	case OCT_IS_ATTACHED_FUNCTION:
            status = RPCoctIsAttached(sendStream, receiveStream);
	    break;
	    
	case OCT_DETACH_FUNCTION:
            status = RPCoctDetach(sendStream, receiveStream);
	    break;
	    
	case OCT_COPY_FACET_FUNCTION:
            status = RPCoctCopyFacet(sendStream, receiveStream);
	    break;
	    
	case OCT_WRITE_FACET_FUNCTION:
            status = RPCoctWriteFacet(sendStream, receiveStream);
	    break;
	    
	case OCT_CREATE_FUNCTION:
            status = RPCoctCreate(sendStream, receiveStream);
	    break;
	    
	case OCT_GET_BY_NAME_FUNCTION:
            status = RPCoctGetByName(sendStream, receiveStream);
	    break;
	    
	case OCT_GET_CONTAINER_BY_NAME_FUNCTION:
            status = RPCoctGetContainerByName(sendStream, receiveStream);
	    break;
	    
	case OCT_GET_OR_CREATE_FUNCTION:
            status = RPCoctGetOrCreate(sendStream, receiveStream);
	    break;
	    
	case OCT_CREATE_OR_MODIFY_FUNCTION:
            status = RPCoctCreateOrModify(sendStream, receiveStream);
	    break;
	
	case OCT_INIT_GEN_CONTAINERS_FUNCTION:
            status = RPCoctInitGenContainers(sendStream, receiveStream);
	    break;
	    
	case OCT_INIT_GEN_CONTENTS_FUNCTION:
            status = RPCoctInitGenContents(sendStream, receiveStream);
	    break;

	case OCT_GENERATE_FUNCTION:
	    status = RPCoctGenerate(sendStream, receiveStream);
	    break;

	case OCT_GENERATE_SPECIAL_FUNCTION:
	    status = RPCoctGenerateSpecial(sendStream, receiveStream);
	    break;

	case OCT_GEN_FIRST_CONTENT_FUNCTION:
	    status = RPCoctGenFirstContent(sendStream, receiveStream);
	    break;

	case OCT_GEN_FIRST_CONTAINER_FUNCTION:
	    status = RPCoctGenFirstContainer(sendStream, receiveStream);
	    break;

	case OCT_FREE_GENERATOR_FUNCTION:
	    status = RPCoctFreeGenerator(sendStream, receiveStream);
	    break;

	case OCT_ERROR_FN_FUNCTION:
            status = RPCoctError(sendStream, receiveStream);
	    break;
	    
	case OCT_GET_POINTS_FUNCTION:
            status = RPCoctGetPoints(sendStream, receiveStream);
	    break;
	    
	case OCT_PUT_POINTS_FUNCTION:
            status = RPCoctPutPoints(sendStream, receiveStream);
	    break;
	    
	case OCT_BB_FUNCTION:
	    status = RPCoctBB(sendStream, receiveStream);
	    break;

        case OCT_GET_FACET_FUNCTION:
            status = RPCoctGetFacet(sendStream, receiveStream);
	    break;
	    
        case OCT_ERROR_STRING_FUNCTION:
            status = RPCoctErrorString(sendStream, receiveStream);
	    break;

        case OCT_TRANSFORM_GEO_FUNCTION:
            status = RPCoctTransformGeo(sendStream, receiveStream);
	    break;

        case OCT_TRANSFORM_MODIFY_GEO_FUNCTION:
            status = RPCoctTransformAndModifyGeo(sendStream, receiveStream);
	    break;

        case OCT_TRANSFORM_POINTS_FUNCTION:
            status = RPCoctTransformPoints(sendStream, receiveStream);
	    break;

        case OCT_SCALE_GEO_FUNCTION:
            status = RPCoctScaleGeo(sendStream, receiveStream);
	    break;

        case OCT_SCALE_MODIFY_GEO_FUNCTION:
            status = RPCoctScaleAndModifyGeo(sendStream, receiveStream);
	    break;

        case OCT_MARK_TEMPORARY_FUNCTION:
            status = RPCoctMarkTemporary(sendStream, receiveStream);
	    break;

        case OCT_UNMARK_TEMPORARY_FUNCTION:
            status = RPCoctUnmarkTemporary(sendStream, receiveStream);
	    break;

        case OCT_IS_TEMPORARY_FUNCTION:
            status = RPCoctIsTemporary(sendStream, receiveStream);
	    break;

        case OCT_XID_FUNCTION:
            status = RPCoctExternalId(sendStream, receiveStream);
	    break;

        case OCT_GET_BY_XID_FUNCTION:
            status = RPCoctGetByExternalId(sendStream, receiveStream);
	    break;

        case OCT_FULL_NAME_FUNCTION:
            status = RPCoctFullName(sendStream, receiveStream);
	    break;

        case OCT_GET_FACET_INFO_FUNCTION:
            status = RPCoctGetFacetInfo(sendStream, receiveStream);
	    break;

        case OCT_OPEN_RELATIVE_FUNCTION:
            status = RPCoctOpenRelative(sendStream, receiveStream);
	    break;

	default:
	    (void) sprintf(RPCErrorBuffer, "RPC Error: illegal OCT function number (%d)\n",
		    functionNumber);
	    errorlog(RPCErrorBuffer);
	    return(0);
    }

    if (status == 0) {
	errorlog("RPC Error: OCT function failure\n");
	return(0);
    }

    RPCFLUSH(sendStream);
    return(1);
}


/*ARGSUSED*/
int
RPCoctAttach(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
        
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object2.objectId = (octId) id;
    
    returnCode = octAttach(&object1, &object2);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


int
RPCoctAttachOnce(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
        
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object2.objectId = (octId) id;

    returnCode = octAttachOnce(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    return(1);
}


int
RPCoctIsAttached(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
        
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object2.objectId = (octId) id;

    returnCode = octIsAttached(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    return(1);
}


/*ARGSUSED*/
int
RPCoctDetach(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object2.objectId = (octId) id;

    returnCode = octDetach(&object1, &object2);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


int
RPCoctCreate(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveOctObject(&object2, receiveStream));
    
    returnCode = octCreate(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}


/*ARGSUSED*/
int
RPCoctDelete(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    returnCode = octDelete(&object);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


int
RPCoctCopyFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    
    RPCASSERT(RPCReceiveOctObject(&object1, receiveStream));
    RPCASSERT(RPCReceiveOctObject(&object2, receiveStream));
    returnCode = octCopyFacet(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object1, sendStream));
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}


int
RPCoctWriteFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    
    RPCASSERT(RPCReceiveOctObject(&object1, receiveStream));
    RPCASSERT(RPCReceiveOctObject(&object2, receiveStream));
    returnCode = octWriteFacet(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object1, sendStream));
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}


/*ARGSUSED*/
int
RPCoctError(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    char *error;
    char buffer[1024];
    
    RPCASSERT(RPCReceiveString(&error, receiveStream));
    (void) sprintf(buffer, "%s: %s\n", error, octErrorString());
    errorlog(buffer);
    return(1);
}


int
RPCoctGetByName(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id, type;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;

    RPCASSERT(RPCReceiveLong(&type, receiveStream));
    object2.type = (int) type;

    switch (object2.type) {
	case OCT_INSTANCE:
	    RPCASSERT(RPCReceiveString(&object2.contents.instance.name, receiveStream));
	    break;

	case OCT_PROP:
	    RPCASSERT(RPCReceiveString(&object2.contents.prop.name, receiveStream));
	    break;

	case OCT_TERM:
	    RPCASSERT(RPCReceiveString(&object2.contents.term.name, receiveStream));
	    break;

	case OCT_NET:
	    RPCASSERT(RPCReceiveString(&object2.contents.net.name, receiveStream));
	    break;

	case OCT_LAYER:
	    RPCASSERT(RPCReceiveString(&object2.contents.layer.name, receiveStream));
	    break;

	case OCT_BAG:
	    RPCASSERT(RPCReceiveString(&object2.contents.bag.name, receiveStream));
	    break;

	case OCT_LABEL:
	    RPCASSERT(RPCReceiveString(&object2.contents.label.label, receiveStream));
	    break;
    }
    returnCode = octGetByName(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}


int
RPCoctGetContainerByName(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id, type;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;

    RPCASSERT(RPCReceiveLong(&type, receiveStream));
    object2.type = (int) type;

    switch (object2.type) {
	case OCT_INSTANCE:
	    RPCASSERT(RPCReceiveString(&object2.contents.instance.name, receiveStream));
	    break;

	case OCT_PROP:
	    RPCASSERT(RPCReceiveString(&object2.contents.prop.name, receiveStream));
	    break;

	case OCT_TERM:
	    RPCASSERT(RPCReceiveString(&object2.contents.term.name, receiveStream));
	    break;

	case OCT_NET:
	    RPCASSERT(RPCReceiveString(&object2.contents.net.name, receiveStream));
	    break;

	case OCT_LAYER:
	    RPCASSERT(RPCReceiveString(&object2.contents.layer.name, receiveStream));
	    break;

	case OCT_BAG:
	    RPCASSERT(RPCReceiveString(&object2.contents.bag.name, receiveStream));
	    break;
    }
    returnCode = octGetContainerByName(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}

/*ARGSUSED*/
int
RPCoctErrorString(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    char *string;

    string = util_strsav(octErrorString());
    RPCASSERT(RPCSendString(string, sendStream));
    FREE(string);

    return(1);
}    


int
RPCoctGetFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object, facet;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    octGetFacet(&object, &facet);
    RPCASSERT(RPCSendOctObject(&facet, sendStream));
    return(1);
}


int
RPCoctGetById(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    returnCode = octGetById(&object);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object, sendStream));
    }
    return(1);
}


int
RPCoctGetOrCreate(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveOctObject(&object2, receiveStream));
    returnCode = octGetOrCreate(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}


int
RPCoctCreateOrModify(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object1, object2;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object1.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveOctObject(&object2, receiveStream));
    returnCode = octCreateOrModify(&object1, &object2);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object2, sendStream));
    }
    return(1);
}


/*ARGSUSED*/
int
RPCoctModify(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    
    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    returnCode = octModify(&object);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


/*ARGSUSED*/
int
RPCoctModifySpecial(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    (void) octGetById(&object);
    
    RPCASSERT(RPCReceiveString(&(object.contents.instance.name), receiveStream));
    RPCASSERT(RPCReceiveOctTransform(&(object.contents.instance.transform), receiveStream));
    
    returnCode = octModify(&object);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


int
RPCoctOpenFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    
    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    returnCode = octOpenFacet(&object);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object, sendStream));
    }
    return(1);
}


/*ARGSUSED*/
int
RPCoctCloseFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    returnCode = octCloseFacet(&object);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


/*ARGSUSED*/
int
RPCoctFlushFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;

    returnCode = octFlushFacet(&object);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif NOWAIT    
    return(1);
}


/*ARGSUSED*/
int
RPCoctFreeFacet(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    returnCode = octFreeFacet(&object);
#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */
    return(1);
}


int
RPCoctOpenMaster(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject inst, facet;
    long id;
    octStatus returnCode;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    inst.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveString(&facet.contents.facet.facet, receiveStream));
    RPCASSERT(RPCReceiveString(&facet.contents.facet.mode, receiveStream));

    facet.type = OCT_FACET;

    returnCode = octOpenMaster(&inst, &facet);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&facet, sendStream));
    }

    return(1);
}


int
RPCoctOpenRelative(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject facet, rfacet;
    long id, dummy;
    int location;
    octStatus returnCode;

    RPCASSERT(RPCReceiveOctObject(&facet, receiveStream));
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    rfacet.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&dummy, receiveStream));
    location = (int) dummy;
    
    returnCode = octOpenRelative(&rfacet, &facet, location);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&facet, sendStream));
    }

    return(1);
}

    
int
RPCoctBB(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    struct octBox bbox;
    octStatus returnCode;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;

    returnCode = octBB(&object, &bbox);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctPoint(&(bbox.lowerLeft), sendStream));
	RPCASSERT(RPCSendOctPoint(&(bbox.upperRight), sendStream));
    }

    return(1);
}


int
RPCoctInitGenContainers(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long id, mask;
    octGenerator generator;
    octStatus returnCode;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveLong(&mask, receiveStream));

    returnCode = octInitGenContainers(&object, (octObjectMask) mask, &generator);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    RPCASSERT(RPCSendOctGenerator(&generator, sendStream));

    return(1);
}


int
RPCoctInitGenContents(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long id, mask;
    octGenerator generator;
    octStatus returnCode;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveLong(&mask, receiveStream));

    returnCode = octInitGenContents(&object, (octObjectMask) mask, &generator);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    RPCASSERT(RPCSendOctGenerator(&generator, sendStream));

    return(1);
}


/* process a remote octGenerate request */
int
RPCoctGenerate(sendStream, receiveStream)
STREAM sendStream;
STREAM receiveStream;
{
    octGenerator generator;
    octObject object;
    octStatus returnCode;

    RPCASSERT(RPCReceiveOctGenerator(&generator, receiveStream));

    returnCode = octGenerate(&generator, &object);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    RPCASSERT(RPCSendOctGenerator(&generator, sendStream));
    
    if (returnCode == OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object, sendStream));
    }

    return(1);
}


/* process a remote octGenerateSpecial request */
int
RPCoctGenerateSpecial(sendStream, receiveStream)
STREAM sendStream;
STREAM receiveStream;
{
    octGenerator generator;
    octObject object;
    long maxcount, count;
    octStatus status;

    RPCASSERT(RPCReceiveOctGenerator(&generator, receiveStream));
    RPCASSERT(RPCReceiveLong(&maxcount, receiveStream));

    count = 0;

    while (count < maxcount) {
	
	status = octGenerate(&generator, &object);
	
	if (status != OCT_OK) {
	    RPCASSERT(RPCSendLong((long) status, sendStream));
	    RPCASSERT(RPCSendOctGenerator(&generator, sendStream));
	    return(1);
	}

	/* XXX I can't remember why I put this in!!  Rick
	if (object.type == OCT_FACET) {
	    object.contents.facet.cell = NIL(char);
	    object.contents.facet.view = NIL(char);
	    object.contents.facet.facet = NIL(char);
	    object.contents.facet.version = NIL(char);
	    object.contents.facet.mode = NIL(char);
	}
	*/
	
	RPCASSERT(RPCSendLong((long) status, sendStream));
	RPCASSERT(RPCSendOctObject(&object, sendStream));
	count++;
    }
    
    RPCASSERT(RPCSendOctGenerator(&generator, sendStream));

    return(1);
}


int
RPCoctGenFirstContent(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object, content;
    long id, mask;
    octStatus returnCode;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveLong(&mask, receiveStream));

    returnCode = octGenFirstContent(&object, (octObjectMask) mask, &content);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));

    /* OCT_GEN_DONE is > OCT_OK */
    if (returnCode == OCT_OK) {
	RPCASSERT(RPCSendOctObject(&content, sendStream));
    }

    return(1);
}
	

int
RPCoctGenFirstContainer(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object, container;
    long id, mask;
    octStatus returnCode;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    RPCASSERT(RPCReceiveLong(&mask, receiveStream));

    returnCode = octGenFirstContainer(&object, (octObjectMask) mask, &container);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));

    /* OCT_GEN_DONE is > OCT_OK */
    if (returnCode == OCT_OK) {
	RPCASSERT(RPCSendOctObject(&container, sendStream));
    }

    return(1);
}


/*ARGSUSED*/
int
RPCoctFreeGenerator(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octGenerator generator;
    octStatus returnCode;

    RPCASSERT(RPCReceiveOctGenerator(&generator, receiveStream));
    returnCode = octFreeGenerator(&generator);

#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */

    return(1);
}


int
RPCoctGetPoints(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long num_points;
    octStatus returnCode;
    long sendPoints;	/* used for the braindamaged function octGetPoints */
    struct octPoint *points;
    long id;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&num_points, receiveStream));

    RPCASSERT(RPCReceiveLong(&sendPoints, receiveStream));

    if (sendPoints == RPC_DO_NOT_SEND_POINTS) {
	points = NULL;
    } else {
	points = ALLOC(struct octPoint, num_points);
    }

    returnCode = octGetPoints(&object, (int *) &num_points, points);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    RPCASSERT(RPCSendLong(num_points, sendStream));

    if ((sendPoints == RPC_SEND_POINTS) && (returnCode != OCT_TOO_SMALL)) {

	if (fwrite((char *) points, sizeof(struct octPoint), (int) num_points, sendStream) != num_points) {
	    return(0);
	}
	FREE(points);
    }
    return(1);
}


/*ARGSUSED*/
int
RPCoctPutPoints(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long num_points;
    octStatus returnCode;
    struct octPoint *points;

    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    RPCASSERT(RPCReceiveLong(&num_points, receiveStream));

    points = ALLOC(struct octPoint, num_points);

    if (fread((char *) points, sizeof(struct octPoint), (int) num_points, receiveStream) != num_points) {
	return(0);
    }

    returnCode = octPutPoints(&object, (int) num_points, points);

#ifndef NOWAIT    
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
#else
    RPCTrappedError(returnCode);
#endif /* NOWAIT */

    FREE(points);
    return(1);
}


int
RPCoctTransformGeo(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    struct octTransform transform;

    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    
    RPCASSERT(RPCReceiveOctTransform(&transform, receiveStream));

    octTransformGeo(&object, &transform);

    RPCASSERT(RPCSendOctObject(&object, sendStream));

    return(1);
}


int
RPCoctTransformPoints(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    struct octTransform transform;
    long num_points;
    struct octPoint *points;
    int i;

    RPCASSERT(RPCReceiveLong(&num_points, receiveStream));

    points = ALLOC(struct octPoint, num_points);

    for (i = 0; i < num_points; i++) {
	RPCASSERT(RPCReceiveOctPoint(&points[i], receiveStream));
    }

    RPCASSERT(RPCReceiveOctTransform(&transform, receiveStream));

    octTransformPoints(num_points, points, &transform);

    for (i = 0; i < num_points; i++) {
	RPCASSERT(RPCSendOctPoint(&(points[i]), sendStream));
    }

    FREE(points);
    return(1);
}


int
RPCoctTransformAndModifyGeo(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    struct octTransform transform;

    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    RPCASSERT(RPCReceiveOctTransform(&transform, receiveStream));

    returnCode = octTransformAndModifyGeo(&object, &transform);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object, sendStream));
    }	

    return(1);
}


int
RPCoctScaleGeo(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    double scale;

    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    RPCASSERT(RPCReceiveFloat(&scale, receiveStream));

    octScaleGeo(&object, scale);

    RPCASSERT(RPCSendOctObject(&object, sendStream));

    return(1);
}


int
RPCoctScaleAndModifyGeo(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject object;
    double scale;

    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));
    RPCASSERT(RPCReceiveFloat(&scale, receiveStream));

    returnCode = octScaleAndModifyGeo(&object, scale);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object, sendStream));
    }	

    return(1);
}


/*ARGSUSED*/
int
RPCoctMarkTemporary(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    octMarkTemporary(&object);
    return(1);
}


/*ARGSUSED*/
int
RPCoctUnmarkTemporary(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    octUnmarkTemporary(&object);
    return(1);
}


int
RPCoctIsTemporary(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    int returnCode;
    octObject object;
    long id;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    
    returnCode = octIsTemporary(&object);
    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    return(1);
}


int
RPCoctExternalId(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octObject object;
    long id;
    long xid = 0;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    octExternalId(&object, &xid);
    RPCASSERT(RPCSendLong((long) xid, sendStream));
    return(1);
}


int
RPCoctGetByExternalId(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    octStatus returnCode;
    octObject facet, object;
    long id;
    int32 xid;
    
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    facet.objectId = (octId) id;
    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    xid = (int32) id;

    returnCode = octGetByExternalId(&facet, xid, &object);

    RPCASSERT(RPCSendLong((long) returnCode, sendStream));
    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendOctObject(&object, sendStream));
    }
    return(1);
}


int
RPCoctFullName(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    char *string;
    octObject object;
    long id;

    RPCASSERT(RPCReceiveLong(&id, receiveStream));
    object.objectId = (octId) id;
    octFullName(&object, &string);

    RPCASSERT(RPCSendString(string, sendStream));
    FREE(string);
    return(1);
}    


int
RPCoctGetFacetInfo(sendStream, receiveStream)
STREAM sendStream, receiveStream;
{
    long returnCode;
    octObject object;
    struct octFacetInfo info;

    RPCASSERT(RPCReceiveOctObject(&object, receiveStream));

    returnCode = octGetFacetInfo(&object, &info);

    RPCASSERT(RPCSendLong(returnCode, sendStream));

    if (returnCode >= OCT_OK) {
	RPCASSERT(RPCSendFacetInfo(&info, sendStream));
    }

    return 1;
}
