#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"

IFsimulator *ofeSpicePtr = NIL(IFsimulator);
IFsimulator *ofeDeckPtr = NIL(IFsimulator);
IFsimulator *ofeSimPtr = NIL(IFsimulator);
char *errRtn = NIL(char);
char *errMsg = NIL(char);
char *ofeDisplayName;

cktDesc *ofeCircuit(contentsPtr, groundPtr)
octObject *contentsPtr;
octObject *groundPtr;
{
    GENERIC *simCkt;
    IFuid uid;
    cktDesc *cktPtr;

    IF_ASSERT(newCircuit, (&simCkt));
    if (!ofeCheckIF(ofeNewUid(NIL(GENERIC), &uid, (IFuid) 0,
			      contentsPtr->contents.facet.cell,
			      UID_OTHER, simCkt),
		    __FILE__, __LINE__, FATAL)) {
	exit(-1);
    }
    cktPtr = (cktDesc *) uid;
    OFE_NEW_UID(&uid, (IFuid) cktPtr, NIL(char), UID_INSTANCE);
    cktPtr->rootPtr = (cellDesc *) uid;

    cktPtr->rootPtr->master = *contentsPtr;

    ofeBuildTopTermList(cktPtr->rootPtr, groundPtr);

    ofeModule(cktPtr->rootPtr);

    return(cktPtr);
}

ofeBuildTopTermList(cellPtr, groundPtr)
cellDesc *cellPtr;
octObject *groundPtr;
{
    octGenerator gen;
    octObject term, net;
    IFuid uid;
    netDesc *groundNetPtr;

    switch (groundPtr->type) {
	case OCT_TERM:
	    if (octGenFirstContainer(groundPtr, OCT_NET_MASK, &net) != OCT_OK) {
		ofeMessage(FATAL,
			    "ground terminal `%s' not connected to any net\n",
			    ohGetName(groundPtr));
	    }
	    break;
	case OCT_NET:
	    net = *groundPtr;
	    break;
	default:
	    ofeMessage(FATAL, "no ground terminal or net specified\n");
    }

    OFE_NEW_UID(&uid, (IFuid) cellPtr, ohGetName(groundPtr), UID_SIGNAL);
    groundNetPtr = (netDesc *) uid;
    groundNetPtr->cellPtr = cellPtr;
    groundNetPtr->netId = net.objectId;
    IF_ASSERT(groundNode,
		    (cellPtr->cktPtr->simEquiv, &groundNetPtr->simEquiv, uid));
    ASSERT( ! st_insert(cellPtr->netTable,
			    (char *) net.objectId, (char *) groundNetPtr));

    octInitGenContents(&cellPtr->master, OCT_TERM_MASK, &gen);
    while (octGenerate(&gen, &term) == OCT_OK) {
	if (octGenFirstContainer(&term, OCT_NET_MASK, &net) == OCT_OK) {
	    ASSERT( ! st_insert(cellPtr->termTable,
				(char *) term.contents.term.formalExternalId,
				(char *) ofeFindNet(cellPtr, ohGetName(&term),
						    net.objectId)));
	} else {
	    ofeMessage(WARNING, "%s on %s not connected\n",
			ohFormatName(&term), ohFormatName(&cellPtr->master));
	}
    }
}

void ofeFreeCircuit(cktPtr)
cktDesc *cktPtr;
{
    /* tell the simulator to blow away the circuit */
    IF_ASSERT(deleteCircuit, (cktPtr->simEquiv));

    /* free last element in the uid list until only circuit itself is left */
    while (cktPtr->bChain != (uidDesc *) cktPtr) {
	ofeFreeUid((uidDesc *) cktPtr->bChain);
    }

    /* free the circuit itself */
    ofeFreeUid((uidDesc *) cktPtr);
}
