#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"
#include "simtrans.h"

void ofeLeaf(instPtr, cellPtr)
octObject *instPtr;
cellDesc *cellPtr;
{
    octObject cellType;
    struct deviceMap *tablePtr;

    if (ohGetByPropName(&cellPtr->master, &cellType, "CELLTYPE") != OCT_OK ||
				cellType.contents.prop.type != OCT_STRING) {
	ofeMessage(SEVERE, "leaf cell %s has no valid `%s' property\n",
			   ohFormatName(&cellPtr->master),
			   cellType.contents.prop.name);
    } else {
	for (tablePtr = ofeTransTable; tablePtr->octCellType; tablePtr++) {
	    if (strcmp(tablePtr->octCellType,
				cellType.contents.prop.value.string) == 0) {
		/* this may be it */
		if (ofeTryLeaf(instPtr, cellPtr, tablePtr)) {
		    return;
		}
	    }
	}
	ofeMessage(SEVERE,
		"leaf cell %s of type `%s' doesn't match simulator device\n",
		ohFormatName(&cellPtr->master),
		cellType.contents.prop.value.string);
    }
}

static int ofeTryLeaf(instPtr, cellPtr,  tablePtr)
octObject *instPtr;
cellDesc *cellPtr;
struct deviceMap *tablePtr;
{
    octObject subType;
    octObject bigModelBag;
    octObject modelBag;

    /* check syb-type property (e.g. -- "SOURCETYPE") */
    if (tablePtr->octSubName) {
	if (	    ohGetByPropName(&cellPtr->master, &subType,
				    tablePtr->octSubName) != OCT_OK ||
		    subType.contents.prop.type != OCT_STRING ||
		    strcmp(subType.contents.prop.value.string,
			   tablePtr->octSubType)) {
	    return(0);
	}
    }

    /* get the model bag */
    if (	ohGetByBagName(&cellPtr->master, &bigModelBag,
			       tablePtr->modBagName) != OCT_OK ||
		ohGetByBagName(&bigModelBag, &modelBag,
			       ofeSimPtr->simulator) != OCT_OK) {
	modelBag.objectId = oct_null_id;
    }

    /* check model-switching property in model bag (e.g. -- MOSFET "level") */
    if (tablePtr->octPropName) {
	if (	    modelBag.objectId == oct_null_id ||
		    ohGetByPropName(&modelBag, &subType,
				    tablePtr->octPropName) != OCT_OK ||
		    subType.contents.prop.type != OCT_STRING ||
		    strcmp(subType.contents.prop.value.string,
			   tablePtr->octPropType)) {
	    return(0);
	}
    }

    /* we have a match -- call the appropriate leaf handling function */
    if (tablePtr->except) {
	(*tablePtr->except)(instPtr, cellPtr, tablePtr, &modelBag);
    } else {
	ofeNormLeafHandler(instPtr, cellPtr, tablePtr, &modelBag);
    }
    return(1);
}

void ofeNormLeafHandler(instPtr, cellPtr, tablePtr, modBagPtr)
octObject *instPtr;
cellDesc *cellPtr;
struct deviceMap *tablePtr;
octObject *modBagPtr;
{
    ofeGetModel(cellPtr, tablePtr->simDevice, modBagPtr);
    IF_ASSERT(newInstance, (cellPtr->cktPtr->simEquiv,
			    cellPtr->modelPtr->simEquiv,
			    &cellPtr->simEquiv, (IFuid) cellPtr));
    ofeSetInstParms(cellPtr, instPtr, tablePtr);
    ofeWireLeaf(cellPtr, tablePtr);
}

void ofeSetInstParms(cellPtr, instPtr, tablePtr)
cellDesc *cellPtr;
octObject *instPtr;
struct deviceMap *tablePtr;
{
    int i;
    struct instParms *parmPtr;
    double v, vRef;

    for (i = 0; i < tablePtr->nIParms; i++) {
	parmPtr = &tablePtr->iParms[i];
	if (parmPtr->mastInstMix == INST_OVERRIDE) {
	    if (ofeGetPropVal(instPtr, parmPtr->octName, &v) ||
		    ofeGetPropVal(&cellPtr->master, parmPtr->octName, &v)) {
		ofeSetOneInstParm(cellPtr, parmPtr->simName, v);
	    }
	} else {
	    if (ofeGetPropVal(instPtr, parmPtr->octName, &v) &&
		    ofeGetPropVal(&cellPtr->master, parmPtr->octName, &vRef)) {
		ofeSetOneInstParm(cellPtr, parmPtr->simName, v/vRef);
	    }
	}
    }
}

int ofeGetPropVal(container, name, vPtr)
octObject *container;
char *name;
double *vPtr;		/* XXX should be able to handle other parm types */
{
    octObject prop;

    if (ohGetByPropName(container, &prop, name) != OCT_OK) return(0);
    switch (prop.contents.prop.type) {
	case OCT_NULL:		/* NULL props are assumed to set flags */
	    *vPtr = 1.0;
	    return(1);
	case OCT_INTEGER:
	    *vPtr = (double) prop.contents.prop.value.integer;
	    return(1);
	case OCT_REAL:
	    *vPtr = prop.contents.prop.value.real;
	    return(1);
	default:
	    ofeMessage(WARNING, "%s on %s is of non-scalar type\n",
				ohFormatName(&prop), ohFormatName(container));
	    return(0);
    }
}

void ofeSetOneInstParm(cellPtr, parmName, parmValue)
cellDesc *cellPtr;
char *parmName;
double parmValue;	/* XXX should be able to handle other parm types */
{
    IFparm *parmPtr;
    int i;
    IFvalue value;

    for (i = 0; i < cellPtr->modelPtr->simDevPtr->numInstanceParms; i++) {
	parmPtr = &cellPtr->modelPtr->simDevPtr->instanceParms[i];
	if (strcmp(parmName, parmPtr->keyword) == 0) {
	    if (parmPtr->dataType & IF_SET) {
		switch (parmPtr->dataType & IF_VARTYPES) {
		    case IF_FLAG:
		    case IF_INTEGER:
			value.iValue = (int) parmValue;
			break;
		    case IF_REAL:
			value.rValue = parmValue;
			break;
		    default:
			ofeMessage(WARNING,
"parameter `%s' (leaf cell %s) is not of scalar type\n",
				parmName, ohFormatName(&cellPtr->master));
			return;
		}
		IF_ASSERT(setInstanceParm, (cellPtr->cktPtr->simEquiv,
					    cellPtr->simEquiv, parmPtr->id,
					    &value, NIL(IFvalue)));
		return;
	    } else if ((parmPtr->dataType & IF_UNIMP_MASK) == 0) {
		/* simulator acknowledges parameter, but doesn't implement */
		return;
	    } else {
		ofeMessage(WARNING,
			   "parameter `%s' (leaf cell %s) is not settable\n",
			   parmName, ohFormatName(&cellPtr->master));
	    }
	}
    }
    ofeMessage(WARNING, "parameter `%s' (leaf cell %s) unknown\n",
	       parmName, ohFormatName(&cellPtr->master));
}

void ofeWireLeaf(cellPtr, tablePtr)
cellDesc *cellPtr;
struct deviceMap *tablePtr;
{
    st_generator *stGen;
    char *key, *termNet;
    octObject term;
    int i, termIndex;
    IFdevice *devPtr;

    devPtr = cellPtr->modelPtr->simDevPtr;
    stGen = st_init_gen(cellPtr->termTable);
    while (st_gen(stGen, &key, &termNet)) {
	if (ohGetTerminal(&cellPtr->master, (int32) key, &term) != OCT_OK) {
	    ofeMessage(SEVERE,
		    "can't find terminal with xid=%ld in %s -- leaving unconnected\n",
		    (long) (int32) key, ohFormatName(&cellPtr->master));
	    continue;
	}
	for (i = 0; i < tablePtr->nTerms; i++) {
	    if (strcmp(tablePtr->terms[i].octName, term.contents.term.name)==0){
		for (termIndex = ABS(devPtr->terms); termIndex--; ) {
		    if (strcmp(devPtr->termNames[termIndex],
					tablePtr->terms[i].simName) == 0) {
			break;
		    }
		}
		if (termIndex >= 0) {
		    IF_ASSERT(bindNode, (cellPtr->cktPtr->simEquiv,
					 cellPtr->simEquiv, termIndex+1,
					 ((netDesc *) termNet)->simEquiv));
		} else {
		    ofeMessage(SEVERE,
			"simulator doesn't know terminal name `%s' for a `%s'\n",
			tablePtr->terms[i].simName, tablePtr->simDevice);
		}
	    }
	}
    }
    st_free_gen(stGen);
}
