#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"
#include "simtrans.h"

void ofeGetModel(cellPtr, devName, modBagPtr)
cellDesc *cellPtr;
char *devName;
octObject *modBagPtr;
{
    int i;
    char *val;
    IFuid uid;
    modelDesc *modelPtr;
    void ofeSetModelParms();

    if (st_lookup(cellPtr->cktPtr->modelTable,
		    (char *) cellPtr->master.objectId, &val)) {
	cellPtr->modelPtr = (modelDesc *) val;
	return;
    }

    if (val = strrchr(cellPtr->master.contents.facet.cell, '/')) {
	val++;
    } else {
	val = cellPtr->master.contents.facet.cell;
    }
    OFE_NEW_UID(&uid, (IFuid) cellPtr->cktPtr, val, UID_MODEL);
    modelPtr = (modelDesc *) uid;
    cellPtr->modelPtr = modelPtr;

    modelPtr->simDevPtr = NIL(IFdevice);
    for (i = 0; i < ofeSimPtr->numDevices; i++) {
	if (strcmp(devName, ofeSimPtr->devices[i]->name) == 0) {
	    modelPtr->simDevPtr = ofeSimPtr->devices[i];
	    break;
	}
    }
    if (modelPtr->simDevPtr == NIL(IFdevice)) {
	ofeMessage(FATAL, "device `%s' not supported by simulator\n", devName);
    }

    ASSERT( ! st_insert(cellPtr->cktPtr->modelTable,
			(char *) cellPtr->master.objectId, (char *) modelPtr));

    IF_ASSERT(newModel, (cellPtr->cktPtr->simEquiv, i, &modelPtr->simEquiv,
			 (IFuid) modelPtr));
    if (modBagPtr->objectId != oct_null_id) {
	ofeSetModelParms(modelPtr, modBagPtr);
    }
}

static void ofeSetModelParms(modelPtr, modBagPtr)
modelDesc *modelPtr;
octObject *modBagPtr;
{
    octGenerator gen;
    octObject prop;
    void ofeSetOneModelParm();

    octInitGenContents(modBagPtr, OCT_PROP_MASK, &gen);
    while (octGenerate(&gen, &prop) == OCT_OK) {
	switch (prop.contents.prop.type) {
	    case OCT_NULL:
		ofeSetOneModelParm(modelPtr, prop.contents.prop.name, 1.0);
		break;
	    case OCT_INTEGER:
		ofeSetOneModelParm(modelPtr, prop.contents.prop.name,
				   (double) prop.contents.prop.value.integer);
		break;
	    case OCT_REAL:
		ofeSetOneModelParm(modelPtr, prop.contents.prop.name,
				   prop.contents.prop.value.real);
		break;
	    default:
		ofeMessage(WARNING,
			   "model property `%s' is of non-scalar type\n",
			   prop.contents.prop.name);
	}
    }
}

void ofeSetOneModelParm(modelPtr, parmName, parmValue)
modelDesc *modelPtr;
char *parmName;
double parmValue;	/* XXX should be able to handle other parm types */
{
    char *lcParmName;
    IFparm *parmPtr;
    int i;
    IFvalue value;

    /* force to lower case */
    lcParmName = util_strsav(parmName);
    for (i = 0; parmName[i] != '\0'; i++) {
	if (isupper(parmName[i])) lcParmName[i] = tolower(parmName[i]);
    }

    for (i = 0; i < modelPtr->simDevPtr->numModelParms; i++) {
	parmPtr = &modelPtr->simDevPtr->modelParms[i];
	if (strcmp(lcParmName, parmPtr->keyword) == 0) {
	    /* we found the parameter -- can we set it? */
	    FREE(lcParmName);
	    if (parmPtr->dataType & IF_SET) {
		/* it is settable */
		switch (parmPtr->dataType & IF_VARTYPES) {
		    case IF_FLAG:
		    case IF_INTEGER:
			value.iValue = (int) parmValue;
			break;
		    case IF_REAL:
			value.rValue = parmValue;
			break;
		    default:
			ofeMessage(WARNING, "parameter %s not of scalar type\n",
								    parmName);
			return;
		}
		IF_ASSERT(setModelParm,
			    (modelPtr->cktPtr->simEquiv, modelPtr->simEquiv,
			     parmPtr->id, &value, NIL(IFvalue)));
	    } else if ((parmPtr->dataType & IF_UNIMP_MASK) == 0) {
		/* simulator acknowledges parameter, but doesn't implement */
	    } else {
		ofeMessage(WARNING, "parameter `%s' is not settable\n",
				    parmName);
	    }

	    return;
	}
    }

    FREE(lcParmName);
    ofeMessage(WARNING, "parameter `%s' unknown\n", parmName);
}
