#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"

void ofeModule(cellPtr)
cellDesc *cellPtr;
{
    octObject instBag;
    octGenerator gen;
    octObject instance;
    cellDesc *subCellPtr;
    IFuid uid;
    int count = 0;
    void ofeBuildTermList(), ofeHandleMaster();

    if (ohGetByBagName(&cellPtr->master, &instBag, "INSTANCES") == OCT_OK) {
	octInitGenContents(&instBag, OCT_INSTANCE_MASK, &gen);
	while (octGenerate(&gen, &instance) == OCT_OK) {
	    OFE_NEW_UID(&uid,(IFuid) cellPtr,ohGetName(&instance),UID_INSTANCE);
	    subCellPtr = (cellDesc *) uid;
	    subCellPtr->parentPtr = cellPtr;
	    (void) st_insert(cellPtr->childTable, (char *) instance.objectId,
			     (char *) subCellPtr);

	    ofeBuildTermList(cellPtr, subCellPtr, &instance);
	    ofeHandleMaster(subCellPtr, &instance);

	    count++;
	}
    }
    if (count == 0) {
	ofeMessage(WARNING, "no instances inside %s\n",
			    ohFormatName(&cellPtr->master));
    }
}

static void ofeBuildTermList(cellPtr, subCellPtr, instPtr)
cellDesc *cellPtr;
cellDesc *subCellPtr;
octObject *instPtr;
{
    octGenerator gen;
    octObject term, net;
    netDesc *netPtr;

    octInitGenContents(instPtr, OCT_TERM_MASK, &gen);
    while (octGenerate(&gen, &term) == OCT_OK) {
	if (octGenFirstContainer(&term, OCT_NET_MASK, &net) == OCT_OK) {
	    netPtr = ofeFindNet(cellPtr, ohGetName(&net), net.objectId);
	    ASSERT( ! st_insert(subCellPtr->termTable,
				(char *) term.contents.term.formalExternalId,
				(char *) netPtr));
	} else {
	    ofeMessage(WARNING, "%s on %s in %s unconnected\n",
		       ohFormatName(&term), ohFormatName(instPtr),
		       ohFormatName(&cellPtr->master));
	}
    }
}

static void ofeHandleMaster(cellPtr, instPtr)
cellDesc *cellPtr;
octObject *instPtr;
{
    octObject cellClass;
    void ofeWireModuleNets();

    OH_ASSERT(ohOpenMaster(&cellPtr->master, instPtr, "interface", "r"));

    if (ohGetByPropName(&cellPtr->master, &cellClass, "CELLCLASS") != OCT_OK ||
				cellClass.contents.prop.type != OCT_STRING) {
	ofeMessage(SEVERE, "no %s property found in %s\n",
			cellClass.contents.prop.name,
			ohFormatName(&cellPtr->master));
    } else {
	if (strcmp(cellClass.contents.prop.value.string, "MODULE") == 0) {
	    cellPtr->cellClass = MODULE;
	    OH_ASSERT(ohOpenMaster(&cellPtr->master, instPtr, "contents", "r"));
	    ofeWireModuleNets(cellPtr);
	    ofeModule(cellPtr);
	} else if (strcmp(cellClass.contents.prop.value.string, "LEAF") == 0) {
	    cellPtr->cellClass = LEAF;
	    ofeLeaf(instPtr, cellPtr);
	} else {
	    ofeMessage(SEVERE, "bad %s property (%s) in %s\n",
		    cellClass.contents.prop.name,
		    ohFormatName(&cellClass), ohFormatName(&cellPtr->master));
	}
    }
}

static void ofeWireModuleNets(cellPtr)
cellDesc *cellPtr;
{
    st_generator *stGen;
    char *key, *termNet;
    octObject term;
    octObject net;
    void ofeDoInstances();

    /* put formal terminal nets in net table */
    stGen = st_init_gen(cellPtr->termTable);
    while (st_gen(stGen, &key, &termNet)) {
	if (ohGetTerminal(&cellPtr->master, (int32) key, &term) == OCT_OK) {
	    if (octGenFirstContainer(&term, OCT_NET_MASK, &net) == OCT_OK) {
		if (st_insert(cellPtr->netTable, (char*)net.objectId, termNet)){
		    ofeMessage(FATAL, "%s illegally aliases terminals in %s\n",
			    ohFormatName(&net), ohFormatName(&cellPtr->master));
		}
	    } else {
		ofeMessage(WARNING, "no net on %s in %s\n",
			ohFormatName(&term), ohFormatName(&cellPtr->master));
	    }
	} else {
	    ofeMessage(SEVERE,
		    "can't find terminal with xid=%ld in %s -- leaving unconnected\n",
		    (long) (int32) key, ohFormatName(&cellPtr->master));
	}
    }
    st_free_gen(stGen);
}
