#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"

void ofeOutInit()
{
    int ofeBeginPlot(), ofeDataPoint(), ofeEndPlot();

    ofeFEstruct.OUTpBeginPlot = ofeBeginPlot;
    ofeFEstruct.OUTpData = ofeDataPoint;
    ofeFEstruct.OUTendPlot = ofeEndPlot;
}

/*ARGSUSED*/
static int ofeBeginPlot(simCkt, simAnal, analUid, refUid, refType,
			    nNodes, dataUidList, dataType, plotPtr)
GENERIC *simCkt;
GENERIC *simAnal;
IFuid analUid;
IFuid refUid;
int refType;
int nNodes;
IFuid *dataUidList;
int dataType;
GENERIC **plotPtr;
{
    analDesc *analPtr = (analDesc *) analUid;
    int i;
    waveStruct *buildWave();

    refType &= IF_VARTYPES;
    dataType &= IF_VARTYPES;
    analPtr->indepWave = buildWave((netDesc *) refUid, refType, 0);
    analPtr->nDepWaves = nNodes;
    analPtr->depWaves = ALLOC(waveStruct *, nNodes);
    analPtr->depValType = dataType;
    for (i = 0; i < nNodes; i++) {
	if (dataUidList[i] == refUid) {
	    /* avoid getting screwed up by storing this twice */
	    analPtr->depWaves[i] = NIL(waveStruct);
	} else {
	    analPtr->depWaves[i] =
			    buildWave((netDesc *) dataUidList[i], dataType, 1);
	}
	if (analPtr->depWaves[i]) {
	    (void) st_insert(analPtr->waveTable,
			     (char *) analPtr->depWaves[i]->netPtr->netId,
			     (char *) analPtr->depWaves[i]);
	}
    }
    *plotPtr = (GENERIC *) analUid;
    return(OK);
}

static waveStruct *buildWave(netPtr, dataType, checkId)
netDesc *netPtr;
int dataType;
int checkId;
{
    waveStruct *wavePtr;
    dary waveValue;

    if (checkId && ! st_is_member(netPtr->cktPtr->rootPtr->netTable,
						(char *) netPtr->netId)) {
	return(NIL(waveStruct));
    }

    switch (dataType) {
	case IF_FLAG:
	case IF_INTEGER:
	case IF_REAL:
	    waveValue = daAlloc(double, 0);
	    break;
	case IF_COMPLEX:
	    waveValue = daAlloc(IFcomplex, 0);
	    break;
	default:
	    return(NIL(waveStruct));
    }

    wavePtr = ALLOC(waveStruct, 1);
    wavePtr->netPtr = netPtr;
    wavePtr->valType = dataType;
    wavePtr->val = waveValue;

    return(wavePtr);
}

#define STORE_INDEP_VAL(T, F)		\
    if (wavePtr = analPtr->indepWave) {					\
	*daSetLast(T, wavePtr->val) = (refValue->F);			\
    }									\
    break;

#define STORE_DEP_VALS(T, F)		\
    for (i = analPtr->nDepWaves; i--; ) {				\
	if (wavePtr = analPtr->depWaves[i]) {				\
	    *daSetLast(T, wavePtr->val) = (dataValueVector->v.vec.F);	\
	}								\
    }									\
    break;

static int ofeDataPoint(plotPtr, refValue, dataValueVector)
GENERIC *plotPtr;
IFvalue *refValue;
register IFvalue *dataValueVector;
{
    register waveStruct *wavePtr;
    register int i;
    register analDesc *analPtr = (analDesc *) plotPtr;

    if (wavePtr = analPtr->indepWave) {
	switch (wavePtr->valType) {
	    case IF_FLAG:	STORE_INDEP_VAL(double, iValue != 0);
	    case IF_INTEGER:	STORE_INDEP_VAL(double, iValue);
	    case IF_REAL:	STORE_INDEP_VAL(double, rValue);
	    case IF_COMPLEX:	STORE_INDEP_VAL(IFcomplex, cValue);
	}
    }

    switch (analPtr->depValType) {
	case IF_FLAG:		STORE_DEP_VALS(double, iVec[i] != 0);
	case IF_INTEGER:	STORE_DEP_VALS(double, iVec[i]);
	case IF_REAL:		STORE_DEP_VALS(double, rVec[i]);
	case IF_COMPLEX:	STORE_DEP_VALS(IFcomplex, cVec[i]);
    }
}

/*ARGSUSED*/
static int ofeEndPlot(plotPtr)
GENERIC *plotPtr;
{
    return(OK);
}
