#include "copyright.h"
#include "port.h"
#include "utility.h"
#include <X/Xlib.h>
#include "XDM.h"
#include <varargs.h>
#include "rpc.h"
#include "oct.h"
#include "oh.h"
#include "st.h"
#include "IFsim.h"
#include "IFerrmsgs.h"
#include "octsim.h"
#include "rpc-spice.h"

static char *progName = "?rpc-spice?";
static octObject groundObj;
static cktDesc *curCktPtr = NIL(cktDesc);
static analDesc *curAnalPtr = NIL(analDesc);

/*ARGSUSED*/
{
    ofeDisplayName = display;

    if (XOpenDisplay(ofeDisplayName) == NIL(Display)) {
	ofeMessage(FATAL, "couldn't open X display\n");
    }

    XDM_ASSERT(XDMInit(progName));


    ofeMessage(INFO, "analysis completed\n");
    return(RPC_OK);
}

static void buildAnalysisDialog(dlgPtr, title)
struct dialog *dlgPtr;
char *title;
{
    Window dWin;
    int bigColId, titleId;

    if (dlgPtr->dialogWin) return;

    dlgPtr->dialogWin = dWin = XDMDialogCreate(RootWindow);

    bigColId = XDMRowColCreate(dWin, XDM_WINDOW);
    XDM_ASSERT(XDMModify(dWin, bigColId, XDM_SPACE, 5, XDM_END));

    titleId = XDMTextCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, title, XDM_END));

    ofeBuildFlagList(dWin, bigColId, dlgPtr->nFlags, dlgPtr->flagArray);

    ofeBuildValueList(dWin, bigColId, dlgPtr->nValues, dlgPtr->valueArray);

    ofeBuildActionButtons(dWin, bigColId, &dlgPtr->OKId, &dlgPtr->abortId);
}

static void buildPlotDialog(dlgPtr)
struct dialog *dlgPtr;
{
    Window dWin;
    int bigColId, titleId, rowId, smallColId;
    void fillPlotData();

    fillPlotData(dlgPtr);

    dlgPtr->dialogWin = dWin = XDMDialogCreate(RootWindow);

    bigColId = XDMRowColCreate(dWin, XDM_WINDOW);
    XDM_ASSERT(XDMModify(dWin, bigColId, XDM_SPACE, 5, XDM_END));

    titleId = XDMTextCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, "Plot Options", XDM_END));

    rowId = XDMRowColCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, rowId, XDM_ISROW, 1, XDM_END));

    /* first column -- dependent variable */
    smallColId = XDMRowColCreate(dWin, rowId);
    titleId = XDMTextCreate(dWin, smallColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, "Y axis", XDM_END));
    ofeBuildFlagList(dWin, smallColId, dlgPtr->nCol1Flags, dlgPtr->flagArray);

    /* second column -- independent variable */
    smallColId = XDMRowColCreate(dWin, rowId);
    titleId = XDMTextCreate(dWin, smallColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, "X axis", XDM_END));
    ofeBuildFlagList(dWin, smallColId, dlgPtr->nFlags - dlgPtr->nCol1Flags,
		    &dlgPtr->flagArray[dlgPtr->nCol1Flags]);

    ofeBuildActionButtons(dWin, bigColId, &dlgPtr->OKId, &dlgPtr->abortId);
}

void ofeBuildFlagList(dWin, parentId, nFlags, fPtr)
Window dWin;
int parentId;
int nFlags;
struct flagStruct *fPtr;
{
    int i;
    int textId;

    for (i = 0; i < nFlags; i++) {
	fPtr->fieldId = XDMButtonCreate(dWin, parentId);
	textId = XDMTextCreate(dWin, fPtr->fieldId);
	XDM_ASSERT(XDMModify(dWin, fPtr->fieldId,
			    XDM_SETSIGNAL, 1, XDM_VALUE, fPtr->flag, XDM_END));
	XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, fPtr->name, XDM_END));
	fPtr++;
    }
}

void ofeBuildValueList(dWin, parentId, nValues, vPtr)
Window dWin;
int parentId;
int nValues;
struct valueStruct *vPtr;
{
    char string[100];
    int i;
    int rowId, textId;

    for (i = 0; i < nValues; i++) {
	rowId = XDMRowColCreate(dWin, parentId);
	XDM_ASSERT(XDMModify(dWin, rowId, XDM_ISROW, 1, XDM_END));

	textId = XDMTextCreate(dWin, rowId);
	XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, vPtr->name, XDM_END));

	(void) sprintf(string, "%g", vPtr->value);
	vPtr->fieldId = XDMEdRegCreate(dWin, rowId);
	XDM_ASSERT(XDMModify(dWin, vPtr->fieldId,
			    XDM_ROWS, 1, XDM_COLS, COL_WIDTH,
			    XDM_USER, (XDMPointer) vPtr,
			    XDM_SETSIGNAL, 1, XDM_TEXT, string, XDM_END));
	vPtr++;
    }
}

void ofeBuildActionButtons(dWin, parentId, OKIdPtr, abortIdPtr)
Window dWin;
int parentId;
int *OKIdPtr;
int *abortIdPtr;
{
    int rowId, textId;

    rowId = XDMRowColCreate(dWin, parentId);
    XDM_ASSERT(XDMModify(dWin, rowId, XDM_ISROW, 1, XDM_END));

    *OKIdPtr = XDMButtonCreate(dWin, rowId);
    textId = XDMTextCreate(dWin, *OKIdPtr);
    XDM_ASSERT(XDMModify(dWin, *OKIdPtr, XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, "OK", XDM_END));

    *abortIdPtr = XDMButtonCreate(dWin, rowId);
    textId = XDMTextCreate(dWin, *abortIdPtr);
    XDM_ASSERT(XDMModify(dWin, *abortIdPtr, XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, "Abort", XDM_END));
}

int ofeDisplayDialog(dlgPtr)
struct dialog *dlgPtr;
{
    Window dWin = dlgPtr->dialogWin;
    XEvent event;
    Window eventWin;
    int eventId;
    int waiting, retVal, i;
    void updateValue(), updateArray();

    XDM_ASSERT(XDMModify(dWin, dlgPtr->OKId, XDM_VALUE, 0, XDM_END));
    XDM_ASSERT(XDMModify(dWin, dlgPtr->abortId, XDM_VALUE, 0, XDM_END));

    XDM_ASSERT(XDMPost(dWin, 0, 0, (int (*)()) 0, XDM_INTERACT));

    waiting = 1;
    while (XPending() > 0 || waiting) {
	XNextEvent(&event);
	if (XDMFilter(&event, &eventWin, &eventId) < 0 && eventWin == dWin) {
	    switch (XDMTypeQuery(eventWin, eventId)) {
		case XDM_BTN_OBJ:
		    if (eventId == dlgPtr->abortId) {
			XDM_ASSERT(XDMEnd(dWin));
			waiting = 0;
			retVal = 0;
		    } else if (eventId == dlgPtr->OKId) {
			XDM_ASSERT(XDMEnd(dWin));
			waiting = 0;
			retVal = 1;
		    }
		    break;
		case XDM_EDREG_OBJ:
		    for (i = 0; i < dlgPtr->nValues; i++) {
			if (eventId == dlgPtr->valueArray[i].fieldId) {
			    updateValue(dWin, &dlgPtr->valueArray[i]);
			    break;
			}
		    }
		    for (i = 0; i < dlgPtr->nArrays; i++) {
			if (eventId == dlgPtr->arrayArray[i].fieldId) {
			    updateArray(dWin, &dlgPtr->arrayArray[i]);
			    break;
			}
		    }
		    break;
	    }
	}
    }

    for (i = 0; i < dlgPtr->nFlags; i++) {
	XDM_ASSERT(XDMQuery(dWin, dlgPtr->flagArray[i].fieldId,
			    XDM_VALUE, &dlgPtr->flagArray[i].flag, XDM_END));
    }
    for (i = 0; i < dlgPtr->nValues; i++) {
	updateValue(dWin, &dlgPtr->valueArray[i]);
    }
    for (i = 0; i < dlgPtr->nArrays; i++) {
	updateArray(dWin, &dlgPtr->arrayArray[i]);
    }
    return(retVal);
}

static void updateValue(dWin, vPtr)
Window dWin;
struct valueStruct *vPtr;
{
    char *inStr, outStr[100];
    double atof();

    XDM_ASSERT(XDMQuery(dWin, vPtr->fieldId, XDM_TEXT, &inStr, XDM_END));
    vPtr->value = atof(inStr);
    (void) sprintf(outStr, "%g", vPtr->value);
    XDM_ASSERT(XDMModify(dWin, vPtr->fieldId, XDM_TEXT, outStr, XDM_END));
}

static void updateArray(dWin, aPtr)
Window dWin;
struct arrayStruct *aPtr;
{
    char *inStr, outStr[50*MAX_ARRAY];
    int i;
    double atof();

    XDM_ASSERT(XDMQuery(dWin, aPtr->fieldId, XDM_TEXT, &inStr, XDM_END));
    for (i = 0; i < MAX_ARRAY; i++) {
	while (*inStr != '\0' && isspace(*inStr)) inStr++;
	if (*inStr == '\0') break;
	aPtr->values[i] = atof(inStr);
	while (*inStr != '\0' && ! isspace(*inStr)) inStr++;
    }
    aPtr->nVals = i;

    outStr[0] = '\0';
    for (i = 0; i < aPtr->nVals; i++) {
	(void) sprintf(outStr + strlen(outStr), "%g\n", aPtr->values[i]);
    }
    XDM_ASSERT(XDMModify(dWin, aPtr->fieldId, XDM_TEXT, outStr, XDM_END));
}
