#include "copyright.h"
#include "port.h"
#include "utility.h"
#include <X/Xlib.h>
#include "XDM.h"
#include <varargs.h>
#include "rpc.h"
#include "oct.h"
#include "oh.h"
#include "st.h"
#include "IFsim.h"
#include "IFerrmsgs.h"
#include "octsim.h"
#include "rpc-spice.h"

/*ARGSUSED*/
int instanceProps(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
/*
 * Edits instance properties
 * Format for the command:  instance : instance-props
 */
{
    RPCArg *firstArgPtr;
    octObject argBag, instance, facet;
    char *format = "\nformat:  instance : instance-props\n";
    struct dialog *dlgPtr;
    struct dialog *getInstDialog();
    void updateInstProps();

    if (lsLength(cmdList) != 1) {
	BAD_FMT(format);
    }

    LS_ASSERT(lsFirstItem(cmdList, (Pointer *) &firstArgPtr, NIL(lsHandle)));
    if (firstArgPtr->argType != VEM_OBJ_ARG || firstArgPtr->length != 1) {
	BAD_FMT(format);

    }

    argBag.objectId = firstArgPtr->argData.objArg.theBag;
    OH_ASSERT(octGetById(&argBag));
    if (octGenFirstContent(&argBag, OCT_INSTANCE_MASK, &instance) != OCT_OK) {
	BAD_FMT(format);
    }
    OH_ASSERT(octGetFacet(&instance, &facet));

    dlgPtr = getInstDialog(&instance);
    if (dlgPtr && ofeDisplayDialog(dlgPtr)) {
	updateInstProps(dlgPtr, &instance, &facet);
    }
    return(RPC_OK);
}

static struct dialog *getInstDialog(instPtr)
octObject *instPtr;
{
    static st_table *dialogTable = NIL(st_table);
    octObject master;
    char *valPtr;
    struct dialog *dlgPtr;
    int buildInstDialog();
    void fillInstDialog();

    if (dialogTable == NIL(st_table)) {
	dialogTable = st_init_table(st_numcmp, st_numhash);
    }

    master.type = OCT_FACET;
    master.contents.facet.facet = "interface";
    master.contents.facet.mode = "r";
    OH_ASSERT(octOpenMaster(instPtr, &master));
    if (st_lookup(dialogTable, (char *) master.objectId, &valPtr)) {
	dlgPtr = (struct dialog *) valPtr;
    } else {
	dlgPtr = ALLOC(struct dialog, 1);
	CLEAR_DIALOG(*dlgPtr);
	if (buildInstDialog(dlgPtr, &master)) {
	    ASSERT( ! st_insert(dialogTable,
				(char *) master.objectId, (char *) dlgPtr));
	} else {
	    FREE(dlgPtr);
	    return(NIL(struct dialog));
	}
    }

    fillInstDialog(dlgPtr, instPtr);

    return(dlgPtr);
}

static int buildInstDialog(dlgPtr, masterPtr)
struct dialog *dlgPtr;
octObject *masterPtr;
{
    char *cellType;
    Window dWin;
    int bigColId, titleId;
    octGenerator gen;
    octStatus stat;
    octObject prop;
    int i;
    void buildArrayList();

    prop.type = OCT_PROP;
    prop.contents.prop.name = "CELLTYPE";
    if (octGetByName(masterPtr, &prop) != OCT_OK) {
	ofeMessage(SEVERE, "Can't find `%s' property in master\n",
						prop.contents.prop.name);
	return(0);
    }
    cellType = prop.contents.prop.value.string;

    dlgPtr->nFlags = dlgPtr->nValues = dlgPtr->nArrays = 0;
    dlgPtr->valueArray = NIL(struct valueStruct);
    dlgPtr->arrayArray = NIL(struct arrayStruct);

    OH_ASSERT(octInitGenContents(masterPtr, OCT_PROP_MASK, &gen));
    while ((stat = octGenerate(&gen, &prop)) == OCT_OK) {
	switch (prop.contents.prop.type) {
	    case OCT_REAL:
		i = dlgPtr->nValues++;
		dlgPtr->valueArray = REALLOC(struct valueStruct,
				    dlgPtr->valueArray, dlgPtr->nValues);
		CLEAR_VALUE(dlgPtr->valueArray[i]);
		dlgPtr->valueArray[i].name = prop.contents.prop.name;
		dlgPtr->valueArray[i].dfltValue = prop.contents.prop.value.real;
		break;
	    case OCT_REAL_ARRAY:
		i = dlgPtr->nArrays++;
		dlgPtr->arrayArray = REALLOC(struct arrayStruct,
				    dlgPtr->arrayArray, dlgPtr->nArrays);
		CLEAR_ARRAY(dlgPtr->arrayArray[i]);
		dlgPtr->arrayArray[i].name = prop.contents.prop.name;
		dlgPtr->arrayArray[i].values = ALLOC(double, MAX_ARRAY);
		break;
	}
    }
    OH_ASSERT(stat);

    dlgPtr->dialogWin = dWin = XDMDialogCreate(RootWindow);

    bigColId = XDMRowColCreate(dWin, XDM_WINDOW);
    XDM_ASSERT(XDMModify(dWin, bigColId, XDM_SPACE, 5, XDM_END));

    titleId = XDMTextCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, cellType, XDM_END));

    ofeBuildValueList(dWin, bigColId, dlgPtr->nValues, dlgPtr->valueArray);
    buildArrayList(dWin, bigColId, dlgPtr->nArrays, dlgPtr->arrayArray);
    ofeBuildActionButtons(dWin, bigColId, &dlgPtr->OKId, &dlgPtr->abortId);

    return(1);
}

static void buildArrayList(dWin, parentId, nArrays, aPtr)
Window dWin;
int parentId;
int nArrays;
struct arrayStruct *aPtr;
{
    int i;
    int bigRowId, colId, textId;

    bigRowId = XDMRowColCreate(dWin, parentId);
    XDM_ASSERT(XDMModify(dWin, bigRowId, XDM_ISROW, 1, XDM_END));

    for (i = 0; i < nArrays; i++) {
	colId = XDMRowColCreate(dWin, bigRowId);

	textId = XDMTextCreate(dWin, colId);
	XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, aPtr->name, XDM_END));

	aPtr->fieldId = XDMEdRegCreate(dWin, colId);
	XDM_ASSERT(XDMModify(dWin, aPtr->fieldId,
			    XDM_ROWS, MAX_ARRAY, XDM_COLS, COL_WIDTH,
			    XDM_USER, (XDMPointer) aPtr,
			    XDM_SETSIGNAL, 1, XDM_END));
	aPtr++;
    }
}

static void fillInstDialog(dlgPtr, instPtr)
struct dialog *dlgPtr;
octObject *instPtr;
{
    octObject prop;
    int i, j;
    char string[50*MAX_ARRAY];

    prop.type = OCT_PROP;

    for (i = 0; i < dlgPtr->nValues; i++) {
	prop.contents.prop.name = dlgPtr->valueArray[i].name;
	if (octGetByName(instPtr, &prop) == OCT_OK &&
				prop.contents.prop.type == OCT_REAL) {
	    dlgPtr->valueArray[i].value = prop.contents.prop.value.real;
	} else {
	    dlgPtr->valueArray[i].value = dlgPtr->valueArray[i].dfltValue;
	}
	(void) sprintf(string, "%g", dlgPtr->valueArray[i].value);
	XDM_ASSERT(XDMModify(dlgPtr->dialogWin, dlgPtr->valueArray[i].fieldId,
						XDM_TEXT, string, XDM_END));
    }

    for (i = 0; i < dlgPtr->nArrays; i++) {
	prop.contents.prop.name = dlgPtr->arrayArray[i].name;
	string[0] = '\0';
	if (octGetByName(instPtr, &prop) == OCT_OK &&
				prop.contents.prop.type == OCT_REAL_ARRAY) {
	    dlgPtr->arrayArray[i].nVals =
		    MIN(prop.contents.prop.value.real_array.length, MAX_ARRAY);
	    for (j = 0; j < dlgPtr->arrayArray[i].nVals; j++) {
		dlgPtr->arrayArray[i].values[j] =
				prop.contents.prop.value.real_array.array[j];
		(void) sprintf(string + strlen(string), "%g\n",
					    dlgPtr->arrayArray[i].values[j]);
	    }
	} else {
	    dlgPtr->arrayArray[i].nVals = 0;
	}
	XDM_ASSERT(XDMModify(dlgPtr->dialogWin, dlgPtr->arrayArray[i].fieldId,
						XDM_TEXT, string, XDM_END));
    }
}

static void updateInstProps(dlgPtr, instPtr, facetPtr)
struct dialog *dlgPtr;
octObject *instPtr;
octObject *facetPtr;
{
    int i;
    octObject prop;
    struct octBox bb;
    void fixLabels();

    prop.type = OCT_PROP;
    for (i = 0; i < dlgPtr->nValues; i++) {
	prop.contents.prop.name = dlgPtr->valueArray[i].name;
	if (octGetByName(instPtr, &prop) == OCT_OK) {
	    OH_ASSERT(octDelete(&prop));
	}
	if (dlgPtr->valueArray[i].value != dlgPtr->valueArray[i].dfltValue) {
	    prop.contents.prop.type = OCT_REAL;
	    prop.contents.prop.value.real = dlgPtr->valueArray[i].value;
	    OH_ASSERT(octCreate(instPtr, &prop));
	}
    }

    for (i = 0; i < dlgPtr->nArrays; i++) {
	prop.contents.prop.name = dlgPtr->arrayArray[i].name;
	if (octGetByName(instPtr, &prop) == OCT_OK) {
	    OH_ASSERT(octDelete(&prop));
	}
	if (dlgPtr->arrayArray[i].nVals > 0) {
	    prop.contents.prop.type = OCT_REAL_ARRAY;
	    prop.contents.prop.value.real_array.length =
						dlgPtr->arrayArray[i].nVals;
	    prop.contents.prop.value.real_array.array =
						dlgPtr->arrayArray[i].values;
	    OH_ASSERT(octCreate(instPtr, &prop));
	}
    }

    fixLabels(instPtr, facetPtr);

    OH_ASSERT(octBB(instPtr, &bb));
    /* XXX -- grody and basically ineffective hack to make labels update */
    bb.lowerLeft.y -= 2 * (bb.upperRight.y - bb.lowerLeft.y);
    bb.upperRight.x += 3 * (bb.upperRight.x - bb.lowerLeft.x);
    bb.lowerLeft.x--;
    bb.upperRight.y++;
    vemWnQRegion(facetPtr, &bb);
    vemWnFlush();
}

/* XXX -- from here to the end of the file is a grotesque hack !!! */

static void fixLabels(instPtr, facetPtr)
octObject *instPtr;
octObject *facetPtr;
{
    octObject master;
    octObject labelLayer;
    octObject newLabel;
    octObject labelBag;
    octObject oldLabel;
    octGenerator gen;
    octStatus stat;
    int buildLabel();


    master.type = OCT_FACET;
    master.contents.facet.facet = "interface";
    master.contents.facet.mode = "r";
    OH_ASSERT(octOpenMaster(instPtr, &master));

    if (buildLabel(instPtr, &master, &newLabel, &labelLayer)) {
	OH_ASSERT(octGetOrCreate(facetPtr, &labelLayer));
	OH_ASSERT(octCreate(&labelLayer, &newLabel));

	labelBag.type = OCT_BAG;
	labelBag.contents.bag.name = LABEL_BAG_NAME;
	OH_ASSERT(octGetOrCreate(instPtr, &labelBag));

	OH_ASSERT(octInitGenContents(&labelBag, OCT_LABEL_MASK, &gen));
	while ((stat = octGenerate(&gen, &oldLabel)) == OCT_OK) {
	    OH_ASSERT(octDelete(&oldLabel));
	}
	OH_ASSERT(stat);

	OH_ASSERT(octAttach(&labelBag, &newLabel));
    }
}

static int buildLabel(instPtr, masterPtr, labelPtr, layerPtr)
octObject *instPtr;
octObject *masterPtr;
octObject *labelPtr;
octObject *layerPtr;
{
    int isLeaf(), getPlace(), getText();

    if (isLeaf(masterPtr) && getPlace(masterPtr, labelPtr, layerPtr) &&
			    getText(instPtr, masterPtr, labelPtr)) {
	OH_ASSERT(octTransformGeo(labelPtr,
				&instPtr->contents.instance.transform));
	return(1);
    } else {
	return(0);
    }
}

static int isLeaf(facetPtr)
octObject *facetPtr;
{
    octObject cellClassProp;

    cellClassProp.type = OCT_PROP;
    cellClassProp.contents.prop.name = "CELLCLASS";

    return( octGetByName(facetPtr, &cellClassProp) == OCT_OK &&
	    cellClassProp.contents.prop.type == OCT_STRING &&
	    strcmp(cellClassProp.contents.prop.value.string, "LEAF") == 0 );
}

static int getPlace(facetPtr, labelPtr, layerPtr)
octObject *facetPtr;
octObject *labelPtr;
octObject *layerPtr;
{
    octGenerator gen;
    octStatus stat;

    OH_ASSERT(octInitGenContents(facetPtr, OCT_LABEL_MASK, &gen));
    while ((stat = octGenerate(&gen, labelPtr)) == OCT_OK) {
	if (labelPtr->contents.label.label == NIL(char) ||
			    strlen(labelPtr->contents.label.label) == 0) {
	    OH_ASSERT(octFreeGenerator(&gen));
	    return(octGenFirstContainer(labelPtr, OCT_LAYER_MASK, layerPtr)
								    == OCT_OK);
	}
    }
    OH_ASSERT(stat);
    return(0);
}

struct nameStruct {
    char *type;
    struct format {
	char *text;
	int (*formatter)();
	char *propName;
    } formatList[10];
};

#define NIL_FUNC	((int (*)()) 0)
static int engrFormat(), capFormat(), floatFormat(), netNameFormat();
struct nameStruct nameTable[] ={
    {	"RESISTOR",
	{   {	"",	engrFormat,		"RESISTANCE"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"CAPACITOR",
	{   {	"",	capFormat,		"CAPACITANCE"	},
	    {	"F",	NIL_FUNC,		""		}   }	},
    {	"INDUCTOR",
	{   {	"",	engrFormat,		"INDUCTANCE"	},
	    {	"H",	NIL_FUNC,		""		}   }	},
    {	"DIODE",
	{   {	"",	floatFormat,		"DIODEAREA"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"BJT",
	{   {	"",	floatFormat,		"BJTAREA"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"MOSFET",
	{   {	"",	floatFormat,		"MOSFETWIDTH"	},
	    {	"/",	floatFormat,		"MOSFETLENGTH"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"CONTACT",
	{   {	"",	netNameFormat,		"CELLTYPE"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"SOURCE",
	{   {	"",	engrFormat,		"SOURCEDCVALUE"	},
	    {	"",	NIL_FUNC,		""		}   }	},
};
#define N_NAMES		(sizeof(nameTable)/sizeof(nameTable[0]))

static int getText(instPtr, facetPtr, labelPtr)
octObject *instPtr;
octObject *facetPtr;
octObject *labelPtr;
{
    octObject prop;
    struct nameStruct *nameEnt;
    static char labelText[1000];
    int i;
    struct nameStruct *getNameEnt();
    char *strcat();

    prop.type = OCT_PROP;
    prop.contents.prop.name = "CELLTYPE";

    if (octGetByName(facetPtr, &prop) == OCT_OK &&
		prop.contents.prop.type == OCT_STRING &&
		(nameEnt = getNameEnt(prop.contents.prop.value.string))) {
	labelText[0] = '\0';
	for (i = 0; ; i++) {
	    (void) strcat(labelText, nameEnt->formatList[i].text);
	    if (nameEnt->formatList[i].formatter == NIL_FUNC) {
		/* done it all successfully */
		labelPtr->contents.label.label = labelText;
		return(1);
	    } else {
		prop.contents.prop.name = nameEnt->formatList[i].propName;
		if (	(octGetByName(instPtr, &prop) == OCT_OK ||
				octGetByName(facetPtr, &prop) == OCT_OK) &&
			(*nameEnt->formatList[i].formatter)(labelText, &prop,
						instPtr, facetPtr)) {
		    /* formatted this part successfully */
		} else {
		    return(0);
		}
	    }
	}
    }
    return(0);
}

static struct nameStruct *getNameEnt(cellType)
char *cellType;
{
    struct nameStruct *namePtr;

    for (namePtr = nameTable; namePtr < &nameTable[N_NAMES]; namePtr++) {
	if (strcmp(cellType, namePtr->type) == 0) return(namePtr);
    }
    return(NIL(struct nameStruct));
}

struct engrScale {
    double low;
    double factor;
    char *prefix;
};
static struct engrScale fullScale[] = {
    {	1e+16,	1,	""	},
    {	1e+12,	1e+12,	"T"	},
    {	1e+9,	1e+9,	"G"	},
    {	1e+6,	1e+6,	"M"	},
    {	1e+3,	1e+3,	"k"	},
    {	1e-1,	1,	""	},
    {	1e-3,	1e-3,	"m"	},
    {	1e-6,	1e-6,	"u"	},
    {	1e-9,	1e-9,	"n"	},
    {	1e-12,	1e-12,	"p"	},
    {	1e-15,	1e-15,	"f"	},
    {	1e-18,	1e-18,	"a"	},
    {	0,	1,	""	},
};
static struct engrScale capScale[] = {
    {	1e-1,	1,	""	},
    {	1e-8,	1e-6,	"u"	},
    {	1e-12,	1e-12,	"p"	},
    {	1e-15,	1e-15,	"f"	},
    {	1e-18,	1e-18,	"a"	},
    {	0,	1,	""	},
};
static struct engrScale noScale[] = {
    {	0,	1,	""	},
};

/*ARGSUSED*/
static int engrFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;
octObject *instPtr, *masterPtr;		/* not used */
{
    int realFormat();

    return(realFormat(outString, propPtr, fullScale));
}

/*ARGSUSED*/
static int capFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;
octObject *instPtr, *masterPtr;		/* not used */
{
    int realFormat();

    return(realFormat(outString, propPtr, capScale));
}

/*ARGSUSED*/
static int floatFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;
octObject *instPtr, *masterPtr;		/* not used */
{
    int realFormat();

    return(realFormat(outString, propPtr, noScale));
}

static int realFormat(outString, propPtr, scalePtr)
char *outString;
octObject *propPtr;
struct engrScale *scalePtr;
{
    double v;
    char *strchr();

    if (propPtr->contents.prop.type != OCT_REAL) return(0);
    v = propPtr->contents.prop.value.real;

    outString += strlen(outString);

    if (v < 0) {
	v = -v;
	*outString++ = '-';
    }
    while (scalePtr->low > v) scalePtr++;

    (void) sprintf(outString, "%.3g%s", v / scalePtr->factor, scalePtr->prefix);

    return(1);
}

/*ARGSUSED*/
static int netNameFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;			/* not used */
octObject *instPtr, *masterPtr;
{
    octObject terminal;
    octObject prop1, prop2;
    octObject net;
    char *strcpy();
    int getNetOfTerm();

    prop1.type = prop2.type = OCT_PROP;
    prop1.contents.prop.name = "GLOBALNET";
    prop2.contents.prop.name = "LOCALNET";
    if (octGenFirstContent(masterPtr, OCT_TERM_MASK, &terminal) == OCT_OK &&
		(octGetByName(&terminal, &prop1) == OCT_OK ||
		 octGetByName(&terminal, &prop2) == OCT_OK) &&
		octGetByName(instPtr, &terminal) &&
		getNetOfTerm(&terminal, &net) == OCT_OK &&
		net.contents.net.name != NIL(char) &&
		strlen(net.contents.net.name) > 0) {
	(void) strcpy(outString + strlen(outString), net.contents.net.name);
	return(1);
    } else {
	return(0);
    }
}

static int getNetOfTerm(termPtr, netPtr)
octObject *termPtr;
octObject *netPtr;
{
    octStatus stat;
    octObject path;

    stat = octGenFirstContainer(termPtr, OCT_NET_MASK, netPtr);
    OH_ASSERT(stat);
    if (stat == OCT_OK) return(OCT_OK);

    stat = octGenFirstContainer(termPtr, OCT_PATH_MASK, &path);
    OH_ASSERT(stat);
    if (stat != OCT_OK) return(OCT_NOT_FOUND);

    stat = octGenFirstContainer(&path, OCT_NET_MASK, netPtr);
    OH_ASSERT(stat);
    return(stat == OCT_OK ? OCT_OK : OCT_NOT_FOUND);
}
