#include "copyright.h"
#include "port.h"
#include "utility.h"
#include <X/Xlib.h>
#include "XDM.h"
#include <varargs.h>
#include "rpc.h"
#include "oct.h"
#include "oh.h"
#include "st.h"
#include "IFsim.h"
#include "IFerrmsgs.h"
#include "octsim.h"
#include "rpc-spice.h"

static char *progName = "?rpc-spice?";
static octObject groundObj;
static cktDesc *curCktPtr = NIL(cktDesc);
static analDesc *curAnalPtr = NIL(analDesc);

/*ARGSUSED*/
UserMain(display, spotPtr, cmdList, userOptionWord, array)
char *display;
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
RPCFunction **array;
{
    int nCommand;

    groundObj.objectId = oct_null_id;

    ofeDisplayName = display;
    if (XOpenDisplay(ofeDisplayName) == NIL(Display)) {
	ofeMessage(FATAL, "couldn't open X display\n");
    }

    nCommand = tabulateCommands(array);

    XDM_ASSERT(XDMInit(progName));

    ofeMessage(INFO, "Operational\n");

    return(nCommand);
}

/*ARGSUSED*/
static int quit(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
/*
 * Terminates the application.
 */
{
    RPCExit(0);
}

static int chooseGround(), plot(), quit();
extern int instanceProps();
static int af0(), af1(), af2(), af3(), af4(), af5(), af6(), af7();
static struct dialog ap[8];

static RPCFunction CommandArray[] = {
    { instanceProps,	NIL(char),	"instance-props"	},
    { chooseGround,	NIL(char),	"choose-ground-node"	},
    { plot,		NIL(char),	"plot"			},
    { quit,		NIL(char),	"quit-spice"		},
    { af0,		"ANALYSES",	NIL(char)		},
    { af1,		"ANALYSES",	NIL(char)		},
    { af2,		"ANALYSES",	NIL(char)		},
    { af3,		"ANALYSES",	NIL(char)		},
    { af4,		"ANALYSES",	NIL(char)		},
    { af5,		"ANALYSES",	NIL(char)		},
    { af6,		"ANALYSES",	NIL(char)		},
    { af7,		"ANALYSES",	NIL(char)		},
};

static int tabulateCommands(array)
RPCFunction **array;
{
    RPCFunction *tabPtr;
    int nAnal = 0;
    int i;
    int fillAnalData();

    ofeOutInit();
    ofeInitSimulator();
    progName = ALLOC(char, strlen(ofeSimPtr->simulator) + 10);
    (void) sprintf(progName, "rpc-%s", ofeSimPtr->simulator);

    for (tabPtr = CommandArray; tabPtr->menuString; tabPtr++) {
	if (tabPtr->paneString == NIL(char)) tabPtr->paneString = progName;
    }

    for (i = 0; i < ofeSimPtr->numAnalyses; i++) {
	CLEAR_DIALOG(ap[nAnal]);
	if (fillAnalData(ofeSimPtr->analyses[i], i, &ap[nAnal])) {
	    nAnal++;
	    tabPtr++->menuString = ofeSimPtr->analyses[i]->name;
	}
	if (nAnal >= 8) break;
    }

    *array = CommandArray;
    return(tabPtr - CommandArray);
}

static int fillAnalData(analPtr, index, dlgPtr)
IFanalysis *analPtr;
int index;
struct dialog *dlgPtr;
{
    IFparm *parmPtr;
    int i, nFlags, nValues;
    struct flagStruct *flagArray;
    struct valueStruct *valueArray;

    nFlags = nValues = 0;
    flagArray = NIL(struct flagStruct);
    valueArray = NIL(struct valueStruct);
    for (i = 0; i < analPtr->numParms; i++) {
	parmPtr = &analPtr->analysisParms[i];
	if (parmPtr->dataType & IF_SET) {
	    switch (parmPtr->dataType & IF_VARTYPES) {
		case IF_FLAG:
		    nFlags++;
		    flagArray = REALLOC(struct flagStruct, flagArray, nFlags);
		    CLEAR_FLAG(flagArray[nFlags-1]);
		    flagArray[nFlags-1].name = parmPtr->keyword;
		    flagArray[nFlags-1].parmPtr = parmPtr;
		    break;
		case IF_INTEGER:
		case IF_REAL:
		    nValues++;
		    valueArray = REALLOC(struct valueStruct,valueArray,nValues);
		    CLEAR_VALUE(valueArray[nValues-1]);
		    valueArray[nValues-1].name = parmPtr->keyword;
		    valueArray[nValues-1].parmPtr = parmPtr;
		    break;
		default:
		    FREE(flagArray);
		    FREE(valueArray);
		    return(0);
	    }
	}
    }

    dlgPtr->analIndex = index;
    dlgPtr->analPtr = analPtr;
    dlgPtr->dialogWin = 0;
    dlgPtr->nFlags = nFlags;
    dlgPtr->nCol1Flags = nFlags;
    dlgPtr->flagArray = flagArray;
    dlgPtr->nValues = nValues;
    dlgPtr->valueArray = valueArray;

    return(1);
}

#define ANAL_FUNC(FN, INDEX)	\
    /*ARGSUSED*/							\
    static int FN(spotPtr, cmdList, userOptionWord)			\
    RPCSpot *spotPtr;							\
    lsList cmdList;							\
    long userOptionWord;						\
    {									\
	int ofeAnalysis();						\
									\
	return(ofeAnalysis(spotPtr, cmdList, &ap[INDEX]));		\
    }


ANAL_FUNC(af0, 0)
ANAL_FUNC(af1, 1)
ANAL_FUNC(af2, 2)
ANAL_FUNC(af3, 3)
ANAL_FUNC(af4, 4)
ANAL_FUNC(af5, 5)
ANAL_FUNC(af6, 6)
ANAL_FUNC(af7, 7)

/*ARGSUSED*/
static int ofeAnalysis(spotPtr, cmdList, dlgPtr)
RPCSpot *spotPtr;
lsList cmdList;
struct dialog *dlgPtr;
/*
 * Do an analysis
 * Format for the command:  : {analysis-name}
 */
{
    octObject facet;
    char *analName = dlgPtr->analPtr->description;
    IFuid uid;
    void buildAnalysisDialog(), setAnalysisParms();

    if (groundObj.objectId == oct_null_id) {
	ofeMessage(SEVERE,
		"you must specify a ground node before running simulation\n");
	return(RPC_OK);
    }

    if (octGetFacet(&groundObj, &facet) != OCT_OK) {
	ofeMessage(SEVERE, "the ground net no longer exists\n");
	return(RPC_OK);
    }
    if (facet.objectId != spotPtr->facet) {
	ofeMessage(SEVERE, "the ground net is defined in another facet\n");
	return(RPC_OK);
    }

    buildAnalysisDialog(dlgPtr, analName);

    if ( ! ofeDisplayDialog(dlgPtr)) {
	ofeMessage(INFO, "analysis aborted\n");
	return(RPC_OK);
    }

    ofeMessage(INFO, "beginning analysis...\n");

    /* create the circuit */
    if (curCktPtr) ofeFreeCircuit(curCktPtr);
    curCktPtr = ofeCircuit(&facet, &groundObj);

    /* create the task */
    OFE_NEW_UID(&uid, (IFuid) curCktPtr, progName, UID_TASK, NIL(GENERIC));
    curCktPtr->taskPtr = (taskDesc *) uid;
    IF_ASSERT(newTask, (curCktPtr->simEquiv, &curCktPtr->taskPtr->simEquiv, uid));

    /* create the analysis */
    OFE_NEW_UID(&uid, (IFuid) curCktPtr, analName, UID_ANALYSIS, NIL(GENERIC));
    curCktPtr->taskPtr->analPtr = curAnalPtr = (analDesc *) uid;
    IF_ASSERT(newAnalysis, (curCktPtr->simEquiv, dlgPtr->analIndex, uid,
			    &curAnalPtr->simEquiv, curCktPtr->taskPtr->simEquiv));

    /* do the analysis */
    setAnalysisParms(dlgPtr);
    IF_CHECK(doAnalyses, (curCktPtr->simEquiv, RESTART, curCktPtr->taskPtr->simEquiv),
		SEVERE,
		{ofeMessage(INFO, "analysis aborted\n"); return(RPC_OK);} );
    IF_ASSERT(deleteTask, (curCktPtr->simEquiv, curCktPtr->taskPtr->simEquiv));

    ofeMessage(INFO, "analysis completed\n");
    return(RPC_OK);
}

static void buildAnalysisDialog(dlgPtr, title)
struct dialog *dlgPtr;
char *title;
{
    Window dWin;
    int bigColId, titleId;

    if (dlgPtr->dialogWin) return;

    dlgPtr->dialogWin = dWin = XDMDialogCreate(RootWindow);

    bigColId = XDMRowColCreate(dWin, XDM_WINDOW);
    XDM_ASSERT(XDMModify(dWin, bigColId, XDM_SPACE, 5, XDM_END));

    titleId = XDMTextCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, title, XDM_END));

    ofeBuildFlagList(dWin, bigColId, dlgPtr->nFlags, dlgPtr->flagArray);

    ofeBuildValueList(dWin, bigColId, dlgPtr->nValues, dlgPtr->valueArray);

    ofeBuildActionButtons(dWin, bigColId, &dlgPtr->OKId, &dlgPtr->abortId);
}

static void setAnalysisParms(dlgPtr)
struct dialog *dlgPtr;
{
    int i;
    IFvalue value;
    IFparm *parmPtr;

    /* set flags */
    for (i = 0; i < dlgPtr->nFlags; i++) {
	parmPtr = dlgPtr->flagArray[i].parmPtr;
	value.iValue = dlgPtr->flagArray[i].flag;
	if (value.iValue) {
	    IF_ASSERT(setAnalysisParm, (curCktPtr->simEquiv, curAnalPtr->simEquiv,
					parmPtr->id, &value, NIL(IFvalue)));
	}
    }

    /* set values */
    for (i = 0; i < dlgPtr->nValues; i++) {
	parmPtr = dlgPtr->valueArray[i].parmPtr;
	switch (parmPtr->dataType & IF_VARTYPES) {
	    case IF_INTEGER:
		value.iValue = dlgPtr->valueArray[i].value;
		break;
	    case IF_REAL:
		value.rValue = dlgPtr->valueArray[i].value;
		break;
	    default:
		ASSERT(0);
	}
	IF_ASSERT(setAnalysisParm, (curCktPtr->simEquiv, curAnalPtr->simEquiv,
				    parmPtr->id, &value, NIL(IFvalue)));
    }
}

/*ARGSUSED*/
static int chooseGround(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
/*
 * Choose ground node
 * Format for the command:  net-or-term : choose-ground-node
 */
{
    RPCArg *firstArgPtr;
    octObject argBag;
    char *format = "\nformat:  net-or-term : choose-ground-node\n";

    if (lsLength(cmdList) != 1) {
	BAD_FMT(format);
    }

    LS_ASSERT(lsFirstItem(cmdList, (Pointer *) &firstArgPtr, NIL(lsHandle)));
    if (firstArgPtr->argType != VEM_OBJ_ARG || firstArgPtr->length != 1) {
	BAD_FMT(format);
    }

    OH_ASSERT(ohGetById(&argBag, firstArgPtr->argData.objArg.theBag));
    if (octGenFirstContent(&argBag, OCT_TERM_MASK|OCT_NET_MASK, &groundObj)
								!= OCT_OK) {
	BAD_FMT(format);
    }

    return(RPC_OK);
}

/*ARGSUSED*/
static int plot(spotPtr, cmdList, userOptionWord)
RPCSpot *spotPtr;
lsList cmdList;
long userOptionWord;
/*
 * Plot output from a simulation
 * Format for the command:  net : plot
 */
{
    RPCArg *firstArgPtr;
    octObject argBag, net;
    char *val;
    int dependFlags, indepFlags;
    char *format = "\nformat:  net : plot\n";
    int queryPlotFlags();

    if (lsLength(cmdList) != 1) {
	BAD_FMT(format);
    }

    LS_ASSERT(lsFirstItem(cmdList, (Pointer *) &firstArgPtr, NIL(lsHandle)));
    if (firstArgPtr->argType != VEM_OBJ_ARG || firstArgPtr->length != 1) {
	BAD_FMT(format);
    }

    OH_ASSERT(ohGetById(&argBag, firstArgPtr->argData.objArg.theBag));
    if (octGenFirstContent(&argBag, OCT_NET_MASK, &net) != OCT_OK) {
	BAD_FMT(format);
    }

    if (curCktPtr == NIL(cktDesc)) {
	ofeMessage(SEVERE,
		    "you must do an analysis before you can plot anything\n");
	return(RPC_OK);
    }

    if ( ! st_lookup(curAnalPtr->waveTable, (char *) net.objectId, &val)) {
	ofeMessage(SEVERE, "no data available for %s\n", ohFormatName(&net));
	return(RPC_OK);
    }

    if ( ! queryPlotFlags(&indepFlags, &dependFlags)) {
	ofeMessage(INFO, "plot aborted\n");
	return(RPC_OK);
    }

    ofeGraph(curAnalPtr,
	     indepFlags, curAnalPtr->indepWave, NIL(waveStruct),
	     dependFlags, (waveStruct *) val, NIL(waveStruct));

    return(RPC_OK);
}

static int queryPlotFlags(indepPtr, dependPtr)
int *indepPtr;
int *dependPtr;
{
    static struct dialog plotFlags;
    static int initializedP = 0;
    int i;
    void buildPlotDialog();

    if ( ! initializedP) {
	initializedP = 1;
	CLEAR_DIALOG(plotFlags);
	buildPlotDialog(&plotFlags);
    }

    if ( ! ofeDisplayDialog(&plotFlags)) return(0);

    *indepPtr = *dependPtr = 0;
    for (i = 0; i < plotFlags.nFlags; i++) {
	if (plotFlags.flagArray[i].flag) {
	    if (i < plotFlags.nCol1Flags) {
		*dependPtr |= plotFlags.flagArray[i].weight;
	    } else {
		*indepPtr |= plotFlags.flagArray[i].weight;
	    }
	}
    }

    return(1);
}

static void buildPlotDialog(dlgPtr)
struct dialog *dlgPtr;
{
    Window dWin;
    int bigColId, titleId, rowId, smallColId;
    void fillPlotData();

    fillPlotData(dlgPtr);

    dlgPtr->dialogWin = dWin = XDMDialogCreate(RootWindow);

    bigColId = XDMRowColCreate(dWin, XDM_WINDOW);
    XDM_ASSERT(XDMModify(dWin, bigColId, XDM_SPACE, 5, XDM_END));

    titleId = XDMTextCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, "Plot Options", XDM_END));

    rowId = XDMRowColCreate(dWin, bigColId);
    XDM_ASSERT(XDMModify(dWin, rowId, XDM_ISROW, 1, XDM_END));

    /* first column -- dependent variable */
    smallColId = XDMRowColCreate(dWin, rowId);
    titleId = XDMTextCreate(dWin, smallColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, "Y axis", XDM_END));
    ofeBuildFlagList(dWin, smallColId, dlgPtr->nCol1Flags, dlgPtr->flagArray);

    /* second column -- independent variable */
    smallColId = XDMRowColCreate(dWin, rowId);
    titleId = XDMTextCreate(dWin, smallColId);
    XDM_ASSERT(XDMModify(dWin, titleId, XDM_TEXT, "X axis", XDM_END));
    ofeBuildFlagList(dWin, smallColId, dlgPtr->nFlags - dlgPtr->nCol1Flags,
		    &dlgPtr->flagArray[dlgPtr->nCol1Flags]);

    ofeBuildActionButtons(dWin, bigColId, &dlgPtr->OKId, &dlgPtr->abortId);
}

static void fillPlotData(dlgPtr)
struct dialog *dlgPtr;
{
    static struct {
	char *name;
	int initVal;
	int weight;
    } plotOptions[] = {
	{	"Real part",		1,	PFL_REAL	},
	{	"Imaginary part",	0,	PFL_IMAG	},
	{	"Magnitude",		0,	PFL_MAG		},
	{	"dB(magnitude)",	0,	PFL_DB		},
	{	"Phase",		0,	PFL_PHASE	},
	{	"Logarithmic axis",	0,	PFL_LOG		}
    };
    int nOpts = sizeof(plotOptions)/sizeof(plotOptions[0]);
    int i;

    dlgPtr->nValues = 0;
    dlgPtr->valueArray = NIL(struct valueStruct);
    dlgPtr->nFlags = 2 * nOpts;
    dlgPtr->nCol1Flags = nOpts;
    dlgPtr->flagArray = ALLOC(struct flagStruct, 2 * nOpts);

    for (i = 0; i < nOpts; i++) {
	CLEAR_FLAG(dlgPtr->flagArray[i]);
	dlgPtr->flagArray[i].name = plotOptions[i].name;
	dlgPtr->flagArray[i].flag = plotOptions[i].initVal;
	dlgPtr->flagArray[i].weight = plotOptions[i].weight;

	CLEAR_FLAG(dlgPtr->flagArray[i+nOpts]);
	dlgPtr->flagArray[i+nOpts].name = plotOptions[i].name;
	dlgPtr->flagArray[i+nOpts].flag = plotOptions[i].initVal;
	dlgPtr->flagArray[i+nOpts].weight = plotOptions[i].weight;
    }
}

void ofeBuildFlagList(dWin, parentId, nFlags, fPtr)
Window dWin;
int parentId;
int nFlags;
struct flagStruct *fPtr;
{
    int i;
    int textId;

    for (i = 0; i < nFlags; i++) {
	fPtr->fieldId = XDMButtonCreate(dWin, parentId);
	textId = XDMTextCreate(dWin, fPtr->fieldId);
	XDM_ASSERT(XDMModify(dWin, fPtr->fieldId,
			    XDM_SETSIGNAL, 1, XDM_VALUE, fPtr->flag, XDM_END));
	XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, fPtr->name, XDM_END));
	fPtr++;
    }
}

void ofeBuildValueList(dWin, parentId, nValues, vPtr)
Window dWin;
int parentId;
int nValues;
struct valueStruct *vPtr;
{
    char string[100];
    int i;
    int rowId, textId;

    for (i = 0; i < nValues; i++) {
	rowId = XDMRowColCreate(dWin, parentId);
	XDM_ASSERT(XDMModify(dWin, rowId, XDM_ISROW, 1, XDM_END));

	textId = XDMTextCreate(dWin, rowId);
	XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, vPtr->name, XDM_END));

	(void) sprintf(string, "%g", vPtr->value);
	vPtr->fieldId = XDMEdRegCreate(dWin, rowId);
	XDM_ASSERT(XDMModify(dWin, vPtr->fieldId,
			    XDM_ROWS, 1, XDM_COLS, COL_WIDTH,
			    XDM_USER, (XDMPointer) vPtr,
			    XDM_SETSIGNAL, 1, XDM_TEXT, string, XDM_END));
	vPtr++;
    }
}

void ofeBuildActionButtons(dWin, parentId, OKIdPtr, abortIdPtr)
Window dWin;
int parentId;
int *OKIdPtr;
int *abortIdPtr;
{
    int rowId, textId;

    rowId = XDMRowColCreate(dWin, parentId);
    XDM_ASSERT(XDMModify(dWin, rowId, XDM_ISROW, 1, XDM_END));

    *OKIdPtr = XDMButtonCreate(dWin, rowId);
    textId = XDMTextCreate(dWin, *OKIdPtr);
    XDM_ASSERT(XDMModify(dWin, *OKIdPtr, XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, "OK", XDM_END));

    *abortIdPtr = XDMButtonCreate(dWin, rowId);
    textId = XDMTextCreate(dWin, *abortIdPtr);
    XDM_ASSERT(XDMModify(dWin, *abortIdPtr, XDM_SETSIGNAL, 1, XDM_END));
    XDM_ASSERT(XDMModify(dWin, textId, XDM_TEXT, "Abort", XDM_END));
}

int ofeDisplayDialog(dlgPtr)
struct dialog *dlgPtr;
{
    Window dWin = dlgPtr->dialogWin;
    XEvent event;
    Window eventWin;
    int eventId;
    int waiting, retVal, i;
    void updateValue(), updateArray();

    XDM_ASSERT(XDMModify(dWin, dlgPtr->OKId, XDM_VALUE, 0, XDM_END));
    XDM_ASSERT(XDMModify(dWin, dlgPtr->abortId, XDM_VALUE, 0, XDM_END));

    XDM_ASSERT(XDMPost(dWin, 0, 0, (int (*)()) 0, XDM_INTERACT));

    waiting = 1;
    while (XPending() > 0 || waiting) {
	XNextEvent(&event);
	if (XDMFilter(&event, &eventWin, &eventId) < 0 && eventWin == dWin) {
	    switch (XDMTypeQuery(eventWin, eventId)) {
		case XDM_BTN_OBJ:
		    if (eventId == dlgPtr->abortId) {
			XDM_ASSERT(XDMEnd(dWin));
			waiting = 0;
			retVal = 0;
		    } else if (eventId == dlgPtr->OKId) {
			XDM_ASSERT(XDMEnd(dWin));
			waiting = 0;
			retVal = 1;
		    }
		    break;
		case XDM_EDREG_OBJ:
		    for (i = 0; i < dlgPtr->nValues; i++) {
			if (eventId == dlgPtr->valueArray[i].fieldId) {
			    updateValue(dWin, &dlgPtr->valueArray[i]);
			    break;
			}
		    }
		    for (i = 0; i < dlgPtr->nArrays; i++) {
			if (eventId == dlgPtr->arrayArray[i].fieldId) {
			    updateArray(dWin, &dlgPtr->arrayArray[i]);
			    break;
			}
		    }
		    break;
	    }
	}
    }

    for (i = 0; i < dlgPtr->nFlags; i++) {
	XDM_ASSERT(XDMQuery(dWin, dlgPtr->flagArray[i].fieldId,
			    XDM_VALUE, &dlgPtr->flagArray[i].flag, XDM_END));
    }
    for (i = 0; i < dlgPtr->nValues; i++) {
	updateValue(dWin, &dlgPtr->valueArray[i]);
    }
    for (i = 0; i < dlgPtr->nArrays; i++) {
	updateArray(dWin, &dlgPtr->arrayArray[i]);
    }
    return(retVal);
}

static void updateValue(dWin, vPtr)
Window dWin;
struct valueStruct *vPtr;
{
    char *inStr, outStr[100];
    double atof();

    XDM_ASSERT(XDMQuery(dWin, vPtr->fieldId, XDM_TEXT, &inStr, XDM_END));
    vPtr->value = atof(inStr);
    (void) sprintf(outStr, "%g", vPtr->value);
    XDM_ASSERT(XDMModify(dWin, vPtr->fieldId, XDM_TEXT, outStr, XDM_END));
}

static void updateArray(dWin, aPtr)
Window dWin;
struct arrayStruct *aPtr;
{
    char *inStr, outStr[50*MAX_ARRAY];
    int i;
    double atof();

    XDM_ASSERT(XDMQuery(dWin, aPtr->fieldId, XDM_TEXT, &inStr, XDM_END));
    for (i = 0; i < MAX_ARRAY; i++) {
	while (*inStr != '\0' && isspace(*inStr)) inStr++;
	if (*inStr == '\0') break;
	aPtr->values[i] = atof(inStr);
	while (*inStr != '\0' && ! isspace(*inStr)) inStr++;
    }
    aPtr->nVals = i;

    outStr[0] = '\0';
    for (i = 0; i < aPtr->nVals; i++) {
	(void) sprintf(outStr + strlen(outStr), "%g\n", aPtr->values[i]);
    }
    XDM_ASSERT(XDMModify(dWin, aPtr->fieldId, XDM_TEXT, outStr, XDM_END));
}

/*VARARGS2*/
void ofeMessage(type, format, va_alist)
messageType type;
char *format;
va_dcl
{
    char *typeName;
    va_list ap;
    char buffer[1000];

    switch (type) {
	case INFO:
	    typeName = "\0062";
	    break;
	case WARNING:
	    typeName = "\0061Warning: ";
	    break;
	case SEVERE:
	    typeName = "\0061Error: ";
	    break;
	case FATAL:
	    typeName = "\0061Fatal error: ";
	    break;
	default:
	    typeName = "\0061Unknown error code: ";
	    type = FATAL;
	    break;
    }

    va_start(ap);		/*SUPPRESS62*/
    (void) sprintf(buffer, "%s: %s", progName, typeName);
    VEMMSG(buffer);
    (void) vsprintf(buffer, format, ap);
    VEMMSG(buffer);
    VEMMSG("\0060");
    vemPrompt();
    if (type == FATAL) RPCExit(-1);
    va_end(ap);
}
