#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"

void ofeSetInstProps(instPtr)
octObject *instPtr;	/* the instance to play with */
{
    struct dialog *dlgPtr;
    octObject facet;
    struct dialog *getInstDialog();
    void updateInstProps();

    octGetFacet(instPtr, &facet);

    dlgPtr = getInstDialog(instPtr);
    if (dlgPtr && ofeDisplayDialog(dlgPtr)) {
	updateInstProps(dlgPtr, instPtr, &facet);
    }
}

static struct dialog *getInstDialog(instPtr)
octObject *instPtr;
{
    static st_table *dialogTable = NIL(st_table);
    octObject master;
    char *valPtr;
    struct dialog *dlgPtr;
    int buildInstDialog();
    void fillInstDialog();

    if (dialogTable == NIL(st_table)) {
	dialogTable = st_init_table(st_numcmp, st_numhash);
    }

    OH_ASSERT(ohOpenMaster(&master, instPtr, "interface", "r"));
    if (st_lookup(dialogTable, (char *) master.objectId, &valPtr)) {
	dlgPtr = (struct dialog *) valPtr;
    } else {
	dlgPtr = ALLOC(struct dialog, 1);
	if (buildInstDialog(dlgPtr, &master)) {
	    st_add_direct(dialogTable, (char*) master.objectId, (char*) dlgPtr);
	} else {
	    FREE(dlgPtr);
	    return(NIL(struct dialog));
	}
    }

    fillInstDialog(dlgPtr, instPtr);

    return(dlgPtr);
}

static int buildInstDialog(dlgPtr, masterPtr)
struct dialog *dlgPtr;
octObject *masterPtr;
{
    octGenerator gen;
    octObject prop;
    struct scalarStruct *scalarPtr;
    struct arrayStruct *arrayPtr;

    INIT_DIALOG(dlgPtr);

    if (ohGetByPropName(masterPtr, &prop, "CELLTYPE") != OCT_OK ||
		    prop.contents.prop.type != OCT_STRING) {
	ofeMessage(SEVERE, "Can't find `%s' property in master\n",
						prop.contents.prop.name);
	return(0);
    }
    dlgPtr->title = prop.contents.prop.value.string;

    octInitGenContents(masterPtr, OCT_PROP_MASK, &gen);
    while (octGenerate(&gen, &prop) == OCT_OK) {
	switch (prop.contents.prop.type) {
	    case OCT_REAL:
		scalarPtr = daSetLast(struct scalarStruct, dlgPtr->scalars);
		INIT_SCALAR(scalarPtr);
		scalarPtr->name = prop.contents.prop.name;
		scalarPtr->dfltValue = prop.contents.prop.value.real;
		break;
	    case OCT_REAL_ARRAY:
		arrayPtr = daSetLast(struct arrayStruct, dlgPtr->arrays);
		INIT_ARRAY(arrayPtr);
		arrayPtr->name = prop.contents.prop.name;
		break;
	}
    }

    return(1);
}

static void fillInstDialog(dlgPtr, instPtr)
struct dialog *dlgPtr;
octObject *instPtr;
{
    struct scalarStruct *scalarPtr;
    struct arrayStruct *arrayPtr;
    octObject prop;
    int i, j;

    for (i = 0; i < daLen(dlgPtr->scalars); i++) {
	scalarPtr = daGet(struct scalarStruct, dlgPtr->scalars, i);
	if (ohGetByPropName(instPtr, &prop, scalarPtr->name) == OCT_OK &&
				prop.contents.prop.type == OCT_REAL) {
	    scalarPtr->value = prop.contents.prop.value.real;
	} else {
	    scalarPtr->value = scalarPtr->dfltValue;
	}
    }

    for (i = 0; i < daLen(dlgPtr->arrays); i++) {
	arrayPtr = daGet(struct arrayStruct, dlgPtr->arrays, i);
	daReset(arrayPtr->values);
	if (ohGetByPropName(instPtr, &prop, arrayPtr->name) == OCT_OK &&
				prop.contents.prop.type == OCT_REAL_ARRAY) {
	    for (j = 0; j < prop.contents.prop.value.real_array.length; j++) {
		*daSetLast(double, arrayPtr->values) =
				prop.contents.prop.value.real_array.array[j];
	    }
	}
    }
}

/*ARGSUSED*/
static void updateInstProps(dlgPtr, instPtr, facetPtr)
struct dialog *dlgPtr;
octObject *instPtr;
octObject *facetPtr;
{
    int i;
    octObject prop;
    struct scalarStruct *scalarPtr;
    struct arrayStruct *arrayPtr;
    void fixLabels();

    for (i = 0; i < daLen(dlgPtr->scalars); i++) {
	scalarPtr = daGet(struct scalarStruct, dlgPtr->scalars, i);
	if (scalarPtr->value == scalarPtr->dfltValue) {
	    if (ohGetByPropName(instPtr, &prop, scalarPtr->name) == OCT_OK) {
		OH_ASSERT(octDelete(&prop));
	    }
	} else {
	    OH_ASSERT(ohCreateOrModifyPropReal(instPtr, &prop,
					scalarPtr->name, scalarPtr->value));
	}
    }

    for (i = 0; i < daLen(dlgPtr->arrays); i++) {
	arrayPtr = daGet(struct arrayStruct, dlgPtr->arrays, i);
	if (daLen(arrayPtr->values) == 0) {
	    if (ohGetByPropName(instPtr, &prop, arrayPtr->name) == OCT_OK) {
		OH_ASSERT(octDelete(&prop));
	    }
	} else {
	    prop.type = OCT_PROP;
	    prop.contents.prop.type = OCT_REAL_ARRAY;
	    prop.contents.prop.value.real_array.length =
					    daLen(arrayPtr->values);
	    prop.contents.prop.value.real_array.array =
					    daData(double, arrayPtr->values);
	    OH_ASSERT(octCreateOrModify(instPtr, &prop));
	}
    }

#ifdef LABEL_HACK
    fixLabels(instPtr, facetPtr);
#endif /*LABEL_HACK*/
}

#ifdef LABEL_HACK
/* XXX -- from here to the end of the file is a grotesque hack !!! */
void ofeFixLabels(facetPtr)
octObject *facetPtr;
{
    octObject instance;
    octGenerator gen;
    void fixLabels();

    octInitGenContents(facetPtr, OCT_INSTANCE_MASK, &gen);
    while (octGenerate(&gen, &instance) == OCT_OK) {
	fixLabels(&instance, facetPtr);
    }
}

static void fixLabels(instPtr, facetPtr)
octObject *instPtr;
octObject *facetPtr;
{
    octObject master;
    octObject labelLayer;
    octObject newLabel;
    octObject labelBag;
    int buildLabel();


    OH_ASSERT(ohOpenMaster(&master, instPtr, "interface", "r"));

    if (buildLabel(instPtr, &master, &newLabel, &labelLayer)) {
	OH_ASSERT(octGetOrCreate(facetPtr, &labelLayer));
	OH_ASSERT(octCreate(&labelLayer, &newLabel));

	OH_ASSERT(ohGetOrCreateBag(instPtr, &labelBag, LABEL_BAG_NAME));
	ohDeleteContents(&labelBag, OCT_LABEL_MASK);

	OH_ASSERT(octAttach(&labelBag, &newLabel));
    }
}

static int buildLabel(instPtr, masterPtr, labelPtr, layerPtr)
octObject *instPtr;
octObject *masterPtr;
octObject *labelPtr;
octObject *layerPtr;
{
    int isLeaf(), getPlace(), getText();

    if (isLeaf(masterPtr) && getPlace(masterPtr, labelPtr, layerPtr) &&
			    getText(instPtr, masterPtr, labelPtr)) {
	octTransformGeo(labelPtr, &instPtr->contents.instance.transform);
	return(1);
    } else {
	return(0);
    }
}

static int isLeaf(facetPtr)
octObject *facetPtr;
{
    octObject cellClassProp;

    return( ohGetByPropName(facetPtr, &cellClassProp, "CELLCLASS") == OCT_OK &&
	    cellClassProp.contents.prop.type == OCT_STRING &&
	    strcmp(cellClassProp.contents.prop.value.string, "LEAF") == 0 );
}

static int getPlace(facetPtr, labelPtr, layerPtr)
octObject *facetPtr;
octObject *labelPtr;
octObject *layerPtr;
{
    octGenerator gen;

    octInitGenContents(facetPtr, OCT_LABEL_MASK, &gen);
    while (octGenerate(&gen, labelPtr) == OCT_OK) {
	if (labelPtr->contents.label.label == NIL(char) ||
			    labelPtr->contents.label.label[0] == '\0') {
	    OH_ASSERT(octFreeGenerator(&gen));
	    return(octGenFirstContainer(labelPtr, OCT_LAYER_MASK, layerPtr)
								    == OCT_OK);
	}
    }

    return(0);
}

struct nameStruct {
    char *type;
    struct format {
	char *text;
	int (*formatter)();
	char *propName;
    } formatList[10];
};

#define NIL_FUNC	((int (*)()) 0)
static int engrFormat(), capFormat(), floatFormat(), netNameFormat();
struct nameStruct nameTable[] ={
    {	"RESISTOR",
	{   {	"",	engrFormat,		"RESISTANCE"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"CAPACITOR",
	{   {	"",	capFormat,		"CAPACITANCE"	},
	    {	"F",	NIL_FUNC,		""		}   }	},
    {	"INDUCTOR",
	{   {	"",	engrFormat,		"INDUCTANCE"	},
	    {	"H",	NIL_FUNC,		""		}   }	},
    {	"DIODE",
	{   {	"",	floatFormat,		"DIODEAREA"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"BJT",
	{   {	"",	floatFormat,		"BJTAREA"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"MOSFET",
	{   {	"",	floatFormat,		"MOSFETWIDTH"	},
	    {	"/",	floatFormat,		"MOSFETLENGTH"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"CONTACT",
	{   {	"",	netNameFormat,		"CELLTYPE"	},
	    {	"",	NIL_FUNC,		""		}   }	},
    {	"SOURCE",
	{   {	"",	engrFormat,		"SOURCEDCVALUE"	},
	    {	"",	NIL_FUNC,		""		}   }	},
};
#define N_NAMES		(sizeof(nameTable)/sizeof(nameTable[0]))

static int getText(instPtr, facetPtr, labelPtr)
octObject *instPtr;
octObject *facetPtr;
octObject *labelPtr;
{
    octObject prop;
    struct nameStruct *nameEnt;
    static char labelText[1000];
    int i;
    struct nameStruct *getNameEnt();

    if (ohGetByPropName(facetPtr, &prop, "CELLTYPE") == OCT_OK &&
		prop.contents.prop.type == OCT_STRING &&
		(nameEnt = getNameEnt(prop.contents.prop.value.string))) {
	labelText[0] = '\0';
	for (i = 0; ; i++) {
	    (void) strcat(labelText, nameEnt->formatList[i].text);
	    if (nameEnt->formatList[i].formatter == NIL_FUNC) {
		/* done it all successfully */
		labelPtr->contents.label.label = labelText;
		return(1);
	    } else {
		prop.type = OCT_PROP;
		prop.contents.prop.name = nameEnt->formatList[i].propName;
		if (	(octGetByName(instPtr, &prop) == OCT_OK ||
				octGetByName(facetPtr, &prop) == OCT_OK) &&
			(*nameEnt->formatList[i].formatter)(labelText, &prop,
						instPtr, facetPtr)) {
		    /* formatted this part successfully */
		} else {
		    return(0);
		}
	    }
	}
    }
    return(0);
}

static struct nameStruct *getNameEnt(cellType)
char *cellType;
{
    struct nameStruct *namePtr;

    for (namePtr = nameTable; namePtr < &nameTable[N_NAMES]; namePtr++) {
	if (strcmp(cellType, namePtr->type) == 0) return(namePtr);
    }
    return(NIL(struct nameStruct));
}

struct engrScale {
    double low;
    double factor;
    char *prefix;
};
static struct engrScale fullScale[] = {
    {	1e+16,	1,	""	},
    {	1e+12,	1e+12,	"T"	},
    {	1e+9,	1e+9,	"G"	},
    {	1e+6,	1e+6,	"M"	},
    {	1e+3,	1e+3,	"k"	},
    {	1e-1,	1,	""	},
    {	1e-3,	1e-3,	"m"	},
    {	1e-6,	1e-6,	"u"	},
    {	1e-9,	1e-9,	"n"	},
    {	1e-12,	1e-12,	"p"	},
    {	1e-15,	1e-15,	"f"	},
    {	1e-18,	1e-18,	"a"	},
    {	0,	1,	""	},
};
static struct engrScale capScale[] = {
    {	1e-1,	1,	""	},
    {	1e-8,	1e-6,	"u"	},
    {	1e-12,	1e-12,	"p"	},
    {	1e-15,	1e-15,	"f"	},
    {	1e-18,	1e-18,	"a"	},
    {	0,	1,	""	},
};
static struct engrScale noScale[] = {
    {	0,	1,	""	},
};

/*ARGSUSED*/
static int engrFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;
octObject *instPtr, *masterPtr;		/* not used */
{
    int realFormat();

    return(realFormat(outString, propPtr, fullScale));
}

/*ARGSUSED*/
static int capFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;
octObject *instPtr, *masterPtr;		/* not used */
{
    int realFormat();

    return(realFormat(outString, propPtr, capScale));
}

/*ARGSUSED*/
static int floatFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;
octObject *instPtr, *masterPtr;		/* not used */
{
    int realFormat();

    return(realFormat(outString, propPtr, noScale));
}

static int realFormat(outString, propPtr, scalePtr)
char *outString;
octObject *propPtr;
struct engrScale *scalePtr;
{
    double v;

    if (propPtr->contents.prop.type != OCT_REAL) return(0);
    v = propPtr->contents.prop.value.real;

    outString += strlen(outString);

    if (v < 0) {
	v = -v;
	*outString++ = '-';
    }
    while (scalePtr->low > v) scalePtr++;

    (void) sprintf(outString, "%.3g%s", v / scalePtr->factor, scalePtr->prefix);

    return(1);
}

/*ARGSUSED*/
static int netNameFormat(outString, propPtr, instPtr, masterPtr)
char *outString;
octObject *propPtr;			/* not used */
octObject *instPtr, *masterPtr;
{
    octObject terminal;
    octObject prop;
    octObject net;
    int getNetOfTerm();

    if (octGenFirstContent(masterPtr, OCT_TERM_MASK, &terminal) == OCT_OK &&
		(ohGetByPropName(&terminal, &prop, "GLOBALNET") == OCT_OK ||
		 ohGetByPropName(&terminal, &prop, "LOCALNET") == OCT_OK) &&
		octGetByName(instPtr, &terminal) &&
		getNetOfTerm(&terminal, &net) == OCT_OK &&
		net.contents.net.name != NIL(char) &&
		net.contents.net.name[0] != '\0') {
	(void) strcat(outString, net.contents.net.name);
	return(1);
    } else {
	return(0);
    }
}

static int getNetOfTerm(termPtr, netPtr)
octObject *termPtr;
octObject *netPtr;
{
    octObject path;

    if (    (octGenFirstContainer(termPtr, OCT_NET_MASK, netPtr) == OCT_OK) ||
	    (octGenFirstContainer(termPtr, OCT_PATH_MASK, &path) == OCT_OK &&
	     octGenFirstContainer(&path, OCT_NET_MASK, netPtr) == OCT_OK)) {
	return(OCT_OK);
    } else {
	return(OCT_NOT_FOUND);
    }
}
#endif /*LABEL_HACK*/
