#include "copyright.h"
#include "port.h"
#include "utility.h"
#include <varargs.h>
#include "oct.h"
#include "oh.h"
#include "options.h"
#include "octspice.h"

char *progName;

/* this structure contains more info than RPCFunction */
struct commandStruct {
    char *paneName;
    char *cmdName;
    char *args;
    char *summary;
    int (*function)();
};
static dary commandTable;	/* contains `struct commandStruct's */

/*ARGSUSED*/
int UserMain(display, spot, cmdList, uow, menuArray)
char *display;			/* name of the X display		*/
RPCSpot *spot;			/* window, facet and mouse location	*/
lsList cmdList;			/* VEM argument list			*/
long uow;			/* for future use			*/
RPCFunction **menuArray;	/* array of menu entries		*/
{
    dary analDialogs;
    int i;
    void buildCommandTable();

    /* XXX -- handle X display */

    analDialogs = ofeInitSimulations();

    buildCommandTable(analDialogs);
    *menuArray = ALLOC(daLen(commandTable), RPCFunction);
    for (i = 0; i < daLen(commandTable); i++) {
	(*menuArray)[i].function =
			daGet(struct commandStruct, commandTable, i)->function;
	(*menuArray)[i].paneString
			daGet(struct commandStruct, commandTable, i)->paneName;
	(*menuArray)[i].menuString
			daGet(struct commandStruct, commandTable, i)->cmdName;
    }
    return(daLen(commandTable));
}

/*ARGSUSED*/
static int iProps(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    char name[BUFSIZ], dummy[2];
    octObject inst;

    if (sscanf(args, "%s %1s", name, dummy) != 1) return(0);

    if (ohGetByInstName(facetPtr, &inst, name) == OCT_OK) {
	ofeSetInstProps(&inst);
    } else {
	ofeMessage(SEVERE, "can't find instance named `%s'\n", name);
    }

    return(1);
}

/*ARGSUSED*/
static int labels(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    ofeFixLabels(facetPtr);
    return(1);
}

/*ARGSUSED*/
static int wFacet(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    /* convert facet mode to append */
    facetPtr->contents.facet.mode = "a";
    if (octOpenFacet(facetPtr) < OCT_OK) {
	ofeMessage(SEVERE, "couldn't reopen facet for append:  %s\n",
		    octErrorString());
    }
    OH_ASSERT(octCloseFacet(facetPtr));
    return(1);
}

/*ARGSUSED*/
static int ground(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    char name[BUFSIZ], dummy[2];
    octObject netOrTerm;

    if (sscanf(args, "%s %1s", name, dummy) != 1) return(0);

    if (ohGetByNetName(facetPtr, &netOrTerm, name) == OCT_OK ||
		    ohGetByTermName(facetPtr, &netOrTerm, name) == OCT_OK) {
	ofeChooseGround(&netOrTerm);
    } else {
	ofeMessage(SEVERE, "can't find net or terminal named `%s'\n", name);
    }

    return(1);
}

/*ARGSUSED*/
static int plot(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    char *netName;
    octObject tempBag;
    octObject net;
    octGenerator gen;
    char *cellName;
    char *fileName;

    if (netName = strtok(args, WHITE_SPACE)) {
	OH_ASSERT(octFullName(facetPtr, &cellName));
	fileName = ALLOC(char, strlen(cellName) + 20);
	(void) sprintf(fileName, "%s.xgout", cellName);

	OH_ASSERT(ohCreateBag(facetPtr, &tempBag, "### plot bag ###"));
	octMarkTemporary(&tempBag);

	do {
	    if (ohGetByNetName(facetPtr, &net, netName) == OCT_OK) {
		OH_ASSERT(octAttach(&tempBag, &net));
	    } else {
		ofeMessage(SEVERE, "can't find the net named %s\n", netName);
	    }
	} while (netName = strtok(NIL(char), WHITE_SPACE));

	octInitGenContents(&tempBag, OCT_NET_MASK, &gen);
	ofeNodePlot(&gen, fileName);
	FREE(cellName);
	FREE(fileName);

	OH_ASSERT(octDelete(&tempBag));
	return(1);
    } else {
	return(0);
    }
}

/*ARGSUSED*/
static int deck(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    char fileName[BUFSIZ], dummy[2];

    if (sscanf(args, "%s %1s", fileName, dummy) != 1) return(0);

    ofeDeck(facetPtr, fileName);

    return(1);
}

/*ARGSUSED*/
static int analyze(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    ofeAnalysis(facetPtr, dlgPtr);
    return(1);
}

/*ARGSUSED*/
static int quit(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    exit(0);
    /*NOTREACHED*/
}

/*ARGSUSED*/
static int help(facetPtr, args, dlgPtr)
octObject *facetPtr;
char *args;
struct dialog *dlgPtr;
{
    char command[25], dummy[2];
    int i;
    void oneCmdHelp(), badCommand();

    switch (sscanf(args, "%20s %1s", command, dummy)) {
	case -1:
	case 0:
	    for (i = 0; i < daLen(commandTable); i++) {
		oneCmdHelp(daGet(struct commandStruct, commandTable, i), "");
	    }
	    return(1);
	case 1:
	    for (i = 0; i < daLen(commandTable); i++) {
		if (strcmp(daGet(struct commandStruct, commandTable, i)->command,
							    command) == 0) {
		    oneCmdHelp(daGet(struct commandStruct, commandTable, i),"");
		    return(1);
		}
	    }
	    badCommand(command);
	    return(1);
	default:
	    return(0);
    }
}

static void oneCmdHelp(cmdPtr, prefix)
struct commandStruct *cmdPtr;
char *prefix;
{
    char cmdAndArgs[50];

    (void) sprintf(cmdAndArgs, "%.20s %.20s", cmdPtr->command, cmdPtr->args);
    (void) printf("%s%-30s%s\n", prefix, cmdAndArgs, cmdPtr->summary);
}

static void badCommand(name)
char *name;
{
    (void) printf("unknown command `%s' -- type `help' for command list\n",
									name);
}

void buildCommandTable(analDialogs)
dary analDialogs;
{
    static struct commandStruct fixedCommandTable[] = {
	"inst-props",	"inst-name",	"edit instance properties",	iProps,
	"fix-labels",	"",		"clean up all instance labels", labels,
	"write",	"",		"write updated facet",		wFacet,
	"ground",	"net-name",	"identify ground net",		ground,
	"plot",		"net-name ...",	"plot voltages on net(s)",	plot,
	"deck",		"file",		"write a spice deck into file",	deck,
	"quit",		"",		"exit from program",		quit,
	"help",		"[command]",	"print help message",		help,
	"?",		"[command]",	"another name for help",	help,
	NIL(char),	NIL(char),	NIL(char),		(int (*)()) 0
    };
    struct commandStruct *cmdPtr;
    int i;

    /* create the command table */
    commandTable = daAlloc(struct commandStruct, 0);

    /* tabulate the analyses */
    for (i = 0; i < daLen(analDialogs); i++) {
	cmdPtr = daSetLast(struct commandStruct, commandTable);
	cmdPtr->command =
		daGet(struct dialog, analDialogs, i)->simAnalPtr->name;
	cmdPtr->args = "";
	cmdPtr->summary =
		daGet(struct dialog, analDialogs, i)->simAnalPtr->description;
	cmdPtr->function = analyze;
    }

    /* copy the fixed commands */
    for (cmdPtr = fixedCommandTable; cmdPtr->command; cmdPtr++) {
	*daSetLast(struct commandStruct, commandTable) = *cmdPtr;
    }
}

/*VARARGS2*/
void ofeMessage(type, format, va_alist)
messageType type;
char *format;
va_dcl
{
    char *typeName;
    va_list ap;

    switch (type) {
	case INFO:
	    typeName = "";
	    break;
	case WARNING:
	    typeName = "Warning: ";
	    break;
	case SEVERE:
	    typeName = "Error: ";
	    break;
	case FATAL:
	    typeName = "Fatal error: ";
	    break;
	default:
	    typeName = "Unknown error code: ";
	    type = FATAL;
	    break;
    }

    va_start(ap);		/*SUPPRESS62*/
    (void) fprintf(stderr, "%s: %s", progName, typeName);
    (void) vfprintf(stderr, format, ap);
    if (type == FATAL) exit(-1);
    va_end(ap);
}
