#include "copyright.h"
#include "port.h"
#include "utility.h"
#include "oct.h"
#include "oh.h"
#include "octspice.h"
#include "simtrans.h"

static struct termMap resTMap[] = {
    {	"TOP",		"R+"	},
    {	"BOTTOM",	"R-"	},
};
static struct instParms resIParms[] = {
    {	"RESISTANCE",	"resistance",	INST_OVERRIDE	},
};

static struct termMap capTMap[] = {
    {	"TOP",		"C+"	},
    {	"BOTTOM",	"C-"	},
};
static struct instParms capIParms[] = {
    {	"CAPACITANCE",	"capacitance",	INST_OVERRIDE	},
};

static struct termMap indTMap[] = {
    {	"TOP",		"L+"	},
    {	"BOTTOM",	"L-"	},
};
static struct instParms indIParms[] = {
    {	"INDUCTANCE",	"inductance",	INST_OVERRIDE	},
};

static struct termMap dioTMap[] = {
    {	"ANODE",	"D+"	},
    {	"CATHODE",	"D-"	},
};
static struct instParms dioIParms[] = {
    {	"DIODEAREA",	"area",		INST_RATIO	},
};

static struct termMap bjtTMap[] = {
    {	"EMITTER",	"emitter"	},
    {	"BASE",		"base"		},
    {	"COLLECTOR",	"collector"	},
    {	"COLLECTOR",	"substrate"	},
};
static struct instParms bjtIParms[] = {
    {	"BJTAREA",	"area",		INST_RATIO	},
};

static struct termMap mosTMap[] = {
    {	"SOURCE",	"Source"	},
    {	"GATE",		"Gate"		},
    {	"DRAIN",	"Drain"		},
    {	"BULK",		"Bulk"		},
};
static struct instParms mosIParms[] = {
    {	"MOSFETLENGTH",	"l",		INST_OVERRIDE	},
    {	"MOSFETWIDTH",	"w",		INST_OVERRIDE	},
};

static struct termMap vSrcTMap[] = {
    {	"TOP",		"V+"	},
    {	"BOTTOM",	"V-"	},
};
static struct instParms vSrcIParms[] = {
    {	"SOURCEDCVALUE",	"dc",		INST_OVERRIDE	},
    {	"SOURCEACMAGNITUDE",	"acmag",	INST_OVERRIDE	},
    {	"SOURCEACPHASE",	"acphase",	INST_OVERRIDE	},
};

static struct termMap iSrcTMap[] = {
    {	"TOP",		"I+"	},
    {	"BOTTOM",	"I-"	},
};
static struct instParms iSrcIParms[] = {
    {	"SOURCEDCVALUE",	"dc",		INST_OVERRIDE	},
    {	"SOURCEACMAGNITUDE",	"acmag",	INST_OVERRIDE	},
    {	"SOURCEACPHASE",	"acphase",	INST_OVERRIDE	},
};

static struct termMap vcvsTMap[] = {
    {	"TOP",		"V+"	},
    {	"BOTTOM",	"V-"	},
    {	"CTOP",		"VC+"	},
    {	"CBOTTOM",	"VC-"	},
};
static struct instParms vcvsIParms[] = {
    {	"GAIN",		"gain",		INST_OVERRIDE	},
};

static struct termMap vccsTMap[] = {
    {	"TOP",		"V+"	},
    {	"BOTTOM",	"V-"	},
    {	"CTOP",		"VC+"	},
    {	"CBOTTOM",	"VC-"	},
};
static struct instParms vccsIParms[] = {
    {	"GAIN",		"gain",		INST_OVERRIDE	},
};

static struct termMap ccvsTMap[] = {
    {	"TOP",		"H+"	},
    {	"BOTTOM",	"H-"	},
    /* other terminals handled by exception function */
};
static struct instParms ccvsIParms[] = {
    {	"GAIN",		"gain",		INST_OVERRIDE	},
};

static struct termMap cccsTMap[] = {
    {	"TOP",		"F+"	},
    {	"BOTTOM",	"F-"	},
    /* other terminals handled by exception function */
};
static struct instParms cccsIParms[] = {
    {	"GAIN",		"gain",		INST_OVERRIDE	},
};

#define COUNT(array)	(sizeof(array)/sizeof((array)[0]))
#define C_A(array)	COUNT(array), array

static void doISense(), doTranSource();

struct deviceMap ofeTransTable[] = {
    {	"RESISTOR",	0,
	0, 0,			0, 0,
	"Resistor",	0,		C_A(resTMap),	C_A(resIParms)	},
    {	"CAPACITOR",	0,
	0, 0,			0, 0,
	"Capacitor",	0,		C_A(capTMap),	C_A(capIParms)	},
    {	"INDUCTOR",	0,
	0, 0,			0, 0,
	"Inductor",	0,		C_A(indTMap),	C_A(indIParms)	},
    {	"DIODE",	"DIODEMODEL",
	0, 0,			0, 0,
	"Diode",	0,		C_A(dioTMap),	C_A(dioIParms)	},
    {	"BJT",		"BJTMODEL",
	0, 0,			0, 0,
	"BJT",		0,		C_A(bjtTMap),	C_A(bjtIParms)	},
    {	"MOSFET",	"MOSFETMODEL",
	0, 0,			"level", "MOS1",
	"Mos1",		0,		C_A(mosTMap),	C_A(mosIParms)	},
    {	"MOSFET",	"MOSFETMODEL",
	0, 0,			"level", "MOS2",
	"Mos2",		0,		C_A(mosTMap),	C_A(mosIParms)	},
    {	"MOSFET",	"MOSFETMODEL",
	0, 0,			"level", "MOS3",
	"Mos3",		0,		C_A(mosTMap),	C_A(mosIParms)	},
    {	"MOSFET",	"MOSFETMODEL",
	0, 0,			"level", "BSIM",
	"BSIM",		0,		C_A(mosTMap),	C_A(mosIParms)	},
    {	"SOURCE",	0,
	"SOURCETYPE", "VOLTAGE",		0, 0,
	"Vsource",	doTranSource,	C_A(vSrcTMap),	C_A(vSrcIParms)	},
    {	"SOURCE",	0,
	"SOURCETYPE", "CURRENT",		0, 0,
	"Isource",	doTranSource,	C_A(iSrcTMap),	C_A(iSrcIParms)	},
    {	"CONTROLLEDSOUCE",	0,
	"CONTROLLEDSOURCETYPE", "VCVS",		0, 0,
	"VCVS",		0,		C_A(vcvsTMap),	C_A(vcvsIParms)	},
    {	"CONTROLLEDSOUCE",	0,
	"CONTROLLEDSOURCETYPE", "VCCS",		0, 0,
	"VCCS",		0,		C_A(vccsTMap),	C_A(vccsIParms)	},
    {	"CONTROLLEDSOUCE",	0,
	"CONTROLLEDSOURCETYPE", "CCVS",		0, 0,
	"CCVS",		doISense,	C_A(ccvsTMap),	C_A(ccvsIParms)	},
    {	"CONTROLLEDSOUCE",	0,
	"CONTROLLEDSOURCETYPE", "CCCS",		0, 0,
	"CCCS",		doISense,	C_A(cccsTMap),	C_A(cccsIParms)	},
    {	0,		0,
	0, 0,			0, 0,
	0,		0,		0,		0	},
};

#define TIME_NAME	"SOURCEPWLTIMES"
#define VAL_NAME	"SOURCEPWLVALUES"
#define SIM_NAME	"pwl"
#define SIM_DATA_TYPE	IF_REALVEC

static void doTranSource(instPtr, cellPtr, tablePtr, modBagPtr)
octObject *instPtr;
cellDesc *cellPtr;
struct deviceMap *tablePtr;
octObject *modBagPtr;
{
    octObject timeProp, valProp;
    int i, j, length;
    IFparm *parmPtr;
    IFvalue value;

    ofeNormLeafHandler(instPtr, cellPtr, tablePtr, modBagPtr);

    if ((ohGetByPropName(instPtr, &timeProp, TIME_NAME) == OCT_OK ||
	     ohGetByPropName(&cellPtr->master, &timeProp, TIME_NAME)==OCT_OK) &&
	    (ohGetByPropName(instPtr, &valProp, VAL_NAME) == OCT_OK ||
	     ohGetByPropName(&cellPtr->master, &valProp, VAL_NAME) == OCT_OK)) {
	if (timeProp.contents.prop.type != OCT_REAL_ARRAY ||
			    valProp.contents.prop.type != OCT_REAL_ARRAY) {
	    ofeMessage(WARNING, "unrecognizeable PWL specification ignored\n");
	    return;
	}
	length = timeProp.contents.prop.value.real_array.length;
	if (length != valProp.contents.prop.value.real_array.length) {
	    ofeMessage(WARNING,
		    "unequal length PWL time and value vectors: ignored\n");
	    return;
	}
	if (length < 1) return;

	for (i = 0; i < cellPtr->modelPtr->simDevPtr->numInstanceParms; i++) {
	    parmPtr = &cellPtr->modelPtr->simDevPtr->instanceParms[i];
	    if (strcmp(SIM_NAME, parmPtr->keyword) == 0 &&
			(parmPtr->dataType & IF_SET) &&
			((parmPtr->dataType & IF_VARTYPES) == IF_REALVEC)) {
		value.v.numValue = 2 * length;
		value.v.vec.rVec = ALLOC(double, value.v.numValue);
		for (j = 0; j < length; j++) {
		    value.v.vec.rVec[2*j] =
			    timeProp.contents.prop.value.real_array.array[j];
		    value.v.vec.rVec[2*j + 1] =
			    valProp.contents.prop.value.real_array.array[j];
		}
		IF_ASSERT(setInstanceParm, (cellPtr->cktPtr->simEquiv,
					    cellPtr->simEquiv, parmPtr->id,
					    &value, NIL(IFvalue)));
		return;
	    }
	}
	ofeMessage(WARNING, "PWL parameter unknown to simulator\n");
    }
}

static void doISense()
{
    ofeMessage(FATAL, "Current-controlled devices not yet implemented\n");
}
