#include "pdl++.h"
#include <stdio.h>

/* String stuff. */
/* Outline this code to avoid potential compiler inlining bugs. */

String& String::operator=(char *other) { 
  if (mystring != NULL) delete mystring;
  mystring = new char[strlen(other) + 1];
  (void)strcpy(mystring,other);
  return *this;
}

String& String::operator=(String other) {
  if (mystring != NULL) delete mystring;
  mystring = new char[other.length() + 1];
  (void)strcpy(mystring,other.mystring);
  return *this;
}

/* Wire stuff. */

int wirenum = 0;

wire::wire() : myname() {
  char temp[100]; 
  (void)sprintf(temp,"ZZZ%d",wirenum++); /* Can't get streams to do this without adding garbage characters.  Hmmm. */
  if (strlen(temp) >= 100) {
     fprintf(stderr,"panic in pdl: string out of bounds\n");
     exit(-1);
  }
  myname = temp; 
}

wire wire::operator=(wire w2) { 
  cout << "= " << w2 << " " << *this << "\n";
  return *this; 
}

wire inv(wire i1) { 
  wireP o1 = new wire; 
  cout << "! " << i1 << " " << *o1 << "\n";
  return *o1;
}

wire and(wire i1, wire i2) {
  wireP o1 = new wire; 
  cout << "& " << i1 << " " << i2 << " " << *o1 << "\n";
  return *o1;
}

wire and(wire i1, wire i2, wire i3) {
  wireP o1 = new wire; 
  cout << "& " << i1 << " " << i2 << " " << i3 << " " << *o1 << "\n";
  return *o1;
}

wire or(wire i1, wire i2) {
  wireP o1 = new wire; 
  cout << "+ " << i1 << " " << i2 << " " << *o1 << "\n";
  return *o1;
}

wire or(wire i1, wire i2, wire i3) {
  wireP o1 = new wire; 
  cout << "+ " << i1 << " " << i2 << " " << i3 << " " << *o1 << "\n";
  return *o1;
}

wire nand(wire i1, wire i2) {
  wireP o1 = new wire; 
  cout << "$ " << i1 << " " << i2 << " " << *o1 << "\n";
  return *o1;
}

wire nand(wire i1, wire i2, wire i3) {
  wireP o1 = new wire; 
  cout << "$ " << i1 << " " << i2 << " " << i3 << " " << *o1 << "\n";
  return *o1;
}

wire nor(wire i1, wire i2) {
  wireP o1 = new wire; 
  cout << "# " << i1 << " " << i2 << " " << *o1 << "\n";
  return *o1;
}

wire nor(wire i1, wire i2, wire i3) {
  wireP o1 = new wire; 
  cout << "# " << i1 << " " << i2 << " " << i3 << " " << *o1 << "\n";
  return *o1;
}

wire xor(wire i1, wire i2) {
  wireP o1 = new wire; 
  cout << "@ " << i1 << " " << i2 << " " << *o1 << "\n";
  return *o1;
}

wire xor(wire i1, wire i2, wire i3) {
  wireP o1 = new wire; 
  cout << "@ " << i1 << " " << i2 << " " << i3 << " " << *o1 << "\n";
  return *o1;
}

wire operator==(wire i1, wire i2) {
  return (i1 && i2) || !(i1 || i2);
}

wire dlatch(wire i1, wire i2) {
  wireP o1 = new wire; 
  cout << "d " << i1 << " " << i2 << " " << *o1 << "\n";
  return *o1;
}

wire drlatch(wire input_wire, wire clock_wire, wire reset_wire) {
  wireP o1 = new wire; 
  cout << "dr " << input_wire << " " << clock_wire << " " << reset_wire << " " << *o1 << "\n";
  return *o1;
}

wire sum(wire a, wire b, wire carry) {
  wireP z1 = new wire, z2 = new wire, z3 = new wire, 
  zs = new wire, zc = new wire;

  cout << "@ " << a << " " << b << " " << *z1;
  cout << "@ " << *z1 << " " << carry << " " << *zs;
  cout << "& " << *z1 << " " << carry << " " << *z2;
  cout << "& " << a << " " << b << " " << *z3;
  cout << "+ " << *z3 << " " << *z2 << " " << *zc;
  return *zc;
}

wire sumcarry(wire a, wire b, wire carry) {
  return xor(a,b,carry);
}

wire diff(wire a, wire b, wire borrow) {
  /* Returns the difference. */
  return and(!a,!b,!borrow) || and(!a,b,!borrow) ||
    and(a,!b,!borrow) || and(a,b,borrow);
}

wire diffcarry(wire a, wire b, wire borrow) {
  /* Returns the borrow out of the difference. */
  return (!a && (b || borrow)) || and(a,b,borrow);
}

wire incr(wire a, wire cin) {
  /* Given value and carry in, computes incremented value. */
  return a ^ cin;
}

wire incrcarry(wire a, wire cin) {
  /* Given value and carry in, computes carry for incremented value. */
  return a && cin;
}

wire decr(wire a, wire bin) {
  /* Given value and borrow in, computes decremented value. */
  return a || bin;
}

wire decrborrow(wire a, wire bin) {
  /* Given value and borrow in, computes borrow for decremented value. */
  return a && bin;
}

/* Wire arrays. */

wire_array::wire_array(int size) {
  /* An anonymous array. */
  int i;
  nwires = size;
  wireptrs = new wireP[nwires];
  for (i=0; i<nwires; i++)
    wireptrs[i] = new wire;
}

wire_array::wire_array(String name, int size) {
  int i;
  char tempname[257];
  nwires = size;
  wireptrs = new wireP[nwires];
  for (i=0; i<nwires; i++) {
    (void)sprintf(tempname,"%s<%d>",name,i);
    if (strlen(tempname) > 256) {
      fprintf(stderr,"wire_array name too long\n");
      abort();
    }
    wireptrs[i] = new wire(tempname);
  }
}

wire_array::wire_array(char * name, int size) {
  int i;
  char tempname[257];
  nwires = size;
  wireptrs = new wireP[nwires];
  for (i=0; i<nwires; i++) {
    (void)sprintf(tempname,"%s<%d>",name,i);
    if (strlen(tempname) > 256) {
      fprintf(stderr,"wire_array name too long\n");
      abort();
    }
    wireptrs[i] = new wire(tempname);
  }
}

wire_array& wire_array::operator=(wire_array a1) {
  /* Array assignment. */
  int i;
  for (i=0; i<nwires; i++)
    wireptrs[i] = a1.wireptrs[i];
  return *this;
}

/* Input wires. */

input_wire::input_wire(String aname) : (aname) { 
  /* Implement the function here because g++ doesn't like it inline. */
  cout << "in " << aname << "\n"; 
}

/* Input wire arrays. */

input_wire_array::input_wire_array(String aname, int n) : (aname,n) {
  int i;

  for (i=0; i<n; i++)
    cout << "in " << aname << "<" << i << ">\n";
}

input_wire_array::input_wire_array(char * aname, int n) : (aname,n) {
  int i;

  for (i=0; i<n; i++)
    cout << "in " << aname << "<" << i << ">\n";
}

/* Output wires. */

output_wire::output_wire(String aname) : (aname) { 
  /* Implement the function here because g++ doesn't like it inline. */
  cout << "out " << aname << "\n"; 
}

/* Output wire arrays. */

output_wire_array::output_wire_array(String aname, int n) : (aname,n) {
  int i;

  for (i=0; i<n; i++)
    cout << "out " << aname << "<" << i << ">\n";
}

output_wire_array::output_wire_array(char * aname, int n) : (aname,n) {
  int i;

  for (i=0; i<n; i++)
    cout << "out " << aname << "<" << i << ">\n";
}


/* Clock wires. */

clock_wire::clock_wire(String aname) : (aname) { 
  /* Implement the function here because g++ doesn't like it to be inline. */
  cout << "clk " << aname << "\n"; 
}
