/* Classes and functions to build wires and gates. */
/* Some of the wire class member functions are outlined in a separate file.
   If you inline all those functions, some C compilers will blow up
   complaining about too-long expressions.  Sigh. */
#include <stream.h>
#include <string.h>

extern int wirenum;

class String {
  /* Primitive string class---can't depend on a nice one being
     in the local library.  This class is ugly but sufficient. */
  char *mystring;
  
 public:
  int length() { return((mystring == NULL) ? 0 : strlen(mystring)); }

  String(int len) { 
    mystring = new char[len + 1]; 
    mystring[0] = '\0'; 
  }
  String(char *astring) { 
    mystring = new char[strlen(astring) + 1]; 
    (void)strcpy(mystring,astring); 
  }
  String(String& astr) { 
    mystring = new char[astr.length() + 1]; 
    (void)strcpy(mystring,astr.mystring); 
  }
  String() { mystring = NULL; }
  ~String() { if (mystring != NULL) delete mystring; }

  operator char *() { return mystring; }
  friend ostream& operator<<(ostream& os, String as) { os << as.mystring; return os; }

  String& operator=(char *);
  String& operator=(String);

  friend int operator==(String s1, String s2) { return strcmp(s1.mystring,s2.mystring); }
};

class wire;

overload and, or, nand, nor, xor;

class wire {
    String myname;
  public:
    wire();
    wire(char *aname) { myname = aname; }
    wire(String aname) { myname = aname; }
    wire(wire& awire) { myname = awire.myname; }
    ~wire() { }

    friend ostream& operator<<(ostream& os, wire w1) { os << w1.myname; return os; }

    wire operator=(wire);
    friend wire inv(wire);
    friend wire operator!(wire i1) { return inv(i1); }
    friend wire and(wire, wire);
    friend wire and(wire, wire, wire);
    friend wire operator&&(wire i1, wire i2) { return and(i1,i2); }
    friend wire or(wire, wire);
    friend wire or(wire, wire, wire);
    friend wire operator||(wire i1, wire i2) { return or(i1,i2); }
    friend wire nand(wire, wire);
    friend wire nand(wire, wire, wire);
    friend wire nor(wire, wire);
    friend wire nor(wire, wire, wire);
    friend wire xor(wire, wire);
    friend wire xor(wire, wire, wire);
    friend wire operator^(wire i1, wire i2) { return xor(i1,i2); }
    friend wire operator==(wire, wire);
    friend wire dlatch(wire /* input */, wire /* clock */);
    friend wire drlatch(wire /* input */, wire /* clock */, wire /* reset */);
    friend wire sum(wire, wire, wire);
    friend wire sumcarry(wire, wire, wire);
    friend wire diff(wire, wire, wire);
    friend wire diffcarry(wire, wire, wire);
    friend wire incr(wire, wire);
    friend wire incrcarry(wire, wire);
    friend wire decr(wire, wire);
    friend wire decrcarry(wire, wire);
};
typedef wire *wireP;

class wire_array {
  /* dynamically allocated array of wire pointers */
  wire **wireptrs; 
  int nwires; /* size of array */
 public:
  wire_array(int);
  wire_array(String, int);
  wire_array(char *, int);
  ~wire_array() { delete wireptrs; }

  wire operator[](int i) { return *wireptrs[i]; }
  wire_array& operator=(wire_array);
};
 
class input_wire : public wire {
 public:
  /* can't make an unnamed input */
  input_wire(char *aname) : (aname) { cout << "in " << aname << "\n"; }
  input_wire(String);
  ~input_wire() { }

  input_wire operator=(wire w1) { ((wire *)this)->operator=(w1); return *this; }
};
typedef input_wire *input_wireP;

class output_wire_array;
class input_wire_array : public wire_array {
 public:
  input_wire_array(String, int);
  input_wire_array(char *, int);
  ~input_wire_array() { }

  input_wire_array& operator=(wire_array a1)  {
     wire_array::operator=(a1);
     return *this;
  }
};

class output_wire : public wire {
 public:
  /* can't make an unnamed output */
  output_wire(char *aname) : (aname) { cout << "out " << aname << "\n"; }
  output_wire(String);
  ~output_wire() { }

  output_wire operator=(wire w1) { ((wire *)this)->operator=(w1); return *this; }
};
typedef output_wire *output_wireP;

class output_wire_array : public wire_array {
 public:
  output_wire_array(String, int);
  output_wire_array(char *, int);
  ~output_wire_array() { }

  output_wire_array& operator=(wire_array a1)  {
     wire_array::operator=(a1);
     return *this;
  }
};

class clock_wire : public wire {
 public:
  /* can't make an unnamed clock */
  clock_wire(char *aname) : (aname) { cout << "clk " << aname << "\n"; }
  clock_wire(String);
  ~clock_wire() { }

  clock_wire operator=(wire w1) { ((wire *)this)->operator=(w1); return *this; }
};
typedef clock_wire *clock_wireP;

class clock_wire_array : public wire_array {
 public:
  clock_wire_array(String, int);
  clock_wire_array(char *, int);
  ~clock_wire_array() { }

  clock_wire_array& operator=(wire_array a1)  {
     wire_array::operator=(a1);
     return *this;
  }
};

