#!/bin/sh
#
# This program generates a layout in an oct facet from
# a pif description.
#

# some constants
# this variable set by sed operation during installation
CADROOT=__CADROOT__
lib=$CADROOT"/lib/pdl++"
layoutfacet=wolfe
mis_script=$lib/syn.script
PATH="$CADROOT/lib/pdl++:"$PATH

# default values
# use TimberWolf V 4 by default
wflags=""
ifile=""

badargs=0
# get command line arguments
while `test \( ! -z "$*" \)`
do
	arg=$1
	case $arg in
	# wolfe arguments
	-wolfeflags)
	shift
	wflags=$1
	;;
	# misII script
	-script) 
	shift
	mis_script=$1
	;;
	# bad argument
	-*)
	echo "unknown argument $1"
	badargs=1
	;;
	# must be file name
	*) 
	ifile=`basename $1 .pif`
	;;
	esac
	shift
done
# was the argument list ok?
if [ $badargs -ne 0 ]
then
	exit 1
fi

trap "rm -f ${ifile}.tpif ${ifile}.bdn ${ifile}.tmp ${ifile}.bl ${ifile}.tmpblif ${ifile}.pre" 0 1 2 3 4 8 9 10  

# does input file exist?
if [ ! -r ${ifile}.pif ]; then
	echo "File ${ifile}.pif does not exist"
	exit 1
fi

# clean up old oct database
rm -rf ${ifile}

# Is this design strictly combinational logic?
n_latches=`egrep '^d' ${ifile}.pif | wc -l`

$lib/preprocess++ ${ifile}.tpif ${ifile}.pre < ${ifile}.pif
rm ${ifile}.pre
echo "Preprocess Stage Successful ..."
$lib/pif++oct ${ifile}.bl ${ifile}.bdn ${ifile}.tmp ${ifile}.tmpblif < ${ifile}.tpif
if [ $? -ne 0 ]
then
	echo "oops---pif error"
	exit 1
fi
cat ${ifile}.tmp ${ifile}.bdn > ${ifile}.bdnet
cat ${ifile}.tmpblif ${ifile}.bl > ${ifile}.blif

# do the logic optimization and layout
echo  "Optimizing . . ."
if [ ! -r ${ifile}.blif ]; then
	echo "The file ${ifile}.blif does not exist"
	exit 1
fi
misII -f $mis_script -T oct -o ${ifile}:logic ${ifile}.blif
octprop -pTECHNOLOGY=scmos ${ifile}:logic:interface

if test $n_latches -gt 0
then
	echo "Starting BDNET . . ."
	if [ ! -r ${ifile}.bdnet ]; then
		echo "The file ${ifile}.bdnet does not exist"
		exit 1
	fi
	bdnet ${ifile}.bdnet
	echo "Flattening Hierarchy . . ."
	octflatten -o:unplaced ${ifile}:memory
else
	# just flatten it
	octflatten -o:unplaced ${ifile}:logic
fi

octprop -pTECHNOLOGY=scmos ${ifile}:unplaced:interface
if [ -r ${ifile}.pins ]
then
   # Add add cell name to pins file, then generate and execute
   # octprop commands
   (echo CELLname $ifile) | cat - ${ifile}.pins | awk -f $lib/padinfo.a | sh
fi
wolfe $wflags -o:$layoutfacet ${ifile}:unplaced
# cleanup
rm -rf ${ifile}/unplaced ${ifile}/logic ${ifile}/memory
